/*

Copyright (c) 2015-2017, cheminfo

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of {{ project }} nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

package java.awt.geom;

/**
 *  * JDK Class Emulation for GWT
 */

import java.awt.Shape;

public abstract class RectangularShape implements Shape {
  protected RectangularShape() {
  }

  public abstract double getX();

  public abstract double getY();

  public abstract double getWidth();

  public abstract double getHeight();

  public double getMinX() {
    return getX();
  }

  public double getMinY() {
    return getY();
  }

  public double getMaxX() {
    return getX() + getWidth();
  }

  public double getMaxY() {
    return getY() + getHeight();
  }

  public double getCenterX() {
    return getX() + getWidth() / 2.0;
  }

  public double getCenterY() {
    return getY() + getHeight() / 2.0;
  }

  public abstract boolean isEmpty();

  public abstract void setFrame(double x, double y, double w, double h);

  public void setFrameFromDiagonal(double x1, double y1, double x2, double y2) {
    if (x2 < x1) {
      double t = x1;
      x1 = x2;
      x2 = t;
    }
    if (y2 < y1) {
      double t = y1;
      y1 = y2;
      y2 = t;
    }
    setFrame(x1, y1, x2 - x1, y2 - y1);
  }

  public boolean contains(Rectangle2D r) {
    return contains(r.getX(), r.getY(), r.getWidth(), r.getHeight());
  }

}
