/*

Copyright (c) 2015-2016, cheminfo

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of {{ project }} nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

package com.actelion.research.chem.prediction;

import com.actelion.research.calc.ThreadMaster;
import com.actelion.research.chem.IDCodeParser;
import com.actelion.research.chem.SSSearcher;
import com.actelion.research.chem.StereoMolecule;

public class DruglikenessPredictor {

	private static String druglikenessNoIndex = "sNy@LDeVjj@XTKU|TH\t5.89\nKA|P@QvRRJjjj@LFaLJnyC@\t5.86\nHiFH@H{IIEUJjj@FGPfES]rA@\t5.82\nJoB@@BUJssoPvxPTA@@@FFpaUpriv\u007FDsP\t5.61\nHeTH@@RV[TYZ`@@AaUMpssHP`\t5.19\nHmtH@@RVYWeVhH@@FCESCJ|rDx\t5.18\nj`qA@@IIULsj\u007FiuoUP@`@@@CAUF\\DkQkN}qFB@\t5.1\nsOx@@eJymUT@pLVOLJp\t4.95\nj`a`@@@YIEEDthdg^jjj`@@@XV`cAJLxKQkN}rD`\t4.89\nHk]@@@yJeWmZ{Sjj`@`@XSBEPrngqLx\t4.83\nHkmL`LfDD@cIIBhhd]MZ`X@@FEXwKoDdp\t4.7\nHefD@Hz\\ddUTuUUP@pFDpj[ayD`\t4.7\nHaF@@@Ri_ijjj@FAPaUsL[`\t4.63\nHeV@@@RUYTYj`@@AatIS\\L|PL@\t4.6\nHif@@@RiUzzjjh@XMBXUMqND\t4.55\nHcuHA@E`dYrRQVYRUUUT@LEaLJfzUt\t4.49\nHcLD@@QIeVuWVj`@@A`q\\LkoDq`\t4.49\nsGY@LDeVj`FABuxhP\t4.38\nHcLL@@RlbbTLRRzuT@@@LFkae]xfD\t4.37\nHcNL`LaA@HrRPjIKKju@`@@pkFzUxeZ\t4.36\nHid@p@kFkAkDfUfn`BH@FF@p|qLx\t4.35\nJoBD@@IS\\rjzv[uP@@@@@X[EMprng[|a@\t4.33\nHg|H@@RYfUWd}nh@@@@@X[ATwCJz]|`@\t4.33\nHeT@p@kNkIkDfY[XXBH`@XXGSsLhP\t4.33\nJoBD@@IKLrjzv[mP@@@@@XKS\\Lkiv\u007FDa@\t4.3\nHmtHpIBlzlVlRYeeXXBHX@FFAr|sJD\t4.28\nHie@@@yJeWkjjj`A`tIaTwDxP\t4.28\nHeU@@@yJeWZzjjj@FCPfES\\PY@\t4.18\nHidH@@RYm^Fjjh@X\\BES\\ODc@\t4.14\nHeV@@@RV[TYz`@@AaUMpsqDH\t4.14\nsNplJyDJHtQddeeUPCBqXwdX\t4.1\nHg|@`@bDfUueZX{`@h@@@FB`fETkiwrN@\t4.05\nJoB`@@]ddRRJTRU][uUU@A@@XKBXUKiv\u007FDXP\t4.04\nJoBD@DIU\\vjzv[uPTEDP@XGFTwCJz]orB@\t4.04\nHg|H@@RYfUWd}nhJBbH@XFFES\\LkiwrH@\t4.04\nHg}@@@yJeWe^nNzjhHB@AaLHUCNg_DqP\t4.04\nHk^@@MRjY]\\]TzhJBH@FDieMpri|Pm@\t4.04\nHcn@@@Re]eRi]jj@B@AaLIa\\LkoDB`\t4.04\nHmt@`@dDfUuZZ@Bj@A`rXUCODyP\t3.99\nHeT@@DjUghP`h`@X]BESCODs`\t3.99\nsNuhLxj@cIHUEi`FDwDF\t3.99\nHmuL@FFnWIIEEJQfjYi`A`cCqLH\t3.98\nj`q`@@@IJsLjoiuoUT@@@@@CBtDXYpTeZMYwnP@\t3.94\nHklHpEbPRPrPrRRqIYHYj`@j@A`UMp|SS@\t3.94\nHo~@@HrRQVKJIJwjjj@@@FApfES\\ng[|`P\t3.94\nHkn@@@rQQEJIKSjj`@@A`lIaTLki|PF@\t3.94\nHeVH@DAImeQej@@@FETwCODS@\t3.94\nsNx@IddbmUPCBqXwdX\t3.89\nHmtD@@QIeyVUZh@@@XTwCJ|SB@\t3.87\nHeW@@@zTjU_kjjjh@X]BXULODzP\t3.83\nHcLD@H[IIDcLdnuT`@@CAlKae^Hf`\t3.83\nHg}@`AlHaIe]YVfNx@Jjh`AaHIaUJz_Dzp\t3.82\nHg~H@CyJieU~SvzhF`b@AaNESCJ{_DPp\t3.82\nHmtL@@Pdf{eYUj`@@A`qMproDS@\t3.79\nsNy@BDivjj@XFZodP\t3.79\nsGY@LDeZj`FAbMy@\t3.79\nsGX@@eLuT@pHVOEJ\t3.79\nHeT@`@dDfUuih@J`@XTfEP|Si@\t3.78\nJoB@@BUJssoPvxPTA@@@FApfEWCJg[|cP\t3.76\nHmuD`LVD@HrRRqIXYV`@`@XUS\\J|c@\t3.75\nHg}L`FM]n@cIIDhTheSCMUUSSP@pzDpjG^PX\t3.74\nsGYhMEPDYIBei@XS\\PH\t3.7\nj`q@A@@RGiAIe]fuzLlz@Bh@H@@XRQ`eNJtZwHR@\t3.69\nj`q@@@DjUggUhJw``hBB@@A`ZBBTxJRcV]{bLZ@\t3.69\nHg~@@@rRIIEQIUqwUUP@P@LEaLJiWSobMp\t3.69\nHk^@@@Ri_YVftzjh@H@FDpaTLki|SF@\t3.69\nHg~@@MRjY]_GUNjB`b`@XRfTwCJgqBl\t3.69\nHkn@@@rRJGJIKSjj`@@A`lIaTLki|a`\t3.69\nHmvH@DAIn[VUZh@@@XLc\\LkqLh\t3.69\nHaFH@NAIYUnZjh@XEaTwLaN@\t3.69\nqCaHK@Hg`dsPFDGME`\t3.69\nKAz@@ArUJmUT@XMBXUMrN@\t3.68\nsJY@LDeZhAahcqC@\t3.68\nHklD`HXxcIIDcLeevjd@H@FGXWCJgrD@\t3.66\nHmtH`NBHRYWVih@Jh@FG@fETkrA@\t3.66\nHiFH@BAJ]ejjh@XMFTwCrD@\t3.66\nKA|P@XBRlmUT@XMBES]r@@\t3.66\nsGT@ATiVj`FAbqxaP\t3.66\nHk]D@DHIRYge\\ftvj`PH@FE\\Jz_Dp`\t3.62\nHid@p@kFkAkDfUfn`BH@FA@cCqLx\t3.62\nsNx@@eRmUPCBqXwdX\t3.62\nsGY@DDfuj`FBMxe`\t3.6\nsGX`JBdruT@pHbwDZ\t3.59\nHknH@JCIDdhhd]Nj`h@@FDqc\\Lki|a@\t3.56\nHmv@pBBlFlVlRYfyZXHJH@FF@J|pnd\t3.56\nHeUL@DpFEIeYzyjfZ@FBIgDzP\t3.56\nJoCB@E|NRRJJIIGKYoU@uAD@AaNESRnf\u007FDFp\t3.55\nHo~D@Hy|ddUTtRVNzjj@B@AalIaTwRnorC@\t3.55\nJoB`P@vDPdrnlktqw@AUUQ@AaHIaUJz_Dyp\t3.55\nHkmL`LZDh@cIHULhdmMZZjj@F@xwCJz_H@\t3.55\nHmvDPLa@BNbLddlTVeUhH@@FAXwCqIT\t3.55\nHknH@JCIDdhhd]Nj`h@@F@qc\\LkoDPp\t3.55\nHcLH@@rJJIPiKkUP@@@p{FxYW^He@\t3.55\nKAzY@XtIPAFRPjYZZ@LDqnHG@\t3.55\nKAxH@@bSKKUT@XXUMqLx\t3.55\nsJQdEbOBAD^u@LINYX`\t3.55\nJgC@PG@DISLroutgmt@P@D@@XBALLki|Sk@\t3.54\nHk^L`FWS`HrRQJEJZfFZjjfZ@FCPfEV|`P\t3.52\nJoc@PG@DISLrj}]Mz}@D@PP@A`HDprnwqC\\\t3.51\nHcNHpBtJpZqZqIf[eZXHJI@Aa`BoLKi@\t3.51\nHcMDPLvD@HzHrRRqQ[RjtDA@@pKFx^IN`\t3.48\nsOt@@tiWMjj`FEbqoDF\t3.48\nHaE@@@YJe~fjjh@XUBXU\\c`\t3.47\nHk]D`FuS`HrRQJEIJfFZjjfZ@FCPfEP|Sm@\t3.47\nHmMH`FeXBLddRJRinFjjih@XUFEW\\QI@\t3.47\nHeV@pBBlzlflRYgea``b@Aa`]OLpa@\t3.44\nHid@`@bDfUvf`@h@FE@fEWDxP\t3.44\nsNy@BDiujj@XFZobQ@\t3.43\nsGY@LDeVj`FAbMxa`\t3.43\nKAxD@@c\\dvlts@FAIa\\\t3.42\njdyA`@@HbdsrnlkbexKT@E@A@@CBVJLxHug^xaTP\t3.4\njdyA@@IIULsZny`ncU@C@D@@@pMQgAJtZsoAcrA@\t3.4\nj`aAb@HLyzHrRQJFYIHf]USAAD@@pE``iJt[wHV@\t3.4\nj`qa@@D@R[fYU_Sk^Zh@@@@@FFdLxJRmFl{wHX@\t3.4\nj`qa@@C\\RjYU_~QmVj`ZBJ`@FBlDhpTeZl{bD^@\t3.4\nj`qA@@@ISKMjoiuoUT@@@@@C@tDLxJRmFl{wHT@\t3.4\nj`q@@@DjUggUhJw``hBB@@AaZBLDipTeFl{wHZ@\t3.4\njhiA@@@ILrsZkf|xKUP@@@@@LCXYpTeZMYw`|``\t3.4\njhi@`@@HRYyWVUEKpVh@J@@@@XRqQgAFl{wDJB\t3.4\nHg]L`LfDD@cIIBhhd]ikTC@P@LFqnW[yD`\t3.4\nJoCD@A|eTrj\u007Fr^wU@tDT@AaNESCJ{_DQp\t3.4\nHg}@@@yJeWe^nNzjhHB@A`lHUCJz]|c@\t3.4\nHcLDpITJsjsZqIfVYXXBHZ@A`P\\n|SA@\t3.4\nHknL`LaA@HrRPjIKY]VhDB@A`vMtj_Hh\t3.4\nHmv@@@RYvUeZj@@@F@pfMproD@`\t3.4\nHeV@@@RiU~njjj`AatIaTp|Si@\t3.4\nHef@@@rRJJIuUU@C@XSBinGdR\t3.4\nj`q@`@HRRjyeU\u007FSk^j`A@@@@FFhTYpRmFl{wHD@\t3.4\nsFt@ATiTvjhA`Xl^HT\t3.4\nHiFD@J@dfWZjj`A`HPfTwCrB@\t3.38\nsNq`@fdvkSHCCKGdh\t3.37\nHcnD`FWXBLddRaRuLLuUUKP@pjDpjxgB\t3.37\nHevH`Ff@cIIDbmS\\MUUM@C@hpjxb\\\t3.37\nHeTD@@QImeQej@@@FETwCODS@\t3.37\nHklL@H[lddRaaRtzjX@@@XMaCJ{qNL\t3.35\nHcNHpBtJpZqZqIfUeZXHJI@A`PDJ|sNd\t3.33\nJoBfPCZio@QddbTJRVuLLuUULsP@X]BXUCoDzp\t3.33\nHk]D`FeS`HrRQHiIZfxZjjfZ@FCQaUt|QM@\t3.33\nHg}D`FeS`HrRQHiIZTwCUUTsT@LFcBkixbV\t3.33\nsOt@@tjWkjj`FEbqoHp\t3.32\nJgC@PG@DISLroutgmt@P@D@@XRALLkiwrN@\t3.31\nJoc@PG@DISLrj}]Mz}@D@PP@AaHDprnv\u007FHL\t3.31\nHeV@@@RYyTYj`@@A`LIeMpsr@@\t3.31\nHeVH@NAIYWVyjjj@FEXULOLaI@\t3.31\nHifH@NAIYW[fjj`A`VES\\rDD\t3.31\nHieH@FxDigwJiej@FFXU\\qND\t3.31\nQO`BNZ`XL|`\t3.31\nsOt@@tjWkjj`FAbMxlp\t3.31\nsJX@@dsU@LMEnHD\t3.3\nHaD@@DjUZzjj`A`pPaUwDq`\t3.28\nHeVD`Ia@BLdabRwBl@D@CBbingd@\t3.26\nHmtD`NTHaIe]Vf`@jP@XTBXU\\PY@\t3.24\nHeTH@@RYe\\YZB@@AaVEWCqFd\t3.24\nHefD@JADeUeZjj@FCXS\\L|b@\t3.24\nKA|H@TBIJjmUPA`VDw\\RJ@\t3.24\nHo|L`H[gbLddRafRvNzjXHB@AavDLz[|aP\t3.24\nHg\\L`H[kbLddRaaRvgUS@A@@p{BFUv~Qp\t3.24\nsGX`JDdjmT@pVOEJ\t3.23\nsJXhMD@bOV`FDwHP\t3.23\njhi``@G\\DeTsJn~iqoUPMPTT@@pu`eFRUhuoAyAp\t3.22\nHkmDPLvD@HzHrRRqQ[JUV``J@AaVMp{qIL\t3.22\nHk^DBIAIeIgeYQe]Zjjjh@XBUMpki|SI@\t3.22\nHcnDBAAAeIVUexYUjjjj@F@xUMtkoD`P\t3.22\nHevHBFAiRYVVfxZjjj@F@pfEWCODA`\t3.22\nHcML@FFnwIIEEDZJuSMS@CCLX^HT`\t3.22\nHcLD@@QIeVuWVjjj`AaqUprn|SF@\t3.22\nHiV@B@BTjYnnXZjjh@X]BXUMwD@P\t3.22\nHmuL@DpFEIeY~nZifh@XXfSqNT\t3.22\nHeTD@@QImeQejjj@FCES\\L|QL@\t3.22\nHmtD@@QIeyVUZjjh@XMS\\LkqLH\t3.22\nHaE@@@yJe~fjjh@XUBES\\SN@\t3.22\nsJX@@eKU@LLH^IX\t3.22\nKA|P@DBSNuUT@X\\DHUMwHp\t3.2\nsNy@LDeVjj@XLKFod`\t3.2\nHeTD@@qI[eQej@@@FCYS\\L|RD@\t3.19\nHid@@DjU^nBBH@FEPaTwLsa@\t3.19\nHidH`NBHRYWZZ@B`@XTBXU\\`P\t3.18\nsGQ`@jdjmR@pfxhP\t3.18\nHid@`@dDfUvf`@h@FAIaUqND\t3.18\nHmuH@DHDf{eYUj`@@A`vMproDP`\t3.17\nHeVD@NADfVuFVh@@@XESCOHH\t3.17\nHaE@@@YJe~fjjh@XEBEWDs`\t3.14\nJoBb@BDByEEEMDdbSV{UTC@P@FGYpr]oqBt\t3.13\nHg}D@DHERYfywIcmjjA@`@XM`siwqBx\t3.13\nHcOLAHePPheNR[e]VyjZfd@XYWCqEt\t3.13\nHie@@@GIHddZzjjh@XMBXU]qFD\t3.13\nsGXhMD@cIHXmPCBGbE@\t3.13\nsOx@@eRimUT@pLQoEf\t3.13\nHevH`Ld@aJY[RZnjjid@XXDIsBxP\t3.13\nsOx@@drm\\@@@prkyab\t3.1\nHiVH`LD@cIEDhRZnjjfPAa`PgLJn@\t3.09\nHg}L`LGSl@cIEDhddXWTuUUTmH@phHPkx`r\t3.08\nHaF@@@RiUkjjj@FEPfEWD[`\t3.08\nHk^L`LWS@HRfVuxnijjjei@FAABGqBD\t3.07\nsNq`@fdskUHCBGbu@\t3.07\nHk]D`LES@HrQQJIKDfkjjjfY@FEABDOLz]@\t3.07\nHg}D`LES@HrQQJIKFDu]UUTsR@LJBDH^Ytv\t3.07\nHmMH`LEPBDieeRZnjjjY@FAABEsNFP\t3.07\nsOx@@drm\\@@@pqky`l\t3.07\nsJX@@eKU@LMEnHD\t3.07\nJoBaPFBivxABTrsjvWTuUUSLr@FCABDM|Rm@\t3.04\nHk]L`LE]L@aJYYwbzfjjjVd@XTDHQ|Ra@\t3.04\nHk]D`LF]@HRfV_xfkjjjYi@FEABDOLkc@\t3.04\nHcmD`LF]@HRfV_rZnjjifPAaPPaCsJyP\t3.04\nHeuH`LFPBDiegIjzjjfPA`PPa\\rnD\t3.04\nHaF@@@RiUkjjj@FCPfEW\\a`\t3.04\nsNt@A\\dbJjj@XFKGbk@\t3.04\nj`qa@@I@RYeU{TRg^jBBJjH@FE`XIQgAJtZso\\a@\t3.03\njdyP@@G^rRQRiQHjIBdFMT@ETuD@CCRLDhqVc^CD\t3.03\nj`q`@@@ITrjwoHsoUAP@@@@C@rLDhJRmFl{wH\\@\t3.03\nj`q@`@H\\RjyY]_Sk^j`ZBbH@FAhDhpTeZMYwnPp\t3.03\nj`qA@@HIU\\rj\u007FiuoUPUAQD@CBtLDhs`eZMYwnPd\t3.03\nj`aP`D@VdXDyIHiTimMAJj`BIj@@XRQ`eFBT{p\t3.03\nHcL@HHbDbLbMbLddjbRtkUP@P@LAaL[ag^PP\t3.03\nHg}L`FM]n@cIIDhTheSCMUUSSP@pZDpjGbZX\t3.03\nHg\\HpEbPRPrPrRRqIYICMT@ET@CBj[aOb[X\t3.03\nJoBb@CCWYIHhiSDdpv}TsP@P@FEKiv\u007FLpa@\t3.03\nHg~L`FWS`HrRQJEJ[TpsUUTsT@LJaLJnHM`\t3.03\nJoCD@B`eLjsj|ioUUAAD@AalXS\\Lz]orD@\t3.03\nHkoDPFF]N`aPcIIEJeeNFjfjU@FEIaUOHT\t3.03\nHk^L`FWS`HrRQJEJZfFZjjfZ@FEPfEWDGP\t3.03\nHk^@@@rQQIFIJftpHH@@@F@ieLLkoDHp\t3.03\nHo~H@EAIYeV^dvhHJJ@A`iTwCNg[|RD@\t3.03\nHknDB@dpgIIEJeMaVj@Hd@FGIaTpkrM@\t3.03\nHg}D@DHERYfywIcmjjA@`@XTLz]|sBx\t3.03\nHc^`@@pjYJYenkae]@bJhh@XDLkoHx\t3.03\nHmvHB@dIrRQRiPUj`B@@X\\fESBoHd\t3.03\nHae@PDp@bOAIfXj[fjj`AaPES]rH@\t3.03\nHcLH`ABPRYW[ZZ@BZ`@XLfETkrM@\t3.03\nHmtH`ABHRYWUih@Jh@FC@fET|Se@\t3.03\nHeV@B@B\\ddTjPsU@@@CCdpj[iy@`\t3.03\nHkl@@LddTjTRFZh@J`@X\\fESKoHL\t3.03\nHcN`@@pjYJYenk`Hbj@A`Prn|c`\t3.03\nHcL@@LddTjTPsU@AP@LNSBiewdR\t3.03\nHeTH@@Rf{TYj`@@A`qTwCODa@\t3.03\nHif@@@RiUzzjjh@X]BXUMwHD\t3.03\nsJPXHlPDQ}T@phbq@\t3.03\nHcnL`Le]@HrQQJEJEuMUURt`C@`aMxaT\t3.02\nHifD@FADfyWaZ@@@FAEWCrD@\t3.02\nHeT@@DjU_k``b`@X]BES]ODpP\t3.01\nsOx@@eR}]UT@plVMyF\t3.01\nsGP`ITjnjPFBqxh`\t3.01\nHie@@@{IHddZzjjh@XMBXUCrF@\t3\nHif@@@RYfVF@b@@XTUMp|`@\t3\nHmv@pIbPRPrPrRRqIPUj`B@@XESBoHd\t2.95\nHg^B@FACmInY]XYZZjid@XXw[sDjp\t2.92\nHknL@FACR[fWTYZZjiPAac]oLRe@\t2.92\nHmvH@JAIUYVUZjjh@X\\fDwCODaP\t2.92\nHmvD@FALbfbbQFVfji@FFMr|qJD\t2.92\nHeVH@JCHeEDcpluUH@prfgfPW@\t2.92\nHcND@JCTefWWaYjje@FFTr|rBd\t2.92\nHet@@DjYUX^dHbH`A`rDwCODr`\t2.92\nHeVH@IAJYW~F``H@FEIaMwDjP\t2.92\nHaF@@@RfYkjYj@FF@S\\sJx\t2.92\nsJPdE`DSpRYZ`FBAxi`\t2.92\nsOt`DPtfWMZi`GfVH\t2.92\nsJXhMD@bOV`FDOI`\t2.92\nHmvH@DAIVYdUZ@b@@XLc\\LkrH@\t2.91\nsNx@AddQeUPCBqXwdX\t2.91\nHaF@@@RfUi`HH@FAIeMqBx\t2.9\nsGX@@eLuT@phbqyJ\t2.89\nsNt@@|dbJjj@XVKF|c@\t2.88\nHaF@`N@HRYWih@H@FEIaUwH@\t2.88\nHid@@DjU^nBBH@FCPaTw\\SA@\t2.88\nqCh@BIWLAaEqS@\t2.84\nsGX@HeTuT@pLVOIP\t2.84\nHie@@@{IHddZzjjh@XUBXU\\Qa@\t2.83\nJoCbPCRzg@QddbQRRtuMpuUULuP@XMFEWSsDmp\t2.82\nHcLL`NWPbDfUuZf`@jX@FE@fEWDGP\t2.82\nHcNH@DAIgfYgVhBH@A`r\\LkoHp\t2.82\nKAxX@QSYIIiZih@pSBxd\\\t2.82\nsNq`IVeUuTpCAX|TX\t2.82\nsGQ@@drmT@pROEj\t2.82\nsOx@@eR}]UT@plQkyL\t2.82\nHidH`ABHRYWZZ@B`@XTBXU\\Sa@\t2.81\nHeTH`ABHRYWVf`@j@AaPIaUqNd\t2.81\nHif@@@Rge^Eh@@@XTUMp|RD@\t2.81\nsGX`BDdwMR@pbxcP\t2.81\nqCx@AYIf`OLE`\t2.81\nHaDH`NBHRYWih@H@FE@fEWHx\t2.81\nj`qAB@H^bDjn[U_UXsjh@@@`@AaheF\\EIVcN}rD@\t2.8\nHk]D`LFD@HrRPjJIEatujB`H@FEYWBoLgS@\t2.8\nHknHpEdJsjsZsHhheEEVF@bFh@FEArz_DpP\t2.8\nJoB@P@QBSJ{MkSV|@EP@@@FF`fETkiv\u007FHx\t2.8\nHcld@dsmBPzIZYyIfVuXUvjjYj@FBToDsP\t2.8\nHcmD`LFD@HrRPjIKLJzmPPD@C@lxYyeKh\t2.8\nHg^H`MDHaIe]UZf`@jj`@XRBXURiwrN@\t2.8\nHkmL@FFnOIIEEDlqvjYjf@FAXSCqIT\t2.8\nHg^H@AAJY[Ue[jhHJ@@XJaTwRng_H`\t2.8\nHeVD`La@BLdaTRPrm@@@CBlinGbAH\t2.8\nHcNH@DAInYWWVjB@@AarTwRn|SA@\t2.8\nHklH@@RYeg_SZjB@@A`NTwBngqDd\t2.8\nHmv@@@Rf~UeZj@@@FDaFDwCJ|a@\t2.8\nHeUH@DpDf^UFVjjh@X\\aTwCOHp\t2.8\nHeT@`@bDfUuih@J`@XLBXUCqNd\t2.8\nHeVH@DAImeQej@@@FCIS\\L|c@\t2.8\nHifD@AADfyWaZjj@FAES]sDpP\t2.8\nHaF@`FBHRYVkh@`@FF@e\\sFx\t2.8\nHaF@@@Rie[jjj@FEPfMwDJ`\t2.8\nsGYiKE`D[lddju@LH^R`\t2.8\nsNx@AdbbMUPCBqXwbC@\t2.8\nsJP`@TfVhAaEqX`\t2.79\nsFx@@eJfuU@LCEcqB`\t2.77\nHeWH`LJQ@HrQQEJYULtt@LLABxcR\t2.76\nHieH`LHPBLdTQRrjYi`Aa`HWD[`\t2.76\nsOy@FDiguie`FFqoDf\t2.76\nQMIDBKpRVAaGFP\t2.76\nHaE@@@yJeVnjjh@XUBXU\\Qn@\t2.75\nsGP`@dfUj`FBuyB\t2.74\nHo~LpBNwBlflvlRYfVVVVBBhfP@XTASm|PU@\t2.73\nHknDpBtpkAkEkDfYnVZXHJI`@XD@j_DJP\t2.73\nHeVH@FAIfUqfhH@@FCAAUp|sFd\t2.73\nHidH@@RUe^Ejjh@XMaTwCqHP\t2.73\nsKT@Adi\\Vj@XYX|d@\t2.73\nsKQ@@eKcRpCAX|TP\t2.72\nsFx@@eJfuU@LKEc^P@\t2.72\nHmth`LKed@cIEDUMLjfYf`Aa`HWDyP\t2.71\nHeWH`LKi@HrQQESYULsL@LLABxgB\t2.71\nHaG@`LK`BLdTQVUSLp@ppDKbMp\t2.71\nHaEH@FxDig|jfV`AafEWDS`\t2.71\nHmtH`ABPRYW[ih@Ih@FEIaUOHT\t2.71\nHif@@@rRIIFnjjj@FCPfEW\\Qa@\t2.71\nHeeH@DhDeUeZjj@FGIaMpsrH@\t2.71\nKAzP@IPIJjmUPAaRXS]qHh\t2.71\nsOtHLPDYHhckSM@LLD^Q`\t2.71\nsFtHLPDISNmLpCCAGdh\t2.71\nsKX@@dmcUPC@qX|`@\t2.71\nsJXhMD@cIHUhAaMq@`\t2.7\nHaFH@JAIYUnX@@@XEaMwH`\t2.69\nJoc@@@YHhhheDbNsYoP@PPD@@XJS\\Lkiv\u007FH@\t2.67\nHg^@pCbPRPrPrRRqIPjCMT@ET@CCjXYW[yA`\t2.67\nHo}D@HggrRQUSQIX{jjh@H@FBpfES\\iorM@\t2.67\nHkl@pHdDdLdLddlRTJFZh@J`@XMSCJgrM@\t2.67\nHmtH@@rQQJEJUjh@@@X\\aMproDb`\t2.67\nHmtH@@RY^UeVj@@@FGHS\\LkrL@\t2.67\nHedL@JZ\\ddTZUTuH@pILKnPH\t2.67\nHiDL@Hilddtjjfh@XYa]qHD\t2.67\nHaF@@@Ri_ijjf@FEPfEWHx\t2.67\nKA|@@Qddbljj`CAhSBinQp\t2.67\nHeTH@@RYe\\YjB@@AapXU\\L|RF@\t2.67\nHeULAHdDJISdfyW[fijT@XYWCrI@\t2.64\nHg|@@DjYU_~V]`@@B`@FDiaTwCN|P{@\t2.64\nsJX@@eMU@LBIcrH\t2.62\nJob@@BULj~jprmnDQE@@@@XJfDwCiv\u007FHh\t2.61\nHcMH@LDDeYWWajjjj@FDaBES]J|cP\t2.61\nHeU@@@yJeWZzjjj@FGPfESSrA@\t2.61\nHeT@@DjU^k``b`@X]BES\\ODrP\t2.61\nsOp@DjWkjj`FEbM_DL\t2.61\nHk\\d@dsmBPzIZY{HhheLdVE]jjfZh@XIR|SC@\t2.6\nHeV@pBBlzlflRYgea``b@A`PYt|Pa@\t2.6\nHeT@p@kNkIkDfY[XXBH`@XDF]ODhP\t2.6\nHmtD@@EJYUgehHJ@@X\\fTwBoDJP\t2.6\nsGX@@eMuT@pHfODj\t2.6\nHkLN@HiiWSrRSQHqRjZfX@XMFDrf\t2.59\nHmwLAHePPheNR[e][fijY@FFUp|aP\t2.58\nHeVH@LAI[eQej@@@FCIS\\L|RL@\t2.57\nsOtHBpDIRocRs@LDIsJl\t2.55\nJoB@H@RCrBSJzroSG\\@E@A@@F@iaUJz_Hx\t2.55\nsNq`@jdvsUPCCKWbK@\t2.55\nHedJ@@RaeImmfji`AaRYSSrB@\t2.54\nsGX@@eRuT@ptVxcP\t2.54\njhi@`@JLRjYeu]TzpZijjjj`@XFQ`gARUhugAyCp\t2.53\njhiA@@HIU\\rjkz}XMT@PA@@@LKPIQgARUhuoAyAp\t2.53\nJoBR@RXI`h]DmJ}dTTTTvQSSiuUTsUP@XEW[_H\\\t2.53\nj`qa@@J@Re]YyTeG^jjj`@`@FAhHpQgIZMYwnPD\t2.53\nj`q@`@@^RYWUe^cKN`@j@B@@FB`HpRbmFm{bNB@\t2.53\nj`qA@@HIU\\rj\u007FiuoUPA@@@@C@tBTYpRmFl{wHR@\t2.53\njhi@`@@HR[YfUWMypVf`@@@@@XJxJRmFl{p^HQ`\t2.53\njdya@@J@RVVyeU{A}Eiz@@@@@A`hTeZMYw`qyB@\t2.53\nj`q@`@@HR[fYU_Sk^Zh@@@@@FFlLxJRmFl{wHH@\t2.53\nj`qA@@@IJsLjoiuoLt@@@@@CASNBdkQkN}qDD@\t2.53\njhqa@@C@RYegV^tyjhHJJ@@XFqQgARUjsoAyA@\t2.53\nHg|d@dpSAPzIZU{Hhhhilbtz]UULuP@prnwdV\t2.53\nHg}DPLFD@HnHrRPjJIGLNfmPTAP@LFrnE[xdz\t2.53\nJoCd@DC_YIHhhdd\\mf}USUUT@FBxULLkiorK@\t2.53\nJoB`@@[ddTRRVJWKNuUUUUP@XZfES\\Lj]|RG@\t2.53\nJoC@A@ANRRIGIQIMuoUUL@D@AalIaTrz]orA@\t2.53\nj`q`@@@IJsLjoiuo\\t@@@@@CASNBdkQkN}rB@\t2.53\nHk]DPLFD@HNHrRPjIKYaWUjB@h@FEYpr_DiP\t2.53\nJoBB@EMddbbbQRrv[uSUUUP@XZfEprng[|a`\t2.53\nJoCFAF``{JSJroJng[UUUUT@FBeS\\Lj]oqLt\t2.53\nHk\\d@dpSAPzIZUyIfYoVgSjjifh@XYW[rC@\t2.53\nHg}@@@wIHhddlUrsmUUUUP@pULJfxYT~IE`\t2.53\nHo~@B@B\\ddTQJbRmzjj`@@A`\\IaTwKiv\u007FHD\t2.53\nHg|`@@pmrQQIPiYTrzAAUUD@LARfxYT~QX\t2.53\nHo|L@@RlbbTVbbQmvjhJ@@AaITLkiv\u007FDs`\t2.53\nHg^L`LaC@HrRPzIKJRju@AT@C@\\InF]OdV\t2.53\nHk\\`@@pmrQQIPiZfWPHJjb@FGIS\\LkrC@\t2.53\nHk]@@@wIHhddl^V]jjjjh@XRfES\\LkqHL\t2.53\nHg~H@DCHhhdeBepg[UU@A@@pdjxYWSodX\t2.53\nHkn@B@B\\dbarbRtzjh@@@XKBXUCJz_DA`\t2.53\nJoBD@@IS\\rjzv[uP@@@@@XKEMprng_DQp\t2.53\nHknL`LaC@HrRPzIKJUVh@J@AavDwCI|cP\t2.53\nHcl`@@pmrQQIPiTrzAAUQ@CCdinFUyF`\t2.53\nHg|H@JRjVWWd}njZjjh@XZfDwCJz]|b@\t2.53\nHg|H@@RYfUWd}nh@@@@@XKATwCJz_DBp\t2.53\nHcm@@@uJfUWyYvjjjj@FDiaTwCJ|RM@\t2.53\nHo~D@IADfUYUgWZjBB`@FDdwBng[|c@\t2.53\nHkmL@FFnOIIEEDRaVjYjV@FEXpsoHH\t2.53\nHeTL`HS@aLddlRPru@@@CC`SJ[ayD`\t2.53\nHcl@@DjYU_egZjjjh@XZBXUMprn|b@\t2.53\nHmvD`La@BLddNbReUj@@@FCXS\\L|aP\t2.53\nHcML@FFnwIIEEDZJuSMS@CCFF^Ia@\t2.53\nHcML@DpFEIeY}kfjYi`AaRYJ{rN@\t2.53\nHcLH@@RVYUYujBB@@X\\ULLkoLaN@\t2.53\nHmvD@EADfVUyUjjj`A`qTwCODqP\t2.53\nHmuL@DpFEIeY~nZifh@XXfKqNd\t2.53\nHeVH@BAJ]eQfjjj@FGQeMpsrD@\t2.53\nHeTH@@RYe\\YjB@@AapPUMp|SI@\t2.53\nHeV@@@RiUvnjjj`AatIaTw\\PY@\t2.53\nHifH@LAIVUxZjj`AatHUMp|`@\t2.53\nHie@@@qJYYhP`h@A`REP|Qn@\t2.53\nHieD@DHNRY[Rijih@XIc\\qBD\t2.53\nHaE@@@qJUwBZZh@XEBEWD[`\t2.53\nHaED@DHNRY[Jfjf@FDYsDI`\t2.53\nKAxX@@gILkmUHA`fEqJX\t2.53\nsOx@@eJm]UT@p\\VM_H@\t2.53\nsJX@@eSU@LDmqP`\t2.53\nsFq@@eLzts@LEkqH`\t2.52\nsJY@DDfvhAaEqX`\t2.52\nqCqPZHAD]XCB[d@\t2.52\nHidH@@RUe^Eh@@@XTUMp|RD@\t2.52\nHeTHPABHfHRYeUn`HJ@AaPXWSqJD\t2.51\nsOx@@eR}]UT@pLQoEf\t2.51\nHknHpBtJpZqZqIfUeffBBbX@FE@Pj_DrP\t2.5\nHifH@DAIVUxVjj`AarXUMp|b@\t2.49\nHifLAHABbTyInU[fijPA`e]rA@\t2.49\nHaFLAHABbTyInUnZfd@XIW\\a`\t2.49\nsFt@AdigUjhAaekrD\t2.46\nqCp@AIZ`LINXK@\t2.46\nHmv@pBBlFlVlRYfyZXHJH@FA@LkqBx\t2.45\nHif@`FBHRYVzz@H`@XDBTwLsa@\t2.45\nsOt@AdigkB@@FFU_Eb\t2.45\nHmtHPABHfHRYeW[hBBh@FEAa]ODiP\t2.45\nJoBbHFSB@DBDYIHUEDdekgZuAHTB@F@xwBng_H\\\t2.44\nHcN@@@rRHiQIKmUP@@@pfDpnFUwdT\t2.44\nsGYDJQDJHR[jf@XXQ\\e@\t2.44\nsGQLKHaQFbLddmU@LCEkrD\t2.44\nsJY@BDfZhAaaEqX`\t2.44\nsJP`@dfvhA`l^Q@\t2.43\nHg^HpBLJrZsZqIfYYye``jI`@XTASm|Se@\t2.41\nqCp@AIZ`LDmqB@\t2.41\nHaF@@@RVU[j@@@FEQaTwDC`\t2.41\nHaD@`@bDfUzZ@B@AaPIaUrN@\t2.41\nHedJ@HiagIIMEYUML@LBcFgdD\t2.39\nHid@@DjU^nBBH@FCPaTp|SN@\t2.39\nJoB`HBvDQLPdsJormuo@PUTi@AaHXWCNf\u007FHh\t2.37\nHo~D@M@\\dTRRbNTyZBBbj@A`ZYS\\Lkiv\u007FHH\t2.37\nHcN@@@rQQHqIKmUP@@@pTHSBxYW^QP\t2.37\nHo~B@D@UMInUeWzV`BJj@AaQUmorD@\t2.37\nHknL@D@UR[eY_iZ@Hj@A`QUi|QI@\t2.37\nHknL@D@]R[e[_iZ@Hj@A`QTi|QN@\t2.37\nHeTH@@RYe\\Yjjj`A`HIaUpsqHX\t2.37\nHmvD@D@TfyV~eh@b@AaaToLSi@\t2.37\nHmvD@DBTfyW^Eh@J@AaaToLQn@\t2.37\nHeU@@@qJYYzDHJH@FEHULODzP\t2.37\nHie@@@qJYWk``b@AaRES]qLD\t2.37\nHaE@@@qJYVnBB@@XDaUwDq`\t2.37\nsNx@@eLmUPCBbKGbk@\t2.37\nsNt`DPtfuZi@XYX|e@\t2.37\nsNt`DQTfuZZ@XQ|Rp\t2.37\nsKT@@Ti\\YZ@XR\\rq@\t2.37\nQMPBcdGcH\t2.37\nHmtD@@QIme\\YZ`@`@XTeMp|SE@\t2.36\nHiFD@FADf]Yjj`A`QUp|c@\t2.36\nKA|H@TBIJkMUPA`VDw\\b@\t2.36\nsGU@DPdvmT@prqyJ\t2.36\nsJY@LDeVdAaEqP`\t2.36\nHeTD@@YIfUqfhH@@FC@U\\L|rLX\t2.36\nHid@@DjU^nBBH@FCPfEP|Qn@\t2.35\nsJT@@TkVhA`enJD\t2.34\nHcML@FFnwIIEEDZJuSMS@C@lXYyA@\t2.33\nHmuL@DpFEIeY~nZifh@XDfRoHx\t2.33\nHaG@@@rdigjXHB@A`fMqDx\t2.33\nsO|@AjeL}XP@@pV_Db\t2.33\nJoCbPCRzg@QddbQRRtuMpuUULuP@X]FEWR\u007FDRp\t2.32\nsNq`@fdskUHCAX|e@\t2.32\nsJX@@eSU@LMEnP`\t2.32\nHmuH@DHDf{eYUj`@@AavDwCJ|a@\t2.3\nsGR@@mTf]jPFFBoEb\t2.3\nsGX@@eLuT@pHbwDZ\t2.3\nqCr@PIKLAy`l\t2.3\nsJX@@eMU@LLL^JH\t2.3\nHg\\B@@z]RVUeWXV@Hjh@FEXRg_H`\t2.29\nHcLL@@yTee[]aX@bh@FAXT{qHX\t2.29\nHeTD@@EIYWVy`@h@FFXT|rJX\t2.29\nsJ\\@@bdjt`LLD^JX\t2.29\nHg^DpBtpkAkEkDfYnVvfBBbZ@A`PBi|Pk@\t2.28\nHklDpEtJpZpzqIfYfVf@bah@FA@NgqNd\t2.28\nsGX@@eLuT@pLVoDB\t2.28\nHo~DpBNpkIkMkDfYeeYe``jIX@FE@Tz\u007FDyP\t2.28\nHeTD@@EJYU^f```@FCIc\\L|Pj@\t2.28\nsKR@AEdi\\Vf@^KD\t2.28\nsJU@DPdvu@LDmrD\t2.28\nHid@@DjUfaBB`@FCPfEP|Qn@\t2.28\nsJY@DDfvhAyeb\t2.28\nsKR@AEdi\\Vf@XS\\a@\t2.27\nqCp@AJZ`LEEqC@\t2.27\nHidL@@pTifvnjjj@FEPaTwLsa@\t2.26\nHaDD@@YJYZnjjh@XEFEWLQn@\t2.26\nsOx@@drm\\@@@pJM_DL\t2.25\nHaDD@@qJYnnjjh@XEBDwLrn@\t2.23\nsGX@AddQjhA`Xl^R`\t2.23\nHidH@@Rfuvz`@`@XXfUsBXP\t2.22\nHkmL`LNDD@cIIBhdmeuZ`PH@FCXwRi|b`\t2.22\nHeVH`IDHaIe]jZ@BX@FE@fEWHd\t2.22\nHmt@@DjU^ZxHHj@A`LIaTpsrE@\t2.22\nHeT@@DjU^k``b`@XCBXULL|c`\t2.22\nHieH@DDDfyWaZ@@@FAES]rD@\t2.22\nsOx@@eRimUT@plVMyF\t2.22\nHiDJ@@PnEInvZjX@XDaTOHp\t2.22\nHcNL`IaM@HrRFIKI\\Jp@T`@phj[ix`J\t2.22\nsNyA@qNRRJuU@LGEcV\t2.22\nsGP@DjVj`FAbMxa`\t2.22\nHcLL@@zTeeY]aX@bh@FAXR{qHD\t2.22\nHeTD@@yIYVvE`BH@FFXT|rFD\t2.22\nHeV@@@Rig\\YjB@@A`rEWCODq`\t2.22\nHeT@P@bIbDfYU[hBB`@XTFEt|Ra@\t2.21\nHeWH@DJPRY[TYZ`@@AaRTwCrB@\t2.21\nHmtD@@QIge\\YZ`@`@XUeMp|aP\t2.21\nsO|@AbeLmXD@@prkyB\t2.21\nsGX`LBdjlt@pexh`\t2.21\nqCr@PISRAaErX\t2.21\nqCqRZHAFzNlAxa`\t2.21\nsJY`hEiNyIej@^P`\t2.21\nHaEH@DxDeeVyjj`A`VES\\RN@\t2.21\nsGQ@HeUuT@ptuxdP\t2.21\nj`qAB@A@bDfUvVuhqwh@Jh@@@AahBLDisQkN}rB`\t2.19\njhiP`@DD@iIf[eW\\cK^Zj`XHh@AaK`iZMYw`|Q``\t2.19\nj`qP`@FBisIIEED[DdpSoUSMP@P@CAfMFl{wLDEH\t2.19\nj`aa`@H\\m{IIEUMDeBYuUU@DP@C@tDXIQgIJM[p\t2.19\nj`qA@@@ISKMjoiuoUT@@@@@CCTDLxJRmFl{bJ^@\t2.19\nj`aA@@OYIHhbeHhyVj``jJ@@XJQ`eFRUhwnP\\\t2.19\nHo\u007FD`Hu~fpcIIEULeDk^jjjZX@XSBXUMpwrC@\t2.19\nHg]LPLfDD@bxcIIBhhd]ikTC@P@LFqnW[yD`\t2.19\nHg_L`L[hR`BLdaTrbRQvmMUUP@pGFFUt{y@@\t2.19\nJoBP@@XVyHhdhTleSKhDEUUD@FDiS\\Lj]|ap\t2.19\nHg~@pNbPRPrPrRRqQISSiwU@@@@@LBZ]obJX\t2.19\nHg~@pJbPRPrPrRRqISICIwU@@@@@LB[SobBh\t2.19\nJoBDP@aDILkmrmM[p@U@@@@XZBXU\\ng[|`P\t2.19\nHk^DB@TpgIIEECTfGijjjih@XSBXUMpsrM@\t2.19\nJoB`PCFDPdrmvnuvg@DUUQ@A`hIe\\Lkm|ap\t2.19\nHg~DB@TpgIIEECThp}MUUUMP@pfDpj[agdV\t2.19\nHo\u007F@@JE|ddTnRbRmzjj`@@AalIaTpkiwrN@\t2.19\nJoBb@BDByEEEMDdbSV{UTC@P@FC\\Lg[|sBt\t2.19\nJoC@@@YIDdThdlUG]US@AP@FFpfESRnf\u007FHx\t2.19\nHknLPLaA@HNHrRPjIKY]VhDB@A`vMtj_Hh\t2.19\nHo|D`JyxcIIEFeDecnjZ`@`@XBfEJz[|a`\t2.19\nHg}H@HP\\ddRJTrUL]uUR@D@CBXpje]N~P`\t2.19\nHg|LA@kTfnRfYgW[iN`jBbP@XTPsi|Q[@\t2.19\nHg}HADH@zIRYeYwIcnjfB@`@X]prng_Hp\t2.19\nHg~@`IBHRYWVUicn@B`@@@XRBXUKiwrN@\t2.19\nHo|H@GrRQQEJQN[j``jH@FBiaTpkiorI@\t2.19\nHcNL`EaLBDrRRqIYCKT@E@@pDDrfxUyF`\t2.19\nJoCD@A|eTrj\u007Fr^wU@tDT@A`nESCJ{[|a@\t2.19\nHk]@`FLHaIe[mZ{S`BJjH@XRBYWCJ{rC@\t2.19\nHg}@`FLHaIe[m^ntx@bjh`AaHIe\\LkoHl\t2.19\nHo}L@EN]_IIEYEEBSnjjfZX@XMBXRgrA@\t2.19\nHk~@@JrRQQIFHUgYNjjjjh@XGBXUMprnf\t2.19\nHo}L@C^moIIEFeEEC^jZjYh@XLfDwCqC\\\t2.19\nHk]@@@YJUueRkSjj`@`@XKBXUCJz_DA`\t2.19\nHmNHB@TIrRQQPuazZjjj`AaLIaTwCOHT\t2.19\nHg]L@ItUoIIEXhiciuUSML@LFaLJ]yE@\t2.19\nHg}@@@UJfUW\u007FegZ`hHj@A`jXUMpsi|bp\t2.19\nHkm@@@kIHbdhdmNjj@@@FBpfDprngrJ@\t2.19\nHmvL`EaLBDrRRqIXYZ`@`@X\\BYS\\OHT\t2.19\nHko@@@q\\dTRbqThP`hj`@XBaTpsi|c`\t2.19\nHk]@@@UJfUW~V]jB`b`@XRfES\\L{rC@\t2.19\nHcND@M@\\dTRRbOKPPTP@LASJ[ae^HV`\t2.19\nHk^@@@Ri_YVftzjh@H@FBpfEPrngrF@\t2.19\nHcm@@@UJfUWyYvhJBH@FDiaTwCN|aP\t2.19\nHigDPLXXP@b`cIHUDnEZfd@XIe\\PN@\t2.19\nHev@B@B\\ddTTKCtuUUT@LAaLJfx^QH\t2.19\nHg\\H@@RYegUvvj```@FDyS\\Jz]|QI@\t2.19\nHknH@FAIfUWaVhHHh@FFaFES\\Lz_HT\t2.19\nHg\\H@NrRQRqZIRmTDAT@CCdpf{SyE`\t2.19\nHcLL@@RlbbTLRRzuUUT@LNinFUwbXP\t2.19\nHkmL@FFnOIIEEDlqvjYjf@FAXpsqBT\t2.19\nHcM@@@gIEDdTiS@PUT@C@TrfxUwdD\t2.19\nHcNH@McIIEXdmJuT@D@CCdpfxYyF`\t2.19\nHcND@E@TfUvzf`@jh@FD`fEWCN|aP\t2.19\nHklH@NrRQRqZJUj``J@AarXS]i|aP\t2.19\nHcO@@@rTie_ZnBBJh@F@hUMpkoHp\t2.19\nHcOH@DKPRYYUYujAB@@XLeMpsrE@\t2.19\nHmtH`ABHRYWUih@Jh@FC@fEW\\RU@\t2.19\nHeW@`FFZBLddTjWCUST`C@dpjyD`\t2.19\nHmtL@@Pdf{eYUjjj`AaqTwCJ|QL@\t2.19\nHmvD@AADfyW^Ejjj`AaQTwSsDqP\t2.19\nHcw@@Hx\\ddRLTeUSU@CBXSBxYWP\t2.19\nHmtD@@EIe]nf`@j`@XBBXU\\J|bP\t2.19\nKAxPPUD`dadataddeRjj`CBFyA@\t2.19\nHeVD@AADfyWxVjjh@XDUMwLSI@\t2.19\nHeVH@BAIUeQfjjj@F@`fTwCOH`\t2.19\nHeT@`@qDfUuih@J`@XLBXUCqNd\t2.19\nHeU@@@yJeWzzjjj@FGPfESCqNd\t2.19\nHmwH@DIPRYYWeVhD@@FEISCOHx\t2.19\nHmvH@DAIe[VUZh@@@X]eMproHP\t2.19\nHie@`FF`cIIEJnFjfd@XDfEWHx\t2.19\nHeVH@FAIfUqfhH@@FGAFEWCqFd\t2.19\nHeTD@@qI[eQejjj@FGXUMpsqHP\t2.19\nHmTL@Fy\\ddUfRjijPAaTIaCrI@\t2.19\nHiT`@@pjRfUj[a@`h`A`gCqDX\t2.19\nHid`@@pjRfUjXBB`@XIp|QF@\t2.19\nHaDH@@RYm[jjj@FC@aTw\\RL@\t2.19\nHaEH@LXDeYVzje`A`REW\\PH@\t2.19\nHieH@BxDivWajjV@FAYSCqHP\t2.19\nHiF@@HrRQVJjj`AatIaTw\\`P\t2.19\nHidH@HRjuVFjjh@XMBYSCqBx\t2.19\nHiDD@F{IIEYjjZ@FAPfErA@\t2.19\nHieD@FFnRjYfFjYh@^EaLH\t2.19\nHaED@DHNRY[Jfjf@GaIaL\t2.19\nKAxH@MvRRJrjf@LBaLKd\\\t2.19\nsGPdE`DSpRYVj@XXF|d@\t2.19\nsNq@JeTuUPCBkFod`\t2.19\nqCbHHAD|DeZ@pPwI@\t2.19\nsFxHB@aJUqiZ@XHSbU@\t2.19\nQMPARVAaWDP\t2.19\nsJX@@eSU@LEEqX`\t2.19\nsOq@@eLm]UT@ptMyef\t2.18\nsOpHADILkW@@@LBEcqU`\t2.18\nqC`P@HRfhCAC\\TP\t2.18\nsFx@@eRfuU@LCD[rX\t2.17\nj`q@`@HRRjyeU\u007FSk^j`B@@@@FFhTYpRmFl{wDDd\t2.17\nqCh@CIKTA`hnHX\t2.16\nsOt@AdiWqZZ`FAbMyF\t2.15\nsGX`LDdsmT@pVOIP\t2.15\nsJX`LDdru@LInP`\t2.15\nHieL@DpFEIeYkfjY`A`bYrN@\t2.15\nsGP`@dfui`FFVOIP\t2.15\nHeVD`La@BLddlRPrm@@@C@j[axfR\t2.15\nsNu@DpdjkUPC@hu|Tp\t2.14\nsGX@@eJuT@pLVODJ\t2.14\nHid@@DjU^nBBH@FGPfEWCrF@\t2.14\nHeT@pHdDdLdLddlRPsU@@@CCJ[nHL`\t2.14\nHkld`LKmePBLdTQTRbrjYffh@XDBD_Ht\t2.13\nHcLB@HZ]rRQJJqCMUUUP@pELJfxUyF`\t2.13\nHcOD`LKiT@cIEDUDeeTsMM@C@`Pmy@`\t2.13\nHmtL@HZ\\ddRblFZjjj@FGIaTwKrA@\t2.13\nHeWH`LKa@HrQQEIYULsT@LBABgdL\t2.13\nHaGDAHK`RISdfyVyjZPAaE\\Pf@\t2.13\nKA|@@@eRkUU@FCPfES\\c`\t2.13\nsGX@AddQjhA`Xm^P`\t2.13\nsJQhHl@bOV`FBq@\t2.13\nHif@@@RfU~F``@@XLfDwCq@h\t2.13\nQMPARZAaWDP\t2.13\nsJX@@eOU@LMEnHD\t2.13\nHieH@DxDee]nZjj@FAXUMqHD\t2.12\nsNxDLHaqBRjuU@LFHcWrX\t2.12\nsNz@@mVdssTpCCAWbs@\t2.12\nqCsAPKR]rSM@^Q@\t2.12\nsGQ@@drmJ@pRwDZ\t2.12\nqCp@AJZ`LHnXS@\t2.12\nHcnLbLe]@HwldTRaRa]SUUTmH@ppHSbEP\t2.11\nHk^LbLWS@HOdiem^KjZjjiZPAa`PgDHP\t2.11\nHcNL@ICNrQQSRjTMUUUP@pfDJfxUwdX\t2.11\nHmvD@ACdiewjZjjj`A`LXUMtkrD@\t2.11\nHif@@@RUe^Fh@@@X]BES\\OH@\t2.11\nqCh@BISLAyhl\t2.11\nHo~LpBtwBlFlVlrJJJZISUi``hfd@FA@JgqBl\t2.1\nHg\\LpEvpkAkCkDfYfYVf@bai@A`PCi|Si@\t2.1\nHg\\h@De]CHeDeMDcimU@PP@LFcFE[yA@\t2.1\nHcL`@@rirQQSRiUpPQU@@pkFx]xdj\t2.1\nHcLh@DxYCHdeEDcJmPDD@CBhqawdH\t2.1\nHeT`@@rirQQSRupPQP@LBqnGdR\t2.1\nsGY@BDeVj`FAbux`P\t2.1\nsNq@@dr{UPCCHwbs@\t2.1\nHaDH@@Rfu[j@@@FFIe\\pfx\t2.1\nQM`AIXFE\\Q@\t2.1\nsJQ@@dju@LEcqQ@\t2.1\nHcLD@@QIeVuWVj`@@AaqMprn|c@\t2.09\nHeTD@@EIe]jZ@Bh@FG@fEWCrI@\t2.09\nHeVH@DAImeQej@@@FCYS\\L|a@\t2.09\nsNx`LDdskUPCAX|UX\t2.09\nHg}LbFM]n@c^rRQJEJITpsUUTtt@LFaLJayA`\t2.09\nHk^LbFWS`HwlddRaRfiafjjif`AaTIaUrA@\t2.09\nQMPBchGfR\t2.09\nsGYALMJRmUPCBbHwdp\t2.08\nRF`EFCqH\t2.08\nHig@@@rdifznBBH@FFXw\\`P\t2.07\nHaG@@@rdifzxHH@AafMwHH\t2.07\nsJT@@TeZhAaaEqX`\t2.07\nHeTH`ABPRYWZf`@f@A`RXU\\bP\t2.07\nqCq@XIIf`LInQ@\t2.07\nHmtD@@QIn[VUZh@@@X\\fTwCOHT\t2.06\nHifH@DAInUxV`@@A`aUpd[a@\t2.06\nsJX`LDdru@LH^S@\t2.06\nHaF@`NBHRYWih@H@FE@fEWHx\t2.06\nHeL`@@JfRiUfnXVfjjjh@XMBYpr\t2.06\nHeU@@@{IHddUWUUUP@pZDpjGbMH\t2.06\nHeTH@@RfV\\YhH`@AaRDpsqJx\t2.05\nsJU@E`djt`LHnJD\t2.05\nsJPH@DISUPCCCGbb@\t2.05\nHifH@DAInUxV`@@Aaa\\OLRf@\t2.04\nHcLL@@PTfye]]ZhH@@F@yS\\LkoHP\t2.04\nHieH@FDDfUfnjjj@FGAATwCqDp\t2.04\nsKXHB@aJWFe`FBDxe`\t2.04\nHmvD@E@TfUvzZ@Bj@AapHU\\L|SE@\t2.03\nHaD@@DjWZXHB@AaTXS]rD@\t2.03\nsJX@@eMU@LMEnHD\t2.03\nHaDH@@RVU[f@@@FBXWBbfx\t2.03\nQMhHchFD|h\t2.02\nHg^``NeSlHaIe][Zf`@ii`@XRBXU]JgrK@\t2.02\nHkl``NeSBHRYWVvf`@ii@AaHIaUtj_Ht\t2.02\nHmw@`NePbDfUujZ@Bf@AapIaUt|aP\t2.02\nHeU@`NdHaIe]ZZ@Bd@FG@fEWSrA@\t2.02\nHidH@@RYm^Fh@@@XMaTwCq@`\t2.02\nsOpHADILkW@@@LJEc^Q`\t2.02\nsOx@@eJqh@P@pLQoHp\t2.01\nHg|@@LdbRbtJUB]aAP@@@CCXSBhYWSod\\\t2.01\nsGY@DDfUj`FDWMcP\t2.01\njdy``@J@BdlsOJkbezKMUPTBH@C@QJLUjsh|q@w`\t1.99\njhiP`@DD@iIf[eW\\cK^Zj`XHh@A`kGARtZsoAy@`\t1.99\njhi``@F@PdwLsvoIW`mMUPPB@@pEQgMFlxOLHex\t1.99\nj`qa@@J@RVYgm\\Tg^YjjB@P@F@bTXkQk^yb@g@\t1.99\nHg]HpBOPkAkEkLbbbfbT[SAAQM@@phAT{xaR\t1.99\nHg}D`LFD@HrRPjJIYLNvmPTDP@LFrnFUyfgX\t1.99\nHg~L`LaC@HrRPqQYI]NvmTBDP@LFXUt{yE@\t1.99\nj`a`P@D@ejrSKZrkbTu@DUT@@pXbcCN}rA@\t1.99\nHg~@@@rQQQUQIUqwSMP@P@LA@fzUv~YeF\t1.99\nHkmH@JXLdRRbbQtzjB`@@XCF\\Lki|a@\t1.99\nHclD@@kIEEKDeeKmUP@P@LNcNFUwbHP\t1.99\nHkmL@FFnOIIEEEBqvjYjf@FAXpsqBT\t1.99\nHknD@EADfVU}eVjjjh@X\\UMpsoDpp\t1.99\nHcLH`ABHRYWYzZ@Bj`@X\\BXURoD{P\t1.99\nHcLH@@rIQQQHkkTE@@@pxj[iW^YBB\t1.99\nHg^L@DB]RY[VWEVh@bh@FEISG_DIP\t1.99\nHg^L@DBSRY[VwEVh@bh@FEIS[_DHP\t1.99\nHcND@EADfVU~UZjjj@FCES\\L|SM@\t1.99\nHmtH`ABHRYWYih@Jh@FC@fET|Se@\t1.99\nHmLD@@qJY{WJeZj@B@A`rMproDb`\t1.99\nHmM@@@qJWYWBeVf@B@AaVMpssHQP\t1.99\nHkmH@DhDfUnYTvj@b@@X\\fCJz_Hh\t1.99\nHcML@DpFEIeY~kfjYj`A`RYJ|Sm@\t1.99\nHeTH`ABHRYWZf`@j@A`pIaUODxP\t1.99\nHmvH@AAJYUgehHJ@@XBfES]J|`P\t1.99\nHmvD@JADeUeYUjjj`AavDwCJ|b@\t1.99\nHmtH@@RYeUEZ```@F@aaTwBoDbP\t1.99\nHcND@DBTfVulUZ`BH@FAIS[sBEP\t1.99\nHcND@DATfVutYZ`@h@FAIS[sBxP\t1.99\nHeULBDpFGNRYV^nZif`AaB\\Si@\t1.99\nHeVH@FAIe]ZZjjj@FG@fEWCqHd\t1.99\nHmtH@@RVYWeVhH@@FCTwCJ|rDH\t1.99\nHeVD@J@TiWUJjjjh@XMBXS]rI@\t1.99\nHmtD@@QIee^UZjjh@XLULLkqLx\t1.99\nHifH@FAIe]ijjj`A`pIaUwD`P\t1.99\nHifD@J@TiWTjjjj@FCPfDw\\`P\t1.99\nHeTD@@QIemQejjj@FETwCODp`\t1.99\nHaFH@FAIe^fjjh@XTBXU\\RN@\t1.99\nHidD@@QInUxVjj`AaQTwCqDp\t1.99\nHaFH@FAIe^fjjh@XMBXU]r@@\t1.99\nHifH@NAJ[W[jjj`A`tXULOHP\t1.99\nHifH@NAIYW[fjj`AaVESCqHP\t1.99\nHiFH@LAIYVjjh@XBBXUMp|b@\t1.99\nHaF@@@Ri][jjj@FCPaTw\\c@\t1.99\nKA|P@TBTkuUT@XUFDw\\a@\t1.99\nKA|@@@eSKUU@FCPfDw\\b`\t1.99\nsNqdEbOBABUVmU@LMJ~P`\t1.99\nsOy@FDfUkjj`FEBqoEF\t1.99\nsNx@AdbbMUPCAbXu|b@\t1.99\nsNy@LDeVjj@XTXu|TP\t1.99\nsNq`IVeUuTpCCXod`\t1.99\nsGY@DDfyj`FBVxcP\t1.99\nHeUH@DDDf[WxVjjh@XDUMwDrP\t1.99\nKA|@`XDQtPdkOUU@FCABDw\\b`\t1.99\nHieH@DDDf[WaZjj@FAES]qLD\t1.99\nsKTHLPDIRxtl@p`xh`\t1.99\nqC`P@HRfhCBCfbp\t1.98\nsJU@DPdvu@OLlP\t1.98\nHieH`B[`BDiWWBiYj@FFA@|QJ@\t1.97\nHeTL@@pTifwkjjjh@X]BESCODs`\t1.97\nHidD@@[IEDiZzjjh@XMFESCqDx\t1.97\nsJP`@dfvdAaEqX`\t1.97\nsGY@DDfuj`FBQxmP\t1.96\nsGX@AddQjhAahcqZ`\t1.96\nsJQ@@dsU@LDCsQX\t1.96\nqCh@CIKTA``nJX\t1.95\nHiDB@@SaR[Ufjf@FAIeCrB@\t1.95\nJoc@HCADRDILkZvm]Jv|@QP@D@AaHIeSiv\u007FHX\t1.94\nsFt@AdigUjXA`d^KT\t1.94\nsFt@@Ti_FVhAaIsKT\t1.94\nHg~@@@RiU~Ukcnjj`@`@X[BXUMJz]|c`\t1.94\nHcND@IADfyW[aZ@Bh@FCES\\n|a@\t1.94\nHeTD@@QIgeQej@@@FEYS\\OHd\t1.94\nHklL@@kldTtTRQEZBHb`@XTcCN|`p\t1.94\nHmtD@@iJ[eWihHH`@XDaCODYP\t1.94\nHidH@@Rfuvz`@`@XXeMsLHP\t1.94\nHeUD@NdBRYgeajjj`A`HPeMpsqLH\t1.93\nsGX@@eSUT@pLVOIP\t1.93\nHg}LbLGSl@b~rQQJIIFEuMUUUKR@LBBDKbCH\t1.92\nJoBaQFBivxAE|eLlzmeuMUUTsL`AaPPaCqJt\t1.92\nHk]LbLE]L@`~RfV]xnijjjei@FAABEqJD\t1.92\nj`aA@@GYIHhbeHfiVj``jJ@@XJQ`eFBdkQnP|\t1.92\nsNqLKIARFdLddjjj@XP|c@\t1.92\nsJP@DiVhAaaCqK@\t1.92\nqCb@AIj`LHNZK@\t1.92\nsGQhHl@cIIBmPCBGdh\t1.91\nsJP`@dfvdA`enP`\t1.91\nsJU@DPdvt`OLlP\t1.9\nHmuD`LVD@HrRRqIXYV`@`@XES\\ODqP\t1.9\nHmtD@@yIYe^UZ``@@XBaTwCJ|b@\t1.9\nHieH@LDDeYWajjj@FGABES]rA@\t1.9\nHeUH@LDDeYWxZjjh@X\\DHUMwHd\t1.9\nqCp@AJZ`LINXK@\t1.9\nHk]H`AdpbDfUmUiev@Bfd`AaPIe[qND\t1.88\nHaED@dpFChdi[dfUjzjjPA`bUqLx\t1.88\nsGY@LDeUj`FFQoEL\t1.88\nsJ\\@@bdks@OEb\t1.88\nHk^@@@rQQIFIJftpHH@@@FDieLLki|``\t1.88\nHieDPLZD@HhHrREQKaVii@FAYWCr@@\t1.88\nHcLD@@QIn[WeVj@B@AarYS\\L|cP\t1.88\nHaDH@@RVU[fjj@FEXUMwDa@\t1.88\nsNx@AddTMUPCBqXwdX\t1.88\nsGX`LDdwMT@pQoH`\t1.88\nHaDH@@Rfu[j@@@FFIS\\sBx\t1.88\nsJX@@eSU@LINYX`\t1.88\nsGY`hEiNyIgZhAaCrT\t1.87\nHid@@DjU^nBBH@FGPaTwCrL@\t1.87\nQMB@HRZAyd`\t1.87\nHcND@MADfVU~UZ``H@FCESCN|c`\t1.86\nHidD@@EIe]ih@J@A`pIaTOHx\t1.86\nKA|PD@pSddbljj`CAhSBknPp\t1.86\nsNxA@IrRQVjhAaXlWrT\t1.86\nsNqhHl@cIICej`FBMyF\t1.86\nHeTD@@iJ[e^f```@FAHPsqFx\t1.86\nsGX@@eLuT@pHfoDb\t1.85\nHmtD@@gIELeDVz`@j@A`RYr|Pa@\t1.85\nsJQ@IGuPCAQ\\VH\t1.85\nsOt@AdieuZf`FBQxjp\t1.84\nHidH@@Rfufz`@`@XXf]sBXP\t1.84\nsJX@IGuPCCKGd`\t1.84\nqCr@XISTAaEqS@\t1.83\nsJP`@TfVhA`d^S@\t1.83\nHidH@@RYm^Fh@@@XLDES]qLD\t1.83\nHcLL@@kdinYTUhHbH@FEHpsoHT\t1.83\nqCb@AIZ`LDmqB@\t1.82\nsJY@LDeZhAa`mqP`\t1.82\nsJXHLHaIVj@XXQ\\VH\t1.81\nHid@`@qDfUvf`@h@FE@fEWDxP\t1.8\nHeTD@@EIe]jZ@Bh@FC@aUp|SI@\t1.8\nJoBD@@IKLrjzv[mP@@@@@XZeMprng[|b@\t1.8\nHidD@@QInUxV`@@AaB\\hj[a@\t1.8\nsNq`IVeUuTpCAQ|VH\t1.8\nHidL@@pTifvnjjj@FCPaTp|SN@\t1.8\nsJQ@@dsU@LMEnHD\t1.8\nqCp@BOTAyhl\t1.8\nHmu@pBXJpZqZqIf[oi`bH`@XX@ksBZP\t1.79\nHcND@DBdeVyU]Zj@@@FGHU\\LksHsP\t1.79\nHmvH@LAIUYVUZX@@@XLc\\LkqHH\t1.79\nHaF@B@B\\ddRK]UUP@pZDrf{dD\t1.79\nsOxA@IRi^njj@XNKFod@\t1.79\nsFx@@eJfuU@LCL[qA@\t1.79\nsGU@E`djmL@pROEJ\t1.79\nHo~DPJHPbDbDf{VyVwjZj@@@FBaBUpkiwqF\\\t1.79\nHkmL`LVDL@cIIKDedaeZ@Bd@FETwBoDsP\t1.79\nHmuL@ATBYIf[oijjjh@XRDIe\\LkqBX\t1.79\nHmvD@EADfVuqej@B@AaUMpkrB@\t1.79\nsGQ`@ZdrmT@prMyL\t1.79\nHaD@`@bDfYVz@`@AaPXS]rJ@\t1.78\nsJX@@eKU@LEEsCD\t1.78\nsFx@@eJf`@@LCEkrD\t1.77\nqCp@AJZ`LDEqS@\t1.77\nsJT@@TeZhAahmq@`\t1.77\njhiQ`@FBisvRRJJJKEYBLxMTsAPI@@LIDXJRmV\\D\t1.76\njhiQ@@DT@drsJsNku`mUAQED@@ptcCIJtZsoAyB`\t1.76\nj`q``@ERuddbRRbnRXk`jjjYjj@AaJBLDxHug\\ax\t1.76\nj`qP@@@BRfUWyWdg^hHH`@@@FA`HpRcNBtZsnHex\t1.76\njhya@@J@RfV]YwIQUoAjTH@@H@@XJYrRmFl{p^PP\t1.76\nj`a`@@@YIEChhhdW^jjhJ@@@XV`cAJBdkQkN}rC@\t1.76\nj`ia@@D@RYYYU\\cMNmyjh@@@@@FDd\\EIVcV]qFO@\t1.76\nj`q`@@@YIEDdcDef\\]zjjhHB@@XN`cAJLyIVcV]x\t1.76\njhiQ`@DD@hJSLwJnyFV|uU@pPp@C@VNBehuo\\PQp\t1.76\njhi@`@H^RjyV}}EjsjjjB@j@@XF`cANBdkQkAyA`\t1.76\nj`qP@@@\\RiUwYVgG^jjf`@`@FFhHpRcMFl{wDBd\t1.76\nj`qa@@D@RYfv]TgK^ZjhHH`@FBbTxJRmFm{bLN@\t1.76\nj`q@`@@TRfYW{TRg^hJB@@@@FFdXIpRmFl{wHL@\t1.76\njdy``@G\\DeTsJn~j\\[uTCTEEP@C@vBTYIVcV|FH\t1.76\nj`qP@@@LRe]UYTeG^@Bj`@@@FAhHpQgAZMYwnPd\t1.76\nj`qP@@@\\RiUwYVgG^jjj`@`@FFhHpRcMFl{wDBd\t1.76\nj`aq@@DV\\CHheEDcddkSTE@UP@C@rBTYpTeFl{p\t1.76\nj`q@`@HNRju^uu`cVjjh@J`@FFhHqQ`iJt[wHD@\t1.76\njhi@`@@HR[fuvUdkpVj@B@@@@XZQagARMYw`|aP\t1.76\njhi@`@@HRYyWVUEKpVh@Ijh`@XJqQgAFl{p^PT\t1.76\nj`qQ`@DXArRSLm\u007FJlD{sUSL@D@@pihug^ycEA@\t1.76\nj`aA`@HLyddbTLrRbMzjfBB`@AaKAARUhug\\ax\t1.76\nJoBfPCRio@QddbQRRtuMpuUULuP@X]FEWS_DQp\t1.76\nj`qA`@@HddwJzroAN|t@E@@@@LAHISQkN}qCN@\t1.76\nj`q@`@@HR[YWYTTg^Z`@`@@@FBdTYpVcV]{dL@\t1.76\nj`aQ@@DZ@dwLjsrkMTDEE@@LMHhs`iJMYwnPp\t1.76\nJoB`@@CddRRJTRU][uUU@A@@X[BXU\\ng[|Qa@\t1.76\nHg^BpLaCl@chcPcIIKEEeiJkPPDh@LJqnE^Qh\t1.76\njhq``@I@BeLljrrIUADQUT@@pT`eF\\tZso\\`D\t1.76\nj`aa@@C@RYegU{Ufj``b`@FFlTYpTeZl{wHH@\t1.76\nJoC@A@ANRQQIUQIDuoUUL@D@AalIeMrz]oq@D\t1.76\nHo}L`L^DL@cIIChdliiUZ`@j`@XSaMpsiwrG@\t1.76\nJoB`@DgddbbRRQpgmuPUAE@@XJfES\\J{_Dep\t1.76\nJob@@FRRJJEIKXRv[uP@@A@@FFyS\\Lkiv\u007FHH\t1.76\nHg\u007F@PBWPbAbLbbbRfaSR]pPLUQ@CBPqae]Np\t1.76\nJoBB@D[ddfbRbJwC[uUTBA@@X[BXWCJg[|a`\t1.76\nHg]L`EnDpHSIIKDeddLmP@UP@LEALinE]OdV\t1.76\njhqQ@@DT@drlsLjXKUAQEP@@ptc@iJtZsoA@\t1.76\nJoBp@@XUMNRJJIESIUunADT@@@AaqMJz]|ap\t1.76\nj`aa@@I@RfVUYXRjBHbj`@FBdDhsfcV]{dE@\t1.76\nJgCFAD`esJSOJrhrn{[UUUUT@F@eS\\Jz_DrP\t1.76\nHg}@@IOIIEDddbI{]TEPQ@@pULJfxUv~II`\t1.76\nJoBDPD@DIWLrjzv[mP@@@@@XZeMprng[|`@\t1.76\nJoCB@BsnRRIIJJyLoCUUSMU@A`LIa\\IwrG@\t1.76\nJoB`P@vDPdrnlktqw@AUUQ@A`hIaUJz[|c`\t1.76\nJoCd@bDpAtRdsLm{euoUTpPD@Aaprng[|a`\t1.76\nHcNDpLa@BJbFbLddLTJRzmT@@@LFinFUyF`\t1.76\nHmnDBJAZgIDeDdXPjzZjjjh@XCBXS\\L|Re@\t1.76\nHk~LBAACFTeYVUxYUSZjjjj@F@xUMtkoD`P\t1.76\nHo~B`LaK\\@cIICEBeeEuZh@I`@XUprk|Rf@\t1.76\nHctNPDpfFUCpkprRQRVSSUST@LFpfzUyF`\t1.76\nJoB`@@NeRk|kuqwUUP@T@AalIaTtkiorN@\t1.76\nJoB@P@QBSLjsnNv}HP@@@@FD`U\\Lj]|sM\\\t1.76\nJoB@P@QBRsLjnmf|t@@@@@FDaS\\Lki|rK\\\t1.76\nHo~L@C`YrRSIFYIRfjjfjj@FDpfTr{[|``\t1.76\nHknL`EaLBDrRRqIYHYZ`@j@AaHIeMpkoHL\t1.76\nHklDPNGX`xcIIDcEeHRjifj@FGPfDvgrE@\t1.76\nHg^L@MAirJIJHsIUt@QUT@CAPSFxYW[yA`\t1.76\nJoBd@BNBRsLjnmf{TEAQD@FBeS\\LkiwqH\\\t1.76\nJoB`@@ReRkkZuuoUUT@D@A`\\IaTwKiv\u007FHd\t1.76\nJoBd@BNBSKLjnmf{TEAQD@FFhUMprng_H\\\t1.76\nHg^B`LaCl@cIIBidliJkT@Dp@LJ[af~Ie@\t1.76\nHg^D`La@BLddLTVRVvmTDD@@pznFUt~Hu`\t1.76\nHg~@@@rRJIJqIMqwUUP@P@LMaLIne]N~QP\t1.76\nJoC`@@QrTvnljwV|uUP@P@FDxULng[|QA@\t1.76\nHg]L`LnDD@cIIBhhd]ikTC@P@LFqnW[yD`\t1.76\nHcoL`DphWP|LddtTSeMpmUUMP@phj[ayF`\t1.76\nJoBD@DIU\\k^x{[uUTDA@@X[BXWCJz]|Qg@\t1.76\nHg~H@KAIfYU^Svz`hJH`A`XXUMprng_H`\t1.76\nJoBf@BXCBdrl\u007FJuunuSL@D@AarXng[|c`\t1.76\nHg~D@Eg\\ddRRTUsKpuUTsU@CChSBxSyE`\t1.76\nHo~H`CDHaIe]gVih@Jjj@A`hIaUJ{[|c`\t1.76\nHk\\H@HrRSIFIJftzjh@H@FBpfTprngrB@\t1.76\nHg~H@DCHhhieBdWG[UU@A@@pDjFUt{xf\\\t1.76\nJoCD@B@dsMl{eunuUPPD@AaIUpriv\u007FDsP\t1.76\nJoBf@BXCBdrl\u007FJuunuSL@D@AaRz]osDs`\t1.76\nHcLDPNEXcXcIIDcDYBUULu@CChSBY]y@`\t1.76\nHg}@@@yJe}YVfNzjj@B@A`lHUMJz]|SN@\t1.76\nJoBD@@ISLk^x{[tE@@@@@XJfEpriv\u007FD[P\t1.76\nHk^@@@rQQIFIJftzjh@H@FFaBYSCJz_HH\t1.76\nHg|@@DjYV}~T{jjjjj@FEaBXS\\LkiwrJ@\t1.76\nHknD`La@BLddJbRRTujBB@@X]S\\LkoDpp\t1.76\nHk^@@@rRIHjIHjtzjh@H@FBpaTprngrL@\t1.76\nHg\u007F@@@adjuueZX{jjh@H@FDiaUJz]|RN@\t1.76\nHclLA@iTfnrQQQQEMt{PUAB@CBbFUwdL\t1.76\nHmTNPDpfFUCpkprRQRVRZjZ@FEXSRoHx\t1.76\nHg}@@@yJeWe^nNzjhHB@AaLIaTLz_DVp\t1.76\nHcL@p@bBBAbDfYVyi``hh@FGAaMpsqEt\t1.76\nHg|H@HRjymUV}NjB`hH@X[FTwCJz]|``\t1.76\nHg~H@DAIenUTfvvj``b@A`IMprnwsDpp\t1.76\nHg~@@@Ri_VUicnjj`@`@X[BES]Jz]|c@\t1.76\nHknB`LaML@cIIKDeeAeZ@BX@FETwB_Hp\t1.76\nHg]H@DlLbbTTRNRzuAPE@@peBinFUOdV\t1.76\nHknL@MAirJIJHsJn`BJj@AaHIc\\LkoHL\t1.76\nHg]L@FFnoIIEEDl\\NuSMTl@LFqaeSyA@\t1.76\nHg]H@FlDfYU^QVhHHj@AahQaTwCNgrK@\t1.76\nHg}@@@UJfUW\u007FegZ`hHj@AaJXS\\Lz_Djp\t1.76\nHmOD`DphTOCIIMEDz[aZjjZ@FEES\\OHd\t1.76\nHg|H@HRju^uXSnjj`@`@XKBYSCJz_DJp\t1.76\nJoB@@BULjo{rsl@@@U@@FBiaTwCNwqC\\\t1.76\nHk\\LA@kTfnRfYgUndzBhJD@FEDLz_HT\t1.76\nHk]@@@yJeWmZ{Sjj`@`@XSBXUCJ{qNL\t1.76\nHk^D@Eg\\ddRRTUY^FjjfZ`A`tIa\\OHL\t1.76\nHmt@p@bBBAbDfYVvfBBb@A`pXSCODyP\t1.76\nHg|H@@RYWVUicn@BZjH@XRfETkiwq@x\t1.76\nHk\\H@@RYfUVSWZ`hJB@FFaaTwCJz_H`\t1.76\nHmL@P@BNBDigy\\jUiffi@F@`fUproHH\t1.76\nHk^H@DAIenUr[SZjB@`@X\\S\\Jz_LSI@\t1.76\nHo}@@@UIYeV^d~hHJJ@A`iTwCNg[|a@\t1.76\nHmtHPAfHVHrRQHiRDjjVh@X]BXURoHX\t1.76\nHcNL@MAirJIJHsUt@QU@@pDDqnFUyF`\t1.76\nHk^@@@Re]YTjtzjh@H@FFpfESCJz_H@\t1.76\nHo}H@DhDfUefWWZhHb`@FDi`rng[|b`\t1.76\nHk]@@@UJfUW~V]jB`b`@XBfDwCN|Rc@\t1.76\nHeNHBJBiReYVDJnfjjjh@X]BXS\\ODjP\t1.76\nHg|L@@RdfV^Urh{Zj``H@FGEpsiwqLT\t1.76\nHkn@`ECDRYWUZf`@jj@AaHIaTLz_Hx\t1.76\nHkn@`EBHRYWUZf`@jj@AaHIaTLz_Hx\t1.76\nHk\\@@DjU^ukmLHDjh`AaLHUCJz_Ds`\t1.76\nHo~B@NAK]IenWVeVijjfPA`Q\\I|Qg@\t1.76\nHcMD`LvD@HrRRqIYCJt@E@@p{J[aWP\t1.76\nHknD@IBdifW^EZB`b`@XRfDwBngrJ@\t1.76\nHcm@@@UJfUWyYvhJBH@F@iaMpsoDiP\t1.76\nHg^H@EAIYeUz]Z``b`@XRUMpsiwqHP\t1.76\nHclL@@RlbbTLRQTnmU@A@@pkae]xfD\t1.76\nHk\\L@@RdfV^uJmMjj@B@A`qCJz_Ds`\t1.76\nHk\\H@@RfYU\\]Tz@@@@@FGXWCJ{sJXp\t1.76\nHknH@DCHheEDceujB`H@F@hUMproHL\t1.76\nHknD@E@TfUvyih@Jj`@XJBXU\\LkoHL\t1.76\nHeWDbLihP@cNrREQICJt@@@LLxYy@@\t1.76\nHmt@P@bABDfUuZZ@Bj@AapIaUp|RU@\t1.76\nHmuD`LVD@HrRRqIXYV`@`@XUeMp|aP\t1.76\nHknH@FAIfYWyuhJBH@FDxU\\Lki|a`\t1.76\nHcnD@EgTjeUeexZjjYh@XMBXWCrE@\t1.76\nHo~H@FAIfYV}dv`hJJ@AanEWCJz]n\t1.76\nHcnH@DAIenuIeujh@H@FCEprn|qLX\t1.76\nHmtL@@Z\\bbRbLkh@bh@FG@fMpsrE@\t1.76\nHcNH@DCHheEDbnmPT@@C@TJfxYWdZ\t1.76\nHeWD`LihP@cIHUDdLkP@@@psNGbAH\t1.76\nHkmL@DpFEIeY}ZyjfZj@FEIbz_DzP\t1.76\nHcMH@DDDfyWWaZjjj@FGHUMtkqLt\t1.76\nHmt@`@bDfUujZ@Bj@A`HIaUtkrA@\t1.76\nHcvL@Hz]rRQQMSUUTt@LFSBimyB`\t1.76\nHknD@FADfy^Utvix@@@XTLki|QJ@\t1.76\nHcND@NALbbTtRLjuMUR@LBKawbMH\t1.76\nHg^L@NAKRY[eueVijje@FAEpgqFt\t1.76\nHcN@@@Re]eUvjh@@@XRDXWCJ{qDh\t1.76\nHid@PNbAbLddUTaJjZ`A`tXU\\OHP\t1.76\nHcNH@AAJYU^zZBBJ@A`NES]J{rD@\t1.76\nHcN@@HrRQJqIKmUP@@@pVDpjxYWP\t1.76\nHiWH`DphCpRjyj[aZjj`A`UMp|``\t1.76\nHmv@@@rRHjIJUjh@@@X]BXwCODIP\t1.76\nHcNH@IaJkU~XZjjj`AaLIeMpkoHH\t1.76\nHknH@EAJ[WUfz`@jh@F@iaTtz_HD\t1.76\nHmt@@LddTQVeZhH@@F@iaTwCOHT\t1.76\nHmuH@DDDfyW^Ejjj`A`rES]ODqP\t1.76\nHaeH`Dr@|DjybinZjj@FAES]r@@\t1.76\nHieH@DhDeVWaZjj@FCIaMp|RJ@\t1.76\nHmtH@@RYWUih@Jh@FG@fETkqND\t1.76\nHeUH@DDDfyWxVjjh@XTaTw\\SI@\t1.76\nHif@`ABHRYWZZ@B`@XLBXU]rA@\t1.76\nHid@`@bDeYWaf@@@FE@UMwLaA@\t1.76\nHeTD@@QImYQejjj@FGXUMpsrD@\t1.76\nHcNH@JAIY^UWVg`@@AaWCJ{qHH\t1.76\nHmtD@@QIevVUZjjh@X]eMproHH\t1.76\nHeVD@AADfyWxVjjh@XTUMt|QL@\t1.76\nHmt@@DjUgjDHJJ@A`LHULLkqLx\t1.76\nHmvD@D@dfueYUj`@@A`vMproHP\t1.76\nHmv@@HrRSFIJUjh@@@XSBXS\\Lj\t1.76\nHmtD@@QIn[VUZh@@@XLfMpsqJT\t1.76\nHie@@@EJe}ijjj`AaTHU]sLXP\t1.76\nHaE@@@{IHhbSUUT@LJaBinIg@\t1.76\nHif@@@rQQIFf@`h@FEIeMwDHP\t1.76\nHieH@DDDfyWaZjj@FEHUMwDpP\t1.76\nHefD@Hz\\ddTSUUUH@piLJfy@`\t1.76\nHaF@B@B\\ddRK]UUP@pZDpj[d\\\t1.76\nHeVH@DAIemQej@@@FAIPssB[`\t1.76\nHieH@JDDiWTjjjj@FCPfDw\\`P\t1.76\nHaf@@@RYfJfzjjh@X]BXUMwH@\t1.76\nHaDH`F@HRf^rjYj@FE@fTwDK`\t1.76\nHaE@@@yJUtjjjh@XUBXS\\pJx\t1.76\nsNqlHlOBqCbLddlZj@XEZodH\t1.76\nHeV@@@RUYTYy`@@AaUMpsqDp\t1.76\nHeTD@@iJ[e^f```@FEHS]OHD\t1.76\nsGPBAbJPLaYAInjhA`aNS@\t1.76\nKA|P@QrUSMUT@XTfES\\c`\t1.76\nKA|P@XBRlkUT@XTaTw\\b@\t1.76\nsNxA@IrRJFjhAaXl[qA`\t1.76\nsFuHEhOAJnMZj@XHwbF@\t1.76\nsNy@BDeVjj@XVKU|`@\t1.76\nsFq@@drfuU@LBl[qB@\t1.76\nsFq@@drfuU@LKEc^P@\t1.76\nsGPhH`DYIHUj@XKWdH\t1.76\nsFt@@TiTt@@A`i^IX\t1.76\nsFt@ADiTt@@A`c^IX\t1.76\nsJY@LDejhAahcqC@\t1.76\nsGY@DDeYj`FAVMyH\t1.76\nsGR@@cTe[f`GbM@\t1.76\nHeTD@@qI[eQej@@@FGIeMpsrH@\t1.76\nHifH@AAJ[W[j@B@A`VES\\QA@\t1.76\nQMhHRYAxb\t1.76\nsJY@BDizhA`inID\t1.76\nHeTD@@EJ[Uvz`@h@FAIgSqBh\t1.75\nHidH@@RYm^Fh@@@XMATwCq@p\t1.75\nsNp`@dfVZj@XJobq@\t1.75\nHkmHpBXpkIkMkDfYemyXHbbP@XDASoLXU@\t1.74\nHidL@@sdifzajjj@FCPaTw\\SA@\t1.74\nHaDD@@yJY^fjjh@XUFEW\\QF@\t1.74\nHaDD@@YJYZnjjh@XUFES\\QN@\t1.74\nsJQ@@dkU@LLL^JH\t1.74\nqCp@AJZ`LDmqB@\t1.74\nHeTH@@ReyTYj`@@AaTTwCsDrP\t1.74\nHaF@@@RYe[hB@@FAXS]q@h\t1.73\nHaDH@@Rfu[j@@@FFIg\\pjX\t1.73\nHg\u007FD@L[`QIeyUTYNvjZ`@@A`vEr{_HL\t1.72\nHcm@`FtHaIe[UkiV@@@`@FC@fTJ|cP\t1.72\nHcoH@DJ`RUeUVy]ZZ`@@AaRTL{qHT\t1.72\nHk^H@IAJ[VuVzUh@JZH@XTfDq|P]@\t1.72\nHk^H@IAJ[VvvzUh@JZH@XTfDq|R]@\t1.72\nHg~H@IAJ[VvUneZ@Bfh`AaRXSWqML\t1.72\nHew@@@pldTTJVTLmP@P@LBqngbDH\t1.72\nHcnH@IAJ[Vw[iV`@ib@FEIaMoDFP\t1.72\nHmN@@@rRJJYDfeZ`XB@A`U\\L|sFT\t1.72\nHev@@@rQQQHyPsPLA@@pHfx^XER\t1.72\nHmN@@@RfYWraV`XH`@XDS\\OLBU@\t1.72\nHcn@@@RifurZ]jA`b@A`U\\L|sEt\t1.72\nHaDH`NBDRYWih@H@FAIa]q@h\t1.72\nsOt@@tigujj`FEbqoDF\t1.72\nHid@P@bAbDfY]n`HH@FEAa\\OHh\t1.72\nHeVH@LAI[eQej@@@FGIeMpsrH@\t1.72\nHaDH@@RVU[jjj@FG@fES]rH@\t1.72\nHeTH@@ReyTYj`@@A`tTwCODS@\t1.72\nQM`BN`^YH\t1.72\nHmNH@NAJ[VUnFZjjj`A`LXULLkrD@\t1.71\nHevH@FAIeYZ[ajjjh@XCBXU\\L|`@\t1.71\nHeTB@@pYRf[^njjj`AatHUMt|SA@\t1.71\nHidL@@X\\dTRekjjj`A`tXU\\ODQ`\t1.71\nHeVD@IADfyWxV`@`@XDUMwDRP\t1.71\nHieD@DDNRYWbijih@XDfEWHD\t1.71\nsGPYHlPDGlddJu@LIP\t1.71\nsGXA@IRjZj@XFKWdH\t1.71\nsJY@LDeZhA`Pl^R@\t1.71\nsGU@DPdsmR@pbxmP\t1.7\nJoC@@@ITskNx{[uA@@@@@XZaUprng[|c@\t1.7\nHg~@@@RfUYwySn``@@@@XZfDwCJz]|`@\t1.7\nHcLHpEbPRPrPrRRqIYCMT@E@@pJfx^Qh\t1.7\nHmtHpEbPRPrPrRRqIXYj`@`@XES\\OHT\t1.7\nHeVD`La@BLddlRPrm@@@CBlinGdR\t1.7\nsNplJyDJHtQddeeUPCAqXu`\t1.7\nqCr@XIJrAyhl\t1.7\nqCa@CIJtA`enR@\t1.7\nHaE@@@aJyUnh@@@XUaTw\\PH@\t1.7\nJoBBPDH@QddabbRRrv[kP@@@@@XCS\\Lki|Rw@\t1.7\nsNx`BDdw[UPCCKObM@\t1.69\nJob@@FRQQIIEI\\Lk[dDaP@@@FBiaMpz]orJ@\t1.68\nHcMB@NTLgTify^ajjjj@FBpaTwCJ{rL@\t1.68\nHmv@pBBlFlVlRYeYZXHJH@FAApksLJP\t1.68\nHmuL@AdFUJYY^njjjh@XSFES\\LkrD@\t1.68\nHifD@ACLdTTqkjjj`A`tHS\\ODr`\t1.68\nHeV@@@rRHqICMT@@@LNaJ[agdX\t1.68\nHmvD@DATf^Uqej@B@A`rYS\\OHT\t1.68\nHeTH@@RfUWihHH@AarXS\\L|b`\t1.68\nHie@@@EJeWkjjj`A`tIaTwDxP\t1.68\nHaFH@NCIEDcSUUT@LJ`j[nHf@\t1.68\nHeVH@DAIemQej@@@FFIS\\HnFP\t1.68\nHaG@@@qdig|ijj`A`RXU\\Qn@\t1.68\nsNx@@eLmUPCBbXwbS@\t1.68\nsNx@@eRmUPCBqFodp\t1.68\nsGQHLHaI[ihA`aNHt\t1.68\nsJX@@dkS@LMD^HX\t1.68\nsJX@@eSU@LDcqS@\t1.68\nqCp@AJv`LHnJX\t1.68\nqCp@AJZ`LDMqQ@\t1.68\nsGX@@eJuT@pHfODj\t1.67\nHaF@@@RVU[f@@@FEXUMwDB@\t1.67\nqCb@AIj`LHnXS@\t1.67\nsGY@DDf]j`FDWMcP\t1.67\nHmuL@DpFEIeY~nZifh@XHf\\siT\t1.66\nQMPBchFD\\Y@\t1.66\nHaD@`@dDfUzZ@B@A`RXU\\c`\t1.66\nqCp@BOTAaIqK@\t1.66\nHidH`ABPRYWZZ@BP@XDfEWHx\t1.65\nqCpB@ReMPFFbwH@\t1.65\nsJP@DiZhAaaMqH`\t1.65\nRFDEYFCBNT\t1.65\nsJY@DDfvhA`l^Q@\t1.64\nQMbDBDf`XUqD\t1.64\nsGU@EPdjmT@prqxiP\t1.64\nHeVH@IAIe]ZZ@Bh@FC@fEW\\PY@\t1.64\nHif@@@rRJEKaj@@@FEES\\ODS@\t1.64\nsOpH@DILkW@@@LBcWqC@\t1.64\nqCp@AIj`LEEqC@\t1.64\nHifL@ABnReYrZjjh@XMaTwCqHP\t1.63\nHmtD@@yIYe^UZ`P@@XLeLLkqHH\t1.63\nJoCFP`s[aH]L|drml\u007FTr{@AUTp`A`RUk|SA@\t1.62\nHcnHaAdICigdfUmffWX@JjD@XXe[sLEP\t1.62\nHeUH@DhDeVYzVjjh@XLfDwSqHD\t1.62\nHeVD@DAdfygFV``@@XYSSsDxP\t1.62\nHifH@BAIfUXXHH@AaPXUMqND\t1.62\nHieH@JXDefWaXB@@FFEp|RF@\t1.62\nHaF@@@Rig[jjj@FCPfDw\\b`\t1.62\nHaE@@@qJYZfZjh@XDeMwHH\t1.62\nsF|HLZ@aJYuif@XP\\RH\t1.62\nsOt@AdiVMZj`FAQkyH\t1.62\nsGY@LDenj`FEDQoI`\t1.62\nsGX@@eJuT@phfMyD\t1.62\nqCx@AYIV`LDEqS@\t1.62\nHeVD@IADfyWxV`@`@XTUMt|a@\t1.62\nsFt@@TiTt@@AaikrH\t1.62\nqCr@HIJtA`enR@\t1.62\nsGQ`@jdvmT@pruyB\t1.62\nsOx@@eSI]UT@pLQ_Ej\t1.62\nsGY@BDfZj`FADQxmP\t1.62\nsJY@BDfZhAaaCqK@\t1.62\nsGQ@@dkMT@pVOMEP\t1.62\nsJQ@@drt`LHnKD\t1.62\nj`qP@@@\\RiUuyVgG^jjf`@`@FAhHpRcAZMYwnPd\t1.61\nJoB`@@\\eRk{JuuoUUL@D@A`\\IaTpkiv\u007FHx\t1.61\nHmvDPLa@BEbLddlRVFUh@H@FETwBoHp\t1.61\nHcLB`HQ]@HrRFIKJ\\Jp@S@@phj[my@@\t1.61\nHmuDbLVD@HslddlRVFUh@H@FETwBoHp\t1.61\nHg\\D@@QIme]XYZ`@j`@XReMpkiwrL@\t1.61\nHcL@@LdbRVbM\\DDUP@LIaLJfF]yB`\t1.61\nHeVB@FCAeJYyzzjjj@FGPUMpsqDp\t1.61\nHcLH@ErRQRiYCMT@E@@pYLinGbEh\t1.61\nHie@B@xIrRQHynjjj@FCPfES\\`P\t1.61\nHeTL`HS@BLddlRPrm@@@CAlinFP\t1.61\nHaE@B@xIrRQHmuUU@CAhSBinQp\t1.61\nHmtH@@Rfuv[j@Bh@FCXUMJ|QA@\t1.61\nHid@`@bDfUvf`@h@FC@fEP|c`\t1.61\nsNyDJQDJHR[nihA`QE^Q`\t1.61\nsNx@@dlwUPCCZobQ@\t1.61\nsNp@Di]jj@XLSFodP\t1.61\nsGXA@IRjZj@XYZ|TH\t1.61\nsGP@Divj`FABMxi`\t1.61\nsJT@@TfZhAahmq@`\t1.61\nHmtL@@JTfYwfzB@j@AapXU\\L|aP\t1.61\nHedF@@PfFTf{nZjf@FCIaTt`\t1.61\nsJX@@eSU@LECqK@\t1.61\nj`q``@D@]dTTtTRVJXKUfjA`bh@AaSdeZ}qFC@\t1.6\njhia`@D@]yIYfU}VgEFZ`XHih@A`q`iJtGbHV@\t1.6\nJoCB@@a^QIQQQHkLNfuHTAT@A`r]Jf\u007FD`P\t1.6\nJoCB@BA^QJJJIKEUrfu@pQT@AaSCJoqHt\t1.6\nJoCB@BA^QQSQQHyLNfuPLAT@AaWRnoqLD\t1.6\nHg~D@D@|bTTTRNU\\nmPLAT@CBfFUobQp\t1.6\nJoCB@B@zRsLk\u007FWJ[TCADp@FELLkoL`{@\t1.6\nHk^H@DCHhihhdVGSZhF@`@XEtkoLpS@\t1.6\nHk^H@ACHdhhhdVGSZdJ@`@XTgRi|rAt\t1.6\nHcnH@DCHeEEDcWKkTC@P@LBXYWfQv`\t1.6\nHmtHPABHVHRYeY[hBBh@FCAaUr|PY@\t1.6\nHcLHPABHVHRYeYvz@`j`@XLFEWKqIt\t1.6\nHk^H@DAIYfUzySZ`XH`@XDproLcS@\t1.6\nHg~H@DAIfye]avvjA`b@A`WRn|sCl\t1.6\nHeWDAHHPRISdfygXVijd@XIW\\Qi@\t1.6\nHigDAHK`RISdfyUnZfi@FBTOHH\t1.6\nHigDAHHPRISdfyeaZfi@FDUqFx\t1.6\nsNy@LDeUjj@XTKF|TX\t1.6\nsOt@@Ti]qej`FBeyef\t1.6\nsF|@AbeLzmU@LLm^P`\t1.6\nsGY@DDeUj`FFQoEL\t1.6\nHif@@@RUe^Gh@@@XTUMp|a@\t1.6\nHcND@DCTeUe^UZh@H@FGHS\\L{qHp\t1.6\nsOy@DDeekZj`FFu_EB\t1.6\nsJP`@dfzhA`inXH`\t1.6\nHmuH@LDDeYW^Fjjj`A`HPaTwSrE@\t1.59\nsJU@DPdru@LHnKD\t1.59\nsJY@HDizXAyeb\t1.59\nqCpPXQBRu@XXQ\\f@\t1.59\nsJQ@@dsU@LLL^JH\t1.59\nsJX@@eSU@LLl^R@\t1.59\nqCb@AIV`LDmrP\t1.59\nqCaPQ@Hck@XS\\`@\t1.59\nsGP`@dfyj`FBMxe`\t1.59\nQM`AIhFE\\Q@\t1.59\nsOx@@eLm]UT@pXbu_HP\t1.58\nHcND@LCTeme^UZh@H@FGHS\\L{qHp\t1.58\nHaDH@@Rfu[j@@@FFYS\\qBx\t1.58\nHg\u007FD`LFyt@aJYY\u007FlfkjjjYjPA`pPaC_Lkc@\t1.57\nHmN@`FBHRYVukiV@@@@@XLBYPkrI@\t1.57\nHeVL@ICARf[^njjj`A`LHUMpsrL@\t1.57\nHifD@AAldTRekjjj`AatXUMp|a@\t1.57\nHid@P@qFqDfUfn`BH@FA@fMqND\t1.57\nHiTH@@RfU|kahDB@A`QMp|QJ@\t1.57\nHieD@DDFR[fVEijX@XQwDR`\t1.57\nHaFD@NBdefZyfi`AaC\\RB@\t1.57\nsFy@HDi[FehAaKqX`\t1.57\nsJT@AdizhAa`mqP`\t1.57\nsGQ@@djmT@pvMxh`\t1.57\nsF|@AjeKxmM@LEMqJ`\t1.57\nsGT@A\\dbMU@LCEcrT\t1.57\nsJP@DizhAa`cqS@\t1.57\nHifD@D@TefUnZjf@FAXS]qHh\t1.57\nsNpP@btf{Zj@XEXodh\t1.57\nsKT@Adi\\Vf@XS\\tQ@\t1.57\nsFx@@eRfuU@LMC^IX\t1.57\nqCp@AIZ`LEEqC@\t1.57\nsGQ@@dkUT@plVMy@\t1.56\nHiDJ@@PnEInvZjX@XDfDOHh\t1.56\nHifH@DAIVUxV`@@AaQTwCqHP\t1.56\nsJQ@@eMU@LEMqH`\t1.56\nHidL@@p\\dTTqkjjj`A`tHS\\ODr`\t1.55\nHidD@@EIe]ih@J@AaPHUCqLx\t1.55\nsBP@H|UT@ppbyL\t1.55\nsGXHLHaIVjhAaaEqF`\t1.54\nHiDJ@@PfEIn{Zjh@XLfESCrN@\t1.54\nKAxH@TvRRJNjZ@LLSByG@\t1.54\nHeTL@@JTfYw[hHB`@XLFEWSrA@\t1.53\nHeVH@DAIgeQej@@@FCIeMp|bP\t1.53\nqCj@KAIV`LHnJX\t1.53\nsOy@JDiWMjj`FAbqyF\t1.53\nj`qPB@AVAHILkkJ{QakP@UUUD@CBRLDiIVcNxcg`\t1.52\nJoB`P@vD`drnlktqw@AUUQ@A`JXURngqN\\\t1.52\nHg}@`AlIAIe]YVfNx@Jjh`A`JXURngqNl\t1.52\nsNxA@IRfZjhA`xlZ~P@\t1.52\nsGY@DDf]j`FBMxe`\t1.52\nHidH@@Rfuvz`@`@XYeMsDHP\t1.52\nHifH@AAIYW[f@B@AaVESCrH@\t1.52\nHidH@@RfU~F``@@XLfDwCrJ@\t1.52\nQMJHBHvAaOH@\t1.52\nQMB@HRZAaOD`\t1.52\nsGY@DDeej`FBexlP\t1.51\nsJY@DDeVhA`h^IX\t1.51\nHidH@@Rfuvz`@`@XXfMsBhP\t1.51\nHaF@@@Rfu[j@@@FEXUMwDA@\t1.51\nHaFH@LAIYfnZjh@XDaUwDa`\t1.5\nsOy@LDeekZj`FAQkxi`\t1.5\nHaDH@@Rfu[j@@@FFIc\\pjx\t1.5\nHeTD`HP@cIHXdmpk@A@@pHj[nHD`\t1.5\nQM`BN`XUqD\t1.5\nHiV@@@Rifzyajjj`AatIaTw\\`P\t1.49\nsJXA@IRij`FFVODH\t1.49\nsGX@@eLuT@pLVODJ\t1.49\nj`iP@@@PRfUUg_EIF]zZb@@@@@FBbLEIVcV]{dD@\t1.48\nj`qaB@MBADILklmkQcoP@UP@@@CAPDXISakN}rB`\t1.48\nJocDP`rDatsvQQISJJVTr{[p@UUTL@FCIVg[|SE@\t1.48\nj`i@B@@ABTsLjotyJvgT@P@aP@@pd@cARUhunH]x\t1.48\njdy``@J@BdlsOJkbezKMUPTBH@C@QJLUYwh|qCQ`\t1.48\nj`qA@@@YHheEdXdaR]z`bB@@@@XZPQgARtZso\\aP\t1.48\nj`aaB@HB]DYIHkECDeK^jjf@H@@XZ`cAJ\\Ujso@\t1.48\njhqa`@M@IIIf]{eYgHHBjjh@A`xFBTxJRmFl{pP\t1.48\nHo~HHNnIAICIAXcIIKEEhcIvjBBJ`@XLwKiwrG@\t1.48\nj`qP@@@XRfV]{V`g^BBb`@@@FFdDhs`mFl{wHJ@\t1.48\nj`qQ@@JV@eMrsooEFmPPTUTP@LEHHs`iJtYwH^@\t1.48\nHg]DqNm]bPRPrPz]{IIKEDheIuPPLP@LLXSyE`\t1.48\nj`qa@@J@RVYgm\\Tg^YjjB@P@F@bTXhug^yb@m@\t1.48\nHcMDpnd]bPRPrPZQyYwlddlRbLkUMUT@LDj^Qp\t1.48\nHg^HHNnIAICIAXcIIKEEhcNuPPQP@LJ[ev~PX\t1.48\nJobD@@ILsJr|wif}PTEPe@AaXXUMprng[|b@\t1.48\nJoc@@@YHhhheEcrsYoPAA@B@@XJS\\Lkiv\u007FH@\t1.48\nHg\\HpMbPRPrPrRRqIPsCMT@ET@CCjXYt{yB`\t1.48\nJgB@P@AFRJJJIIFad}n`@`@`@C@PIne]ObCX\t1.48\nJoBD@@YHihdcDepv}@PP@@@FBeS\\Jz]orD@\t1.48\nJoCDPBbDPdrnwJtuo@AT@@@A`hIaUvg[|`P\t1.48\nj`a`P@A@P{rSKKOkJtuMUUL`@phS`iVxag`\t1.48\nJob@P@ABTsLjoWS^oPA@BD@@XBALLkm|Pw@\t1.48\nHg\u007FH@HyVrRQJIWIUtkUSUUT@LApjxYt~IE`\t1.48\nHko@PITpbNbLbbbTURXXBHZP@XBFTwBi|cP\t1.48\nHg\\LpINxdDdLdLddlRTJ`kU@DS@@pJfF~QH\t1.48\nJoB`@@YdbbTVLRUC[aAQ@@@@XJaTpkiv\u007FHx\t1.48\nHk~@@HrRQQIFHUgYNjjjjh@XJfEWCJz_HX\t1.48\nHg]DpBX{BlFlVlRYf{~zXHbJX@FA@JwqBX\t1.48\nHcl@P@BABLdTRafQTnuKLuH@pDDJae]yG@\t1.48\nJoBdP@sXQBSJvlkSJ\\@EMSD@FC@fUi|Sg@\t1.48\nHknHpINIAICICIIKDeMaVj@Hd@FATLkqFt\t1.48\nHknHpCjIAICICIIJeEDmNjhB@@FADJ{rI@\t1.48\nHg^HpCjIAICICIIJeEDiiuU@PP@LBHUwdV\t1.48\nHcN@HNbPRPrPVHrRRqQZSmTDD@@pInW^QH\t1.48\nHklDpMNIAICICIIKDeeAfj@BX@FATwGrA@\t1.48\nHg\\DpINIAICICIIKDeMhJuPADp@LBiaodJ\t1.48\nHk\\d@dpSAPzIZUyIfYoVgSjjifh@XEWZX\t1.48\nHg|H@@rJJIHqIMqw@PP@@@LEpj[iWSod@\t1.48\nHg~D@DClbbTTTjSB]mUL@D@C@TXYWSodX\t1.48\nHg^H@ACIEDdhlYrtDEEP@LMSBinFUt~QX\t1.48\nHk_H@HyVrRQJIWJneZjZjj@FGXU\\L{qHL\t1.48\nHo~L@M@iRYg^ufzB@jj`@XZFEWCNg[|aP\t1.48\nJoBd@EVBTwKN\u007Frg]AAQUD@FDhS\\Lki|ap\t1.48\nHg~@@@rQQQUQIUqwSMP@P@LA@fz]N~YeJ\t1.48\nJoCD@B@drsJkkYnu@@@@@AaJUprng_DYp\t1.48\nHo}L@FFnoIIEEEB\\avjYjih@X]cCJf\u007FHH\t1.48\nHedBJHSBCpzHFHfHKlddqTcUUP@pkJxY@\t1.48\nHc^@@HRjU^^dUGZjjjj@FBiaTwCJ{rH@\t1.48\nHcmH`FeXBLddRJRUMpuUUKP@pjLJnxbZ\t1.48\nHknH@ACIEDdhl^V``hh@FBiaTwCJgrM@\t1.48\nHg|@@LddTjTrPr]uP@@@@C@TpjXUOb[X\t1.48\nHg|@@LddTjRRPr]uP@@@@C@Tpjy]ObMX\t1.48\nHg}H`AfpbDfUmYZYS`@ijR@FE@fT_D{P\t1.48\nHcoH@HyVrRQJIWUtkUSUU@CAlJnF^IF`\t1.48\nHg}H@JlDinV]~T{hHJJb@FDhS\\Lki|bp\t1.48\nHg}@@@qJYWyVnNp`b@@@A`jES]Jz]|c@\t1.48\nHedDpJZHaHcHcIIKEiUUT@LNaLInGdR\t1.48\nHkmD@AvnrRQQsQKSjfje`AarXSRi|c`\t1.48\nHmvDPLa@BNbLddlTVeUhH@@FEXwBoHd\t1.48\nHg|H@@RYfUWd}mh@@@@@XRUMpriwrM@\t1.48\nHmOH@HyVrRQJIVneZjZj`A`vEWCODaP\t1.48\nHmv@pIbPRPrPrRRqIPUj`B@@XYPkqFd\t1.48\nHmu@PITHchcHhheEVF@bF@A`pYS\\OHT\t1.48\nHmtHpIbPRPrPrRRqIPUj`B@@XYSCqIT\t1.48\nHkmL@FFnOIIEEDlqvjYjf@FEXproHH\t1.48\nHiDDpJZHaHcHcIIKEjjj@FCPfDp|c`\t1.48\nHknD@AALbbTRbqWVijjd@XTWCI|Qe@\t1.48\nHo~L@AAGRYYY}UujZjjT@XTWCM|Qc@\t1.48\nHmMHADp@zIRYeuIiVjh@@@XUgCJ|b`\t1.48\nHg]L@DpFEIeY~^nZifjh@X\\fRng_Hx\t1.48\nHeuHADp@zIRYUTjFZj@@@FEHwCOHp\t1.48\nHklH@@RYWY^f`@jj@A`hIaUpki|bP\t1.48\nHcm@@@qJUvUpeuih@H@F@pa\\LkoHh\t1.48\nHcND@JCTeUe^UZjjj@F@iaMpsoDaP\t1.48\nHcLD@NwIIDcDbpuUUS@C@XSBiagdJ\t1.48\nHcMH@NVlddTZbR{UMTl@LFSBF]yC@\t1.48\nHcmH@DhDfUe]aWVjA@`@XTf]J|P]@\t1.48\nHcNH@IAIfUWEZ``b@A`lXUMprn|`@\t1.48\nHcNH`IDHaIe]mih@Ij@AapIaUJ|cP\t1.48\nHcMD@FFnrRQQQ]LMTsUP@pKIW^XSB\t1.48\nHcmH@DhDfUe]aWVjA@`@XDgRoLp]@\t1.48\nHmwH@Dp`RYyUeVjjj@FGIS\\LkrL@\t1.48\nHmvD@E@TfUvzZ@Bj@A`HIaUpsrE@\t1.48\nHmvH`IDHaIe]nf`@f`@XLBXUSrE@\t1.48\nHetH@@RYnWaFZjjZ@F@`aTwCODc@\t1.48\nHcLD@@QInYWWVjB@@A`JYS\\LkrM@\t1.48\nHmt@`@bDfUuZZ@Bj@AapIaUp|RU@\t1.48\nHiDLpB[`bDbJ|Dfunff`A`PI`|b`\t1.48\nHcND@LADf]YU]Zj@@@FGXWCJ{rJ@\t1.48\nHeV@PIBHzHRYeea`Ha@A`pYS\\OHd\t1.48\nHcND@EADfVU~UZjjj@FGES\\L{rL@\t1.48\nHmuH@DDDeYW^Ejjj`AarXUMt|RE@\t1.48\nHeVD`La@BLddlRPrm@@@C@linx`r\t1.48\nHct`@JF]rRQRzKUUSV@LFaLJfyD`\t1.48\nHcML@DpFEIeY}kfjYj`A`RXn|Si@\t1.48\nHmLD@@yIYe^neVdHB@A`SCJ|RB@\t1.48\nHmUDB@UiB\\ddUbtjji`AatIaUr`\t1.48\nHmv@@@RYvUeZj@@@FDpfDwCJ|`@\t1.48\nHeTH@NrRQHqLMUUT@LAaLJfF^Qp\t1.48\nHmv@@@RgVUeZj@@@FDaBXwCJ|``\t1.48\nHmvH@NAIYWUnZjjh@X\\fESCODaP\t1.48\nHmwH@DJPR[YWFVjjj@FCYS\\J|a@\t1.48\nHeVDB@ZPgIIEXdLuUUP@pzDpjxP\t1.48\nHmuD@DHNRY[]Jfjfj@FEXproDP`\t1.48\nHmuD@DHNRY[]Jfjfj@FALLksDp`\t1.48\nHeV@@@rRJIJmuUUT@LAaLInF^QP\t1.48\nHiEL`LZDh@cIHULeij@FFXp|PN@\t1.48\nHmvD@DCdfVyyUjB@@A`RMr|pad\t1.48\nHeUH@DpDeYUFVjjh@X\\aTwCOH`\t1.48\nHeUH@DHDfVuFVh@@@XLeMpsrB@\t1.48\nHmtH@@RYWYih@Jh@F@`fEWBoHd\t1.48\nHeT@`@bDfUuih@J`@X\\BXU\\OHd\t1.48\nHmVDB@FPgIIETijjih@X]BXULh\t1.48\nHef`@@SFyIeYfjZ`AapIeLL|c`\t1.48\nHieH@DDDeYWaZjj@FCIaTw\\RA@\t1.48\nHeUH@DDDeYWxVjjh@XLfES]qHd\t1.48\nHeU@@@EJYU^f```@FG@fES]qAd\t1.48\nHaD@P@bBbDfYvzB@@A`PDw\\SJ@\t1.48\nHieH@DDDeYWaZjj@FEXUMwLaA@\t1.48\nHeUH@DDDeYWxVjjh@XUaTw\\rDd\t1.48\nHeVH@FAIfUqfhH@@F@aFEWCOHX\t1.48\nHeVHB@XIrRQVICMUSR@LNaLKad\t1.48\nHeUH@DdDf^UFVjjh@XTUMp|qLd\t1.48\nHeUH@LDDeYWxVjjh@XLaTwSrH@\t1.48\nHeg@@JF\\ddTneUTp@pZDpj[bCH\t1.48\nHmW@@JE\\ddTnRjjfpA`tIaTwHx\t1.48\nHif@@@rRIHhjjjj@FGPaTwCrL@\t1.48\nHie@@@QIYUxVjj`AatXUMp|`@\t1.48\nHaEHADH@zIRYUJjij@FBMwDS@\t1.48\nHiFHB@DIrRQUJjjPA`tIaTwHx\t1.48\nHiD`@@SFRYUfjZ@FG@fTwCrB@\t1.48\nHif@`ABPRYWZZ@B`@XTfEW\\`P\t1.48\nHaF@`F@HRf_rjYj@FE@fEWD[`\t1.48\nHiE@@JGIIEKjji@FCPfES\\Sa@\t1.48\nHiE@@JGIIEKjjj@FCPfES\\Sa@\t1.48\nHeUD@FFnRjYeajfZ`AaF\\HfFP\t1.48\nHeV@@@RYyTYj`@@AaHPfTwCH\t1.48\nHifD@D@Tf[WaZjj@FAES]rL@\t1.48\nHaE@@@aJmUnjjh@XLfES]rH@\t1.48\nHieL@DpFEIeYkfjY`AaB\\SN@\t1.48\nHiFH@HGIIEXjji@FCPfEW\\a`\t1.48\nHeT@@DjU]k``b`@XMBESCqLd\t1.48\nHaF@@@ReU[jjj@FGPfES]r@@\t1.48\nHif@@@Ri]^Fjjh@X]BES\\OHp\t1.48\nHaEH@HSlddRmMUTp@prLKbIP\t1.48\nsNqBGIARFdKDYIIYUT@pQoHp\t1.48\nHieH@DHDeYWaZ@@@FEIS\\OH`\t1.48\nHieD@DHNRY[Rijih@XPwBSBD\t1.48\nHiEH@FzlddTZjfh@XTfDw\\`P\t1.48\nHaDH@@RVU[jjj@FCIaTw\\PH@\t1.48\nKA|Y@]BX@QddarVj`CCBinQp\t1.48\nHaE@@@yJe~fjjh@XMBXUMrN@\t1.48\nHid@@LdbRQk``R@AaTHUCqLX\t1.48\nHeT@@DjU]k``R`@XMBESSqLD\t1.48\nHaF@@@rRIHeUUS@CAhSBinQp\t1.48\nsNqBGIARFdKHYIIYUT@payF\t1.48\nHiFH@JAIUfZjh@XUaMp|RD@\t1.48\nHaFD@DAdfUjyjf`AabYWHx\t1.48\nKA|@@Uddbejj`CAhSBinQp\t1.48\nKAzP@MuYIHhzih@pILInQp\t1.48\nKAxH@UvRRJNjZ@LLSBxc\\\t1.48\nsNyiKE`D[lddjVj@XP|UX\t1.48\nsGPdE`DSpRYvj@XXF|Tp\t1.48\nsGQLLIAREdDfvj`FDgHP\t1.48\nsOpH@DISOkSK@LBEkqE`\t1.48\nsKXDD@b^BRvKU@LLL^R@\t1.48\nsOxA@IRjVnjj@XVKU|a@\t1.48\nsOq@@drm]UT@phQkxi`\t1.48\nsFxA@IRiSZj`FEbqoH@\t1.48\nsF|@@ldrfmU@LBl[r@\t1.48\nsGYHMQDIJmT`LLHnKD\t1.48\nsN|@@ldssUPCBaFodp\t1.48\nsGQhHlOAJmZhAaCqB`\t1.48\nsNy@LDeVjj@XTKF|TX\t1.48\nsGT@@deYj`FEBqoI@\t1.48\nsNx@JeTuUPCBqXodh\t1.48\nsNy@GTjfjf@XYX|c@\t1.48\nsJR@@lTfVhAa`mrD\t1.48\nsGY@DDf]j`FFQoI`\t1.48\nqCbHXRBdDfj@pdyL\t1.48\nsJ\\@ABeOL`LECqK@\t1.48\nsGX@@eMuT@phbqyJ\t1.48\nqCsCPKCevyIj`H\t1.48\nHaDD@@{IEDcSUUT@LJ`j[nHf@\t1.48\nQMhDRZAaGFP\t1.48\nHidH`ABHRYWZZ@BP@XTBXU\\c`\t1.48\nsFpH@DISNmLpC@aZ|a@\t1.48\nsGQ`@bdwMT@pROEj\t1.48\nqC`P@HRZhCBSfBp\t1.48\nqCr@XIKTA`hnHX\t1.48\nqCp@AIj`LDmqB@\t1.48\nqCp@AJZ`LEMqA@\t1.48\nJoCb@BvxTeMrkk]IwPPP@P@AaRDporC@\t1.47\nHcNB@JciwIIMEeKruUUU@CBXSBiae^Qh\t1.47\nHk_H@NtpRfumUaWZ@@@`@FAIcGqBd\t1.47\nHk_H@IWPRfunUatz@@H@@FAIcKqBL\t1.47\nHmVB@A@feIf[vjjj@FBaBXULLkrN@\t1.47\nHmwDAHHPRISdfyeVEjZjPAae]OHT\t1.47\nHmvL@JciRjyvyZjjj@F@pfESCOHT\t1.47\nHeWDAHHPRISdfyeXVijd@XIT|c`\t1.47\nHmO@@@rdigmZZUB@@@@FFYp|qIT\t1.47\nHeTB@HhYRjwUnjjj`A`tHS\\ODrP\t1.47\nHidL@HhTjmunjjj@FEPaLODs`\t1.47\nHiFL@F@nRYgZjj@F@aBXS\\OHh\t1.47\nHaE@@@sIEDceMST@LLPfxe\\\t1.47\nsNx`BJdrmUPCAbHu|f@\t1.47\nsOx@@eR}]UT@plqkyD\t1.47\nsGY@JDinj`FFbwDZ\t1.47\nsJY@DDeVhAaecrP\t1.47\nHg^L@M@iRYg^u[hHBjh@FBaaUprnwrC@\t1.47\nHidH`H@HRme^Eh@@@XTUMp|PD@\t1.47\nHid@`@bDfUvf`@h@FC@fEW\\`P\t1.47\nsFt@AdiWEihAahcrT\t1.46\nHaD@@DjUZxHH@AaTXU]qDX\t1.46\nsJ\\@@ldru@LDEqX`\t1.45\nHeTH@@rQQQHcMAP@@LFpj[axbR\t1.45\nHo}BAHLDUxeNR[e]m^Eh@JjP@X\\UMtj_Dvp\t1.44\nHkmLAHLDVISdfyW[xV`@j`@XLUMtkqML\t1.44\nJoBF@@HrS\\srzLv{PPD@@@FGISZ]orE@\t1.44\nHcLDrITJsjsZq^RYefVF@bF`@XXG[qLD\t1.44\nHkLNB@PfESSdf{n{Zjjj@FBxUMprnf\t1.44\nHclH@@rJJIPiMr{@P@@@CA`XYW^Ie@\t1.44\nHcMD@DHMR[nUyUj`@`@X]aMpsoHP\t1.44\nHeUD@BdARYgUnjjj`A`HPaTwSqLD\t1.44\nHif@PBBHzHRYgfFBB@@XTFMp|``\t1.44\nHeTL@@Z\\bbRbM]@DT@CA`SFz^PH\t1.44\nHmtL@@ZTieeqV`hH@A`rEproDi`\t1.44\nHmv@@@Ri^ueZj@@@F@pfMproHh\t1.44\nHeU@@@yJeWZzjjZ@FCPfES\\PY@\t1.44\nHeTD@NGIIDcCBUUMP@pJDpnHt`\t1.44\nHeVH@NAIYWVyjjj@FCIaTp|RI@\t1.44\nHieH@BDDfY}njjj@FGABEWCqLX\t1.44\nHmtH@@RfUWzZBBH@F@iaTwBoHd\t1.44\nHie@@@yJeWkjji`A`tIaTwDxP\t1.44\nHidD@NGIIDcHRjih@XEBXWDXP\t1.44\nHidH@@RUY^Ejjh@X\\fES\\OH`\t1.44\nHidD@F{IIEYhRjfh@XYFEqDD\t1.44\nHidD@@iJ[gxZB@@AaRDwCrJ@\t1.44\nHidH@@RfU~F``@@XMaTwCrD@\t1.44\nHaDH@NrRQHpeUS@C@hSBxeL\t1.44\nHaDD@F{IIEYBUTt@LLcBxb\\\t1.44\nsNyhMEPDYIBhmL@pfx`p\t1.44\nsOq@GddblUUL@ptqxeP\t1.44\nsFu@E`drfmU@LD[qK@\t1.44\nsKXA@IReqjhA`Xl^P@\t1.44\nsJY@DDeVhA`l^JH\t1.44\nsGX`DJdjmT@prMxi`\t1.44\nsFx@@eLzuU@LCEkq@`\t1.44\nHidH@@RYm^Fh@@@XLATwCqHp\t1.44\nHg]HpBXpkIkMkDfYemnVBHhf@AaPENwqFT\t1.43\nHknL@M@iRYg^un``Jj@AaHXU\\LkoHL\t1.43\nsGX`LDdrmT@pexlP\t1.43\nHeT@@DjU^k``b`@XCBXU\\L|a`\t1.43\nHeTD@@iJ[e^f```@FAHSCqNd\t1.43\nsFy@JDiTvjhA`Xl^R`\t1.43\nqCp@AIZ`LMEnP@\t1.42\nHaFH@JAJUtjjjh@XUBXS\\c`\t1.42\nHaDH@@RYWih@H@FFHU\\pLx\t1.42\nHcNLAHAEbTyInUvxV`@j@A`qTwRoHt\t1.41\nHiFDPJHPbDbDf{Vij`AaPPeCqFx\t1.41\nHeTH@@RYe\\YjB@@AatXU\\L|PF@\t1.41\nHidH`BBHRYeVFBB@@XDFTOD[`\t1.41\nHaDH`BBHRYg[hH@@FAAe]qBX\t1.41\nHeTH@@RYm_aZ@B@AaQTwSrL@\t1.41\nsOp@DjUMjj`FCDVMyF\t1.41\nsNx`DFdkKUPC@kE|Th\t1.41\nHif@@@RfU~F``@@XLFES]qDD\t1.4\nsGP`@TeYj`FFVOEJ\t1.4\nsNp`Jtjfjf@XYX|c@\t1.4\nsOq@@drm\\@@@prMyaf\t1.4\nHcNL@M@iRYg^vzB@j`@XBFEWCJ|cP\t1.39\nHiU@@@iJYWrnFP`H@FAESCqDx\t1.39\nKA|X@HC\\dkJuU@FEIaMwDb`\t1.39\nsJU@DPdju@LDmqP`\t1.39\nqCr@PIWTAaEqS@\t1.39\nqC`@ISTAxQE`\t1.39\nHcLHpMbPRPrPrRRqIPcMT@E@@pjfF]yB`\t1.38\nHk^D@IADf^UeFUujjjj`A`ITwBngrL@\t1.38\nHcnD@AADeYVWaeVjjjh@XCaTwRn|b@\t1.38\nHmtL@@Pdf{eYUj`@@AavDwCJ|a@\t1.38\nHmVDB@ZPgIIEXmJjjX@X]BXU\\h\t1.38\nHeTH@@RYWVf`@j@AapIaTL|c`\t1.38\nsGQ@@dsuT@pHVOEJ\t1.38\nsKX@@eKcUPCCSGbB@\t1.38\nqCb@AIj`LEEqC@\t1.38\nsFx@@eJfuU@LJIc^Q@\t1.37\nsGX@@eRuT@pLQoI`\t1.37\nHeVB@ICFEJYyzzjjj@FGPUMpsqDp\t1.36\nHeV@@@Re[TYj`@@AapES\\L|RD@\t1.36\nHifH@JAJUuJjjj`A`tIaMwHD\t1.36\nKAxD@@c\\dvluS@FBXWDS`\t1.36\nsGPa@erS\\uPCB[bM@\t1.36\nqCqPZHAD]XCqC@\t1.36\nsGP`@TfZj`FADMxe`\t1.35\nsJX@@eOU@LML^HH\t1.35\nsNx@IddbmUPCBqXodh\t1.34\nsGY@DDfyj`FBMxe`\t1.34\nQMB@HchFD\\Y@\t1.34\nsNq`@bdw[UPCCKGbk@\t1.33\nqCh@AIJtA``nJX\t1.33\nHif@@@RUe^Fh@@@XLaTwCq@`\t1.33\nHkoBAHePPcbTyInUuVyjZfe@FAUpkrM@\t1.32\nHmvD@NADfVyyUjB@@A`SCJ|Pn@\t1.32\nHeUD@DpFRUVTYZZZPA`REt|RA@\t1.32\nsOx@@eRimUT@plqkyD\t1.32\nsG]@EbDf]jPFBVyJ\t1.32\nqCh@AIJtA`inHH\t1.32\nqCh@BIWTA``nJX\t1.32\nHeTD@@QImeQej@@@FEIS\\ODrP\t1.32\nsJQhHl@bOV`FDOI`\t1.32\nHidD@@IIf][hHB@AaPXU]rA@\t1.32\nsOq@@drm\\@@@prKyaj\t1.32\nHmvD`Ia@BLdabRrxU`@h@FEES]ODAP\t1.32\nHidH@@RfU~F``@@XTfDw\\Ra@\t1.32\nqCr@XIJtAyhl\t1.32\nHknDrBtpkAkEkCyIf[effBBbX@FF@J|Pi@\t1.31\nsJ]@EbDfVhA`enP`\t1.31\nHif@@@RfU~F``@@XLBXS]qJD\t1.31\nHaD@P@bFbDfUjz@H@AaPIeMrN@\t1.31\nsJY@DDefhAaIqX`\t1.31\nHaD@@DjUZxHH@AaTHU]qLX\t1.31\nHg|L@@QdfygUQd{ZB@@@@FGISKiwrN@\t1.3\nsGX@@eSUT@ptQxmP\t1.3\nHeTD@@iIYe^e```@FAMpsqHH\t1.3\nqC`P@HRVhCBKfDp\t1.3\nsJU@DPdvu@LInID\t1.3\nqCqPZH^D]hCsAX\t1.28\nsNp`@df]Zj@XKWbK@\t1.28\nqCp@AJZ`LLHnS@\t1.28\n`Lg@^P\t1.28\nQMhDRVAaOJ@\t1.28\nHaFD@DCdfUriji`A`RXU\\a`\t1.27\nsGX`JBdruT@pHaoDl\t1.27\nsJX`LDdvu@LH^IX\t1.27\nHidH@@RfU~F``@@XEc\\OLPj@\t1.27\nsOt@AdigkB@@FAQkyL\t1.27\nsGXhMD@cIHTmPCB[fAh\t1.27\nRGDPDQ`|R@\t1.27\nqCb@AIj`LDEqS@\t1.27\nqC`@ISTA``nJX\t1.27\nHidD@@GHhdhZZ@B`@XTBTp|SN@\t1.26\nHg}H@DTDeUfUqT{ZjP`H@F@hWRng_Dc@\t1.26\nHcm@@@uJYU_rnf`Pbh@FG@aMpsqEt\t1.26\nHmtD@@iJ[eWihHH`@XTaLL|aP\t1.26\nqCr@XIKTA`enHP\t1.26\nHeTD@@iJ[g^F``H@FAHSCqJd\t1.26\nHmtD@@iJ[eWihHH`@XDaLODeP\t1.26\nqCr@HIJtAaIqK@\t1.26\nsOp@DjWkjj`FCDQkyL\t1.26\nsJX@@eMU@LLInID\t1.26\nsJXA@IczhA`hnKD\t1.26\nqC`@IVtA`anJH\t1.26\nQM`AIhFD|R@\t1.26\nsJY@DDfVhAaEqX`\t1.26\nsKPH@DIRxtl@pHfOH`\t1.25\nHaDD@@IIf]n``@@XTFES\\c`\t1.25\nHeT@pHdDdLdLddlRPsU@@@C@j[iy@`\t1.25\nsGQ@@druT@pHaoDl\t1.25\nQM`BN`XI\\d\t1.25\nQM`AIhFCOH@\t1.25\nQM`AIhGfR\t1.24\nsJY@LDefhAaEsSD\t1.23\nsJY@DDefhAyib\t1.23\nsOpHADILkW@@@LJDZ~S@\t1.22\nHaD@@DjUZxHH@A`TIa\\rfx\t1.22\nsOpHADILkW@@@LLE^[E`\t1.22\nHmvH@DAImYVUZh@@@XLS\\LkqDp\t1.22\nsOq@@drm\\@@@pQoBEf\t1.22\nqCr@HILtAaErX\t1.22\nsJXhMD@bOV`Gbq@\t1.22\nqCp@BOTA`hnS@\t1.22\nHaD@@DjUZxHH@A`THU\\sFx\t1.22\nqCr@HILtAxe`\t1.22\nHidH@@Rfufz`@`@XXfMsBxP\t1.21\nHaF@@@Ri_ijjj@FEPaTwDs`\t1.21\nsOx@@eJymUT@plVMx`p\t1.21\nqC`P@HRZhCAK\\P`\t1.21\nsJY@DDeVhAaMsQD\t1.21\nsOy@LDeekZj`FAVKxiP\t1.2\nsJXA@IRij`FAbqy@\t1.2\nqCa@AIMTA`inHH\t1.2\nHidH@@RfU~F``@@XUaMp|QJ@\t1.2\nsOr@AcTiek@`@GfQX\t1.2\nHidD@@iIYgxVB@@AaQTwCrH@\t1.2\nHaFH@NAIYUnX@@@XEaTwH`\t1.2\nsJQDBHaQBS]R@ppbyL\t1.2\nsGX@AddQjhAah[qK@\t1.2\nHmvDAHAHeNR[e]xV`@h@FAES]qIT\t1.19\nHmLD@@eIfUTfEV``R@A`U\\J|c@\t1.19\nHeTH@@Rfuunh@J@AaRYSCqBd\t1.19\nsGXA@IRijj@XFKGbE@\t1.19\nsJX@IGuPCCQ[dH\t1.19\nsOp@DjWkB@@FAfKxeP\t1.19\nRG@DXOD`\t1.19\nsJY@BDfZhAahmq@`\t1.18\nsGQ`@ldsmT@pQoI`\t1.18\nsGQ@@dsuT@pHaoDl\t1.18\nsJX@@eOU@LBIcrH\t1.18\nHmvLAHAEbTyInUwaZ@B`@XTUMr|bP\t1.18\nKAxUBPfEp^F\\WYIIbfjd@pbnPp\t1.18\nHif@@@Rge^Fh@@@X]BES\\OH@\t1.18\nHaF@B@BTjUZxHH@A`tIaUwHX\t1.18\nqCh@CISTA``nJX\t1.18\nsGU@E`dsmT@pROEj\t1.18\nsKQ@@eKcUPCCSGdP\t1.18\nqCr@PISRAxe`\t1.18\nHknL`IaM@HrRFIKISaV@Bf@A`qTwR_H@\t1.18\nHk^@@@RiU{Vntzjh@H@FBpfEPrngrN@\t1.18\nHifL@NBaReYrZjjh@XMaTwCqHP\t1.18\nHcLD@@kIEMDdcttDDT@CAdIae^Qh\t1.18\nsJY@LDejhAaaEqX`\t1.18\nsNp`@df]Zj@XKGbk@\t1.18\nqC`@ISTAaAsQX\t1.18\nJoba@FCBJzUSsZoSCiwUUTuMP@XTaR]|Rn@\t1.17\nHeTLB@QdyInYqehH@@FETwCH\t1.17\nHaF@`FBHRYVkh@`@FE@fUwHX\t1.17\nHifH@NAJ[VXZ@H@A`RXw\\Pa@\t1.17\nHaE@@@qJYVnBB@@XTfEW\\a`\t1.17\nsFtHLPDISNmLpCCAWdH\t1.17\nj`qaB@GBAHILkmroQegP@SUUD@CBRLDiIVcVyCp\t1.17\nJoCDPCbD`drnwJtuo@AMUQ@AaJXURng_H\\\t1.17\nHkoD`LDxP@cIIBhdmeuZ`PH@FGXwCJgrJ@\t1.17\nHg]L`LnDD@cIIBhhd]ikTC@P@LF[iW[yF@\t1.17\nHmvDRLa@BJbL{IIBidiUZ`@@AaUMpsrE@\t1.17\nHg^L@DAaRUf^uNvjj@@@FBiaTprngqHl\t1.17\nJoBD@@ILvnroC[m@A@@@@XRUMrz]orL@\t1.17\nHo\u007FH@DVpRYYU]YujBBJ`@XJaTwCIv\u007FHT\t1.17\nHo}D@ElDRYYU]YujBBJ`@XRUMpr]orE@\t1.17\nHcOD`LWXP@cIIKDedLkP@R@C@j[axfZ\t1.17\nHcND@FATfYev]jB``@FDaFEWCJ|Qm@\t1.17\nHcLD@NGIIDcDYBUULu@CChSBY]y@`\t1.17\nHeTHPBBlzlRYgea``b@AaPXWCrI@\t1.17\nHmtH@ArRQHiRDjjVh@X]BXURoHX\t1.17\nHeVH@IAIfu~Fh@H@FCAATw\\SI@\t1.17\nHidH`ABHRYWZZ@B`@XLBXUCrN@\t1.17\nHeV@@@RYyTYj`@@A`HIeMpsrH@\t1.17\nHeTD@@qIUeQej@@@FCIS\\L|b@\t1.17\nHid@@LddUTaJjZ`A`tXU\\OHP\t1.17\nHaFH@BAIf]n``@@XDFTwDK`\t1.17\nsNx@HeTuUPC@kU|a@\t1.17\nsKP@DjTZj@XDQGdp\t1.17\nsGX@@eSUT@pJqoI@\t1.17\nQO@HyjA`erP\t1.17\nHeTD@@QIemQej@@@FATwCqBd\t1.17\nQM`AIdGfR\t1.17\nRG@DXLJy@\t1.17\nj`qA@@@YHihhhdd]Nmzj@B@@@@XZ`s`iJtZso\\``\t1.16\nj`qa@@F@RYV{m^gKN`BJ@B@@FB`HqSbmFm{dI@\t1.16\nJgB@P@AFRJJJIIFad}n`@`@`@CBPIne]N~PH\t1.16\nHg]DrBX{BlFlVlodfYn\u007FnfBHbf@Aa`BoDI`\t1.16\nJoBD@@YEEEEDdbmf}P@P@@@FFqS\\Lkiv\u007FH@\t1.16\nJoBP@@XUYHhhceDdwVxEAP@@@FCEWBn|p{\\\t1.16\nHg^HpBLJrZsZqIfVYYe``jI`@XT@sm|SE@\t1.16\nHg|@@LdbbRLrSB]aDP@@@CATJfFUt{yF@\t1.16\nJoCD@C@drm|kuqw@DP@P@A`hIe]Jz[|a`\t1.16\nHg\u007F@@@qdigvU|kSZj`@h@FDiaTLkm|a`\t1.16\nHkmD@ELDrJIQIIF]Z``b@AaqTwCI|aP\t1.16\nHeWD`LjXP@cIHUDdLkP@@@pSFy`\\H\t1.16\nHmtD`NTHaIe]Vf`@jP@XLBXUCrI@\t1.16\nHeVD`La@BLddlRPrm@@@CCLj^XwB\t1.16\nHcLD@@QIeeUgVhHH@AaqTwCN|aP\t1.16\nHeTH`BBHRYeUa``b@A`PYP|Si@\t1.16\nHmvH@DAIge\\YZ`@`@XLfTwCrE@\t1.16\nHeVD@DBTeYWxV`@`@XLaTwSrH@\t1.16\nHeUD@LdDRY[TYZjj`A`UMp|Pi@\t1.16\nHidH`BBHRYgVzB@`@XDFUwDHP\t1.16\nHeTH@@RV[TYZ`@@A`qTwCODa@\t1.16\nHid@@LddRL[jjj`AatIaUp|a`\t1.16\nHeVD@FADfygFV``@@XEWCODQ`\t1.16\nHeVH@IAJ[WVz`@h@FCIaTp|bP\t1.16\nHiT@@DjUfa[``hH@X]BXULOHx\t1.16\nHieD@LDDR[e^Ejjh@XDUMwDPP\t1.16\nHidH@@RYm^Fh@@@XLDESCqLx\t1.16\nHieH@DXDfyWaZ@@@FAEWCrD@\t1.16\nHaDH@HRjw[jjj@FCPfTw\\``\t1.16\nHaF@@@RiUkjjj@FEPaTwDs`\t1.16\nHaD@@DjWzZjj`A`pPaUwDq`\t1.16\nKAxH@@bRjsUT@XUaTw\\b@\t1.16\nsOq@@drm]UT@pXVM_I@\t1.16\nqChPK@HRUhCAA\\Tp\t1.16\nsJYiCE`D[h}Z@^P`\t1.16\nsJX`LDdvu@LEcrH\t1.16\nsGX@@dk]T@pvMyD\t1.16\nsJPa@erS[T@|VH\t1.16\nHaDD@@IIf]n``@@XDFEwDi`\t1.15\nHidD@@IIf][hHB@AafUwL@a@\t1.15\nHiDHPHApiprRQVRjj`A`rXS\\H\t1.15\nsJ\\@@ldru@LLEnP`\t1.14\nsJQ@@dsU@LLEnJD\t1.14\nQMPARVAaGFP\t1.14\nqCb@AIZpLDMrP\t1.13\nHaD@P@bNbDfUZZ@B@AaPHS]rL@\t1.13\nHaD@@DjUZxHH@AadIgBJfx\t1.13\nqCp@AIj`LMEnP@\t1.13\nKA|P@HBS[KUT@XDUMwDS@\t1.12\nsKP@Di\\Zj@XDQ[dH\t1.12\nHidH@@Rfy^F`H@@XDU\\ODQ`\t1.11\nsNu@EPdrsUPCCKWbK@\t1.11\nHaDH@@RYe[hB@@FAXS]q@h\t1.11\nHaDH@@RVU[f@@@FFTw\\rLH\t1.11\nsGP@Divj`FFBoMLP\t1.1\nsKT@Adi\\Vf@XFKD\t1.1\nKAxD@Ae\\dmvuU@FEIaUwDa`\t1.1\nqCx@AYIf`LHNIX\t1.1\nHeTH@@rQSIPmu@AP@LBSNGfDt`\t1.09\nHidD@@EIYW[j@B@A`PXU\\rFD\t1.09\nsGX`LJdmuT@pHboEb\t1.09\nsJX@@eMU@LBHmrD\t1.09\nsJP@DkVhAaacqQ@\t1.09\nHeTD@@IIf]vzB@h@FCAaUp|bP\t1.09\nsKT@Adi\\Vf@XZH|f@\t1.09\nHid@P@kAkDfUff`@h@FA@cCqLx\t1.09\nqCx@AYIf`LDErX\t1.09\nHifH@DAIVUxV`@@AaRES]qHD\t1.09\nsJX@@eKU@LCEcr@\t1.09\nHaF@@@RYe[hB@@FEXUMwH@\t1.08\nsGX@@eNuT@phfMyD\t1.08\nHaD@P@qFqDfUjz@H@A`PIc\\Rn@\t1.07\nsKX@@eKcUPC@qX|`@\t1.07\nHif@@@Rge^Fh@@@XMATwCq@P\t1.07\nqCa@CIJtAaEqS@\t1.07\nqCb@AIV`OME`\t1.07\nHmtB@@zURVUgXV@Hh@FEXWRoH`\t1.06\nqCp@AJZ`LLEnR@\t1.06\nHidH@@Rfuvz`@`@XDfTwDHP\t1.06\nHg|@@DjYV}~T{`d@@@@FBic\\LkiwrJ@\t1.06\nHidH@@RfU~F``@@XEaMwLRa@\t1.06\nHidH@@RYWZZ@B`@XTBXU\\Sa@\t1.06\nsJP`@TeVhA`d^JX\t1.06\nHcNJ@MCNeTify^ajjjj@FBpaTwCJ{rL@\t1.05\nHmvB@EAaeJYY^njjjh@XSFES\\LkrD@\t1.05\nHk\\D@@iIYgU^GSXH@@@@XLURngqHX\t1.05\nHmLH@@RYVuiiV@@@@@XLE\\LkqDp\t1.05\nsFq@@eLzuU@LCEkrD\t1.05\nHaDH@@Rfu[j@@@FFIW\\sBX\t1.04\nsGU@E`dsmT@pQoDl\t1.04\nHaFH@NAIe^f`@`@XDfEWDC`\t1.04\nsGU@EPdrmT@pRoEb\t1.04\n`IJpCr@\t1.04\nqCp@AIZ`LHnXS@\t1.04\nQM@HzAyd`\t1.04\nsNx@@eJ}UPCBqZobA@\t1.03\nsNq`@fdvkUHCAX|Rh\t1.03\nsJX@@dkU@LMEnHD\t1.03\nqCp@AIZ`LEMqA@\t1.02\nsGPXHlPDYIHUj@XP|e@\t1.01\nsOx@@drm\\@@@pQoBEf\t1.01\nsGT`EPTfyi`GbU@\t1.01\nqCpB@RduPFFbwH@\t1.01\nHefDB@XPgIIETiUUL@LJaLJnQH\t1.01\nHidD@@iJ[gxZB@@A`RDp|Rn@\t1.01\nsNyA@qNRRJuU@LCEcqU`\t1.01\nsGU@EPdvmT@pRwDZ\t1\nsJX@@eOU@LBHmrD\t1\nHeVH@DAIYmQej@@@FETwCOD``\t1\nHeTD@@iJ[e^f```@FEHS\\OHd\t1\nsOpHADILkW@@@LBDWqZ`\t1\nHaDH@@RYWih@H@FA@aUqLx\t1\nHidH@@RYm^Eh@@@XDUMwDpP\t1\nsOpHAHILkW@@@LBl[rL\t1\nsFy@DDfTujhAaekrD\t0.99\nsKX@@eKcUPCCCGbb@\t0.99\nqCb@AIZ`LDMqQ@\t0.99\nHg|@`@bLbbbRLRS\\]pDD@@@CAPpfzUt{yE@\t0.98\nHklJ@@R]MIemYqUj@Hh@FETtz_HH\t0.98\nHeTH`ABHRYWZf`@f@AaPIaUrI@\t0.98\nHcLD@@eJ[V][j@Bj@AaRYp{qBT\t0.98\nHie@@@QInUxV`@@A`pHUMwHD\t0.98\nKA|P@HBSMkUT@XDUMwHp\t0.98\nsNx@@eLuUPCBbKWbK@\t0.98\nsNx@@eJmUPCAqXu|`@\t0.98\nqC`@IVtA`enHP\t0.98\nsNyABCJSOUU@LJHj~KD\t0.98\nsGQ`@bdwMT@prqyJ\t0.98\nJoBD@DIU\\vjzv[uP@@@@@X[EMprng[|``\t0.98\nHg|H@@RYfUWd}nh@@@@@XZATwCJz]|b@\t0.98\nHmTFB@PfETyIn{mjjj@FGXUMpkrI@\t0.97\nqCbPPABUm@XI[d@\t0.97\nKAxDD@S]wILsuUPAaPPaMqLh\t0.97\nsNx`JDdrkUPCAE|Vh\t0.97\nHaD@@DjUZxHD@AaTXU]qDX\t0.97\nHidH@@RUe^Fh@@@X\\DES\\OHP\t0.97\nHaDH@@Rfu[j@@@FAIeMqBx\t0.97\nsGX`DJdjmR@pbyib\t0.97\nHaD@P@kNkDfUZZ@B@A`PHw\\SB@\t0.96\nHiFDB@XPgIIETjji@FEPfEWHx\t0.96\nJoCF@F``iLkJ|jz]mUUUUP@XJUMpriv\u007FHp\t0.95\nHg^D@E@TfUvujZ@Bjj@AahHU\\LkiwrL@\t0.95\nHmtD@@eIfUTUjBB@@XBDES\\L|SE@\t0.95\nHmvD@LATenUqej@B@AarYS\\J|b@\t0.95\nHeVH`NdHaIe]ZZ@Bd@FC@fEW\\`P\t0.95\nHif@@@rRJEKajjj@FGPaTwCrL@\t0.95\nHmtD@@QImYVUZX@@@XTwCJ|qLH\t0.95\nHeTH@@RfUWihHH@AaVES]sDFP\t0.95\nHifH@AAIe]ih@J@A`pIaTOHx\t0.95\nHidD@@iIYgxVB@@A`QTw\\RA@\t0.95\nHaFD@DCdeeVyjj`A`VES\\b@\t0.95\nsOp@DiUkjj`FCDVMyF\t0.95\nsJPhH`DYIBm@LEcr@\t0.95\nHeTH@@rQSIPmu@AP@LJSFx^HT`\t0.95\nHeT@pHdDdLdLddlRPsU@@@CCJF^Hw@\t0.95\nKA|PD@pSddbljj`CAhSBinQp\t0.95\nHaDH@@RYe[hB@@FAAaMqJx\t0.95\nsJY@LDeZhAaaMqH`\t0.95\nHcNH@DAIevUWVj`@@AarUprn|SB@\t0.95\nsGP@DiUiPFADVxcP\t0.95\nsJT@@TizhAaaMqH`\t0.95\nqCr@HIJtAyhl\t0.95\nsOx@@drm\\@@@pJq_DJ\t0.94\nsGQ@@dsUT@ppQyij\t0.94\nsGU@DPdwMT@pVODj\t0.94\nqCp@AIZ`LInXQ@\t0.94\nJoB@@BUJssoPvxPTA@@@FFpaTLkiv\u007FDs`\t0.93\nHidHPBBlzlRYgfFBB@@XDF\\ODI`\t0.93\nHmvH@AAIYWYnX@J`@XMaTtkrH@\t0.93\nHidL@@IdfYoa`b@@FE@UMwHD\t0.93\nsFt@ATiTvjhA`qEcP\t0.93\nsNy@DDeeZj@XGFod`\t0.93\nsOx@@drm\\@@@prMyaf\t0.93\nsGP`@dfuj`FBVxcP\t0.93\nHaDH@@Rfu[j@@@FAXUMqDx\t0.93\nsGY@LDeVj`FABqxiP\t0.92\nHcND`La@BLddJfRtjmP@P@LFqnF]yE@\t0.92\nsGU@EPdrmT@pRwDZ\t0.92\nsGQ@@djmT@pruxhP\t0.92\nqCp@AJZ`LInXQ@\t0.92\nqC`@IRtAaAsQX\t0.92\nqCbPXABRt`^XQ@\t0.91\nqC`HBHTPduPFFDWI`\t0.91\nHid@@DjU^nBBH@FCPaUp|SF@\t0.91\nQMPARZAaGFP\t0.91\n`IH`CB@\t0.91\nQM`BN`XQqd\t0.91\nKAzH@IP\\dvjuT`FFIe\\Pf@\t0.9\nsNqaIV~RjzjXAaIqX`\t0.9\nsGY@DDeUj`FBQyij\t0.9\nsJY@DDeVhAaEsSD\t0.9\nsJY@LDejhAa`cqS@\t0.89\nHidH@@Rfuvz`@`@XEaTwDPP\t0.89\nsJQ@@eMU@LEEqX`\t0.89\nsJQ@@dsU@LLHnKD\t0.89\nsJQ@@dju@LH^ZK@\t0.89\nHiDJB@PfENR[nvjj@FEXULOHx\t0.88\nHidH@@RfU~F``@@XDfDwLka@\t0.88\nHaD@`@dDfYVz@`@A`VDw\\b`\t0.88\nKA|H@TBILlmUPAafEwDi`\t0.88\nsNp`@dfUZf@XHobu@\t0.88\nqCp@AIj`LLEnR@\t0.88\nHeV@@@rRHqICMT@@@LFbfxYxbX\t0.88\nHmvH@LAI[YVUZh@@@X\\aMproDc@\t0.88\nsKXHB@aJWFe`FFDwH`\t0.87\nHidD@@iIYgxVB@@A`QMp|RJ@\t0.87\nsOpHADILkW@@@LBEWqX`\t0.87\nHmtD@@UIUe\\YZ`@`@XMeMpkqHP\t0.87\nsJX@@eKU@LLL^JH\t0.87\nsJQ@@dsU@LLH^IX\t0.87\nHkLNCHiiWS[ex~rRSQHqRjZfX@XYFD\t0.86\nHmn@@HRjuYX^gejjjj`A`lIaTwCJ`\t0.86\nHedJA@RaeNgdfvvZjf@FFYS\\``\t0.86\nHefDB@XPgIIETiUUL@LJaLJ^QP\t0.86\nHif@@@Rge^Fh@@@XLDES]qDD\t0.86\nsGPBIHJPtaXcIIKUPCBWdH\t0.86\nsNq@GddbmUHC@qX|e@\t0.86\nsGY@LDefj`FABqxiP\t0.86\nsNx@HeTuUPCCKGbk@\t0.86\nsNp`@dfvZj@XGFodP\t0.86\nsGX@HeTuT@prqxiP\t0.86\nsJPH@DILuPCAQ\\pq@\t0.86\nsJY@BDfZhAaaMqH`\t0.86\nsNp`@dfUZj@XHwbs@\t0.86\nsJQ@@dmU@LLEnJD\t0.86\nsJX@@dkU@LCEcr@\t0.85\nsJXhMD@cIHUhAyab\t0.85\nsJT@@TizhA`h^XK@\t0.84\nqCh@BIWLAaMqQ@\t0.84\nqCj@KAIf`LHnS@\t0.84\nHmN@@@RfUWrnf`Pb`@X\\BDwCOHT\t0.84\nsOq@@drm\\@@@pHQoEf\t0.84\nsGQ`@jdjmT@pVOEJ\t0.83\nsGQLKIARFdLddmU@LH^R`\t0.83\nsJXhMD@cIHUhAaCqC@\t0.83\nHaF@@@Rfu[j@@@FEAaTwDS`\t0.83\nsJQ@@dmU@LLl^HP\t0.82\nsJU@DPdvu@LH^IX\t0.82\nsJY@DDfvhAaCqK@\t0.82\nsJP`@TeVdAyhl\t0.82\nHcLJ@@PUuInUgzV`BJ@AaQTJ{rD@\t0.81\nsOxA@IRi^lH@@XVHu|f@\t0.81\nsOpHADILkW@@@LBCWqK@\t0.81\nsNp`@deVZj@X[U|TH\t0.81\nsNp`@deYZj@X[F|TX\t0.81\nsJX@@eKU@LML^HH\t0.81\nHaDH@@RYVih@H@FE@fTwHx\t0.81\nsFx@@eLzuU@LBIkqH`\t0.81\nsJQ@@dlu@LDmqP`\t0.81\nqCr@XIKTA``nJX\t0.81\nsJQ@@djs@LDmqP`\t0.81\nqCr@HIJtAaEqS@\t0.81\nHmTJ@@IaUIf[njjj@FBaBXUMpkrI@\t0.8\nHedJ@@IaeIf[zjj`AaHPfESCOHx\t0.8\nsGXhMD@cIHXmPCs@t\t0.8\nHeV@@@Re[TYj`@@AatIS\\L|PB@\t0.8\nqCb@AIf`ODl\t0.8\nHedJAHiaenglddtUeTtp@pRLyA@\t0.79\nsJY@DDeVhA`inID\t0.79\nsJX@@dku@LInID\t0.79\nsFp@DjTvjhAaQD[rX\t0.79\nsGP`@deYj`FBqxiP\t0.79\nHiF@@@RifZjj@FGPfES]rA@\t0.79\nqCa@CIJtAyhl\t0.79\nsFp`@dfTujXA`l^R`\t0.78\nsNp`@dfyZj@XEXodh\t0.78\nsGP`@dfyj`FFVOIP\t0.78\n`IHPCr@\t0.78\nsG\\@@ldsmT@pHVOIP\t0.78\nHaDH@@RVU[j@@@FAAaUsH[`\t0.78\nQMhDRVAxr\t0.78\nHeTHPBBlzlRYgea``b@A`PYp|Pi@\t0.77\nHmtD@@QIee^UZ``@@XLULLkrN@\t0.77\nsGQa@bnR[fj@XP|Vh\t0.77\nQO@HyjAaGFP\t0.77\nsGX@@eNuT@pLqoDD\t0.77\nsNq`@fdvkUPCCKObM@\t0.77\nsOx@@drm\\@@@phVMyF\t0.77\nsJX`DBdjt`LHnJX\t0.77\nKAxD@@c\\dvluS@FFIg\\``\t0.76\nHaDD@@IIf]n``@@XYeMs@K`\t0.76\nsOp@DjWkB@@FBaxUjp\t0.76\nHaDH@@RVU[j@@@FCAaTw\\b@\t0.76\nsJX`DBdvt`ODl\t0.75\nHidH@@RVUvy`@`@XUaTp|b@\t0.75\nqCp@AJZ`LLInQ@\t0.75\nsOp`Hdjekjj`FBqxjp\t0.75\nqCb@AIVPLHnJX\t0.75\nHeTB@@sirQQSRtMUUT@LNaB[agbYP\t0.74\nsGQ`@jdvmT@pQoDl\t0.74\nsNx`DJdjsUPC@hu|Tp\t0.74\nsOq@@drm\\@@@pHU_Eb\t0.74\nHidD@@QInUxV`@@A`QTw\\QA@\t0.74\nHaDH@@Rfu[j@@@FEXUMwHP\t0.74\nsOq@@drm\\@@@pRoBLV\t0.74\nHaD@@DjUZxHH@A`tIaUwHX\t0.74\nHeVH@AAIYWVy`@h@FEXULODbP\t0.73\nsNx@AddQUUPC@pu|Rp\t0.73\nsJP`@TeVhA`l^JH\t0.73\nHmt@pHdDdLdLddlRTFZh@H@FATpsrE@\t0.73\nsJT@@deVhAa`mqP`\t0.73\nsOq@@drm\\@@@pROBMV\t0.73\nqCr@XIKTA`aNIX\t0.73\nHidHPBBlzlRYgfFBB@@XTFEp|a`\t0.72\nHiDJA@PnENGdf{Yji`A`aUrL@\t0.72\nHeUH@DdDfyWxV`@`@XDUMwDRP\t0.72\nsJY@LDeZhA`QEnP`\t0.72\nsNqa@jnR[YjhA`m^Hl\t0.72\nsJY@LDeZhAaaEqX`\t0.72\nsOp@DjWkB@@FFbOMjp\t0.72\nqC`@ISTAaEsBX\t0.72\nsKP`Adi\\Zj@^ZH`\t0.71\nHidH@@RYWZZ@B`@XLBXU]rA@\t0.71\nsJQ`@bdvu@LDcrX\t0.71\nQM`BN`XL|`\t0.71\nqCb@AIZ`LDEqS@\t0.71\nKA|X@TB\\dvjuT`FFIe\\Pf@\t0.7\nsGY@DDeej`FBfxcP\t0.7\nsGXA@IRiZj@XVKF|`@\t0.7\nsJQhHl@cIHUhAaCr@\t0.7\nsJP`@TfZhAaacqQ@\t0.69\nHk\\H@@RfYU\\]Tz@@@@@F@xUMproDtp\t0.69\nHidD@@QInUxZ`@@AaQTwCs@a@\t0.69\nsOx@@eJ~MKT@ppeydV\t0.69\nsJY@DDfvhA`enP`\t0.69\nsGQ`@jdvmT@pRoEb\t0.69\nsGY@LDeZj`FFDWDZ\t0.69\nHaF@@@RVU[j@@@FEXUMwDB@\t0.69\nsGP`@df]j`FDwDZ\t0.69\nsOq@@drm\\@@@phVKyJ\t0.67\nHev@@@RfUWJzZABH@FC@aMt|`P\t0.67\nsJQ@@dju@LInZH`\t0.67\n`L@OH\t0.67\nqCp@AIZPLMEnP@\t0.66\nsJQ@@dsU@LDmsBD\t0.66\nsJP`@TizhA`h^IX\t0.66\nHaDH@@RYWih@H@FE@fEWHx\t0.66\nHaD@@DjUZxHH@AaTIaUqFx\t0.66\nqC`HBHTPduHFBByL\t0.65\nsJPXHlPDQzt@pfyB\t0.65\nsJU@DPdvu@LHnKD\t0.65\nqCbPXABRt`XR\\PP\t0.65\nHidH@@RfU~F``@@XTfMp|Pj@\t0.65\nsJP@DiVhA``nZX`\t0.65\nqCb@AIVPOME`\t0.65\nsJQ@@dkU@LEMsAD\t0.64\nsGP`@dfUj`FDoEb\t0.64\nsGY@DDfUj`FBVxcP\t0.64\nsGY@DDfuj`FDoLlP\t0.64\nsGX`BJdruT@pHboEb\t0.63\nHidDB@QNR[e^Eh@@@XDS\\OHh\t0.63\nsNu@DpdvkUPCCKGbk@\t0.63\nsJT@AdiZhA``^ZK@\t0.63\nsGQa@jnR[Vj@XKWdH\t0.63\nsGP`@dfuj`FBMxe`\t0.63\nsGP`@df]j`FBqyJ\t0.62\nHeTH@@RfU\u007FahHB@AaRXS]qJd\t0.62\nHaDH@@Rfu[j@@@FAYS]qDH\t0.62\nRG@DTLHyP\t0.62\nsGQ@@druT@pHbwDZ\t0.62\nsJPXHlPDQzt@pVH\t0.62\nHid@P@bNbDfYYa`H`@FEAc\\OHh\t0.61\nsOq@@drm\\@@@pqky`l\t0.61\nqCaPQ@HRmhCB[d@\t0.61\nQM@HvAyd`\t0.61\nHifH@AAIYW[f@B@A`VES\\RA@\t0.6\nsFx@@eJxtm@LDIsHt\t0.6\nsOq@@drm\\@@@pJM_DL\t0.6\nHmtB`HSBCprRSFIJUZh@@@XMaMpsrE@\t0.59\nHeTDPAdHc`aIe]jZ@BT@FE@fEWHx\t0.59\nHeV@@@rRJEK\\MP@P@LJJfz^Hf@\t0.59\nHif@@@Rfuvz`@`@XUaTp|PD@\t0.59\nqCcPQXPd]hCAK\\`@\t0.59\nsGU@DPdwMT@pVoDb\t0.59\nsGY@DDfuj`FFVOIP\t0.59\nsJP@DizhAa`mqP`\t0.58\nsGP`ATeUfPFBVxhP\t0.58\nqCp@AJZ`LDIqK@\t0.58\nHaDH@@RYVih@H@FFIS\\pLx\t0.57\nQMPARVA`cr`\t0.57\nHidH@@Rge^Fh@@@XTUMp|RD@\t0.57\nsOq@@drm\\@@@ppSylZ\t0.56\nHmvH@DAIUYVUZh@@@X\\aMproDc@\t0.56\nQMPARZAyd`\t0.56\nHmvH@FCHhhhdYVhJ@@AaHQaUproHX\t0.55\nHiF@@@ReUZjj@F@pfES\\OH@\t0.55\nHid@@LdbRQk``b@AaTHUCqLX\t0.55\nsNxA@IrRQVjhA`Xl^Jl\t0.55\nHiV@@@RfU|kahDB@A`pHS\\OHh\t0.54\nsFx@@eJf`@@LCL[rH\t0.54\nsFx@@eJf`@@LMM^ID\t0.54\nHaDH@@Rfu[j@@@FAIaUqFx\t0.54\nHaDH@@RYWih@H@FAIaUq@x\t0.54\nHifH@JAIYgxVB@@A`QMp|RJ@\t0.53\nsGYALMJRmUPC@aF|Tp\t0.53\nqCp@AIj`LDEqS@\t0.53\nsN|@@ldskUPCBaXwdX\t0.53\nHid@@DjU^nBBH@FGPfES]rA@\t0.53\nHaF@@@RVU[j@@@FCQaTw\\`@\t0.53\nsGQ@@druT@pHboEb\t0.52\nHidH@@Rge^Fh@@@XDUMwLaA@\t0.52\nsJP@DjYdA`inID\t0.51\nHifH@DAInUxV`@@A`QTw\\QA@\t0.51\nsGU@E`drmT@pbylZ\t0.51\nsOq@@drm\\@@@pJqoDF\t0.51\nKAxD@Ae\\dmvuT`FAIaUqHX\t0.5\nsGP`@dfuj`FBUxlP\t0.5\nqC`P@HRfhCAB\\Rp\t0.5\nsOpH@DILkW@@@LBl[qA`\t0.5\nqC`@ISTA`anJH\t0.5\nsGY@DDfUj`FBqxeP\t0.49\nHiDB@@IaRYfzjj@F@aBXULOHx\t0.49\nKAxD@@S\\dsOUU@FGABXS]rJ@\t0.49\nsJP@DiYXAaaEqX`\t0.49\nsJQ@@dru@LHnKD\t0.49\nqCa@CIJtAaMqQ@\t0.49\nHidD`HP@cIHXdnEX@@@XDUMwD@P\t0.49\nHaDH@@RYe[hB@@FEAaMwHh\t0.48\nsJU@E`dru@LHnKD\t0.48\nsGP`@df]j`FDOEj\t0.48\nKAxD@@c\\dvluS@FBIwDp`\t0.47\nsGX@@dkUT@pJqoDH\t0.47\nqCa@CIKTA``nJX\t0.47\nsNu@DPdwKUPCCXwbS@\t0.46\nHif@@@RUe^Fh@@@X\\BES\\OH`\t0.46\nsGP`ATeejPFDWMLP\t0.46\nRFD@QFCBnP\t0.46\nsOq@@drm\\@@@phVMyF\t0.46\nQM@HzAaWDP\t0.45\nQMBBHRYAaGFP\t0.44\nsOq@@drm\\@@@prqyaV\t0.44\nsGQ@@djmT@pVoMDP\t0.44\nqCaBAKdfZ@|f@\t0.44\nRFD@QFCqH\t0.44\nsJY@DDfVhAaCqK@\t0.43\nQMPARVAyd`\t0.43\nqC`@Qz`LDIqK@\t0.43\nHmv@B@B\\ddUbReZj@@@F@pfMproHH\t0.42\nsNy@LDeYjj@XDQGbk@\t0.42\nHaD@@DjUZxHD@AaTHU]qLX\t0.42\nHaDH@@RVU[j@@@FEXUMwDB@\t0.42\nqCr@XIKTAaAsQX\t0.42\nqCb@AIZ`LHnXS@\t0.42\nqCb@AIZ`LHNZK@\t0.42\nsOx@@eLmXD@@pJu_HP\t0.41\nsGY@DDfUj`FBUxlP\t0.41\nsGY@LDeVj`FADQxmP\t0.41\nsGT`EaTf]jPFDWEb\t0.41\nHaDH@@RVU[j@@@FEAaTwDc`\t0.41\nsOp@DjWkB@@FBbx\\Zp\t0.41\nsJP`@dfVhAaMrD\t0.41\nqC`@ISJA`aNIX\t0.41\nRG@DTOD`\t0.41\nsOp@DjWkB@@FAbMxlp\t0.41\nHidH@@RUe^Eh@@@XLaTwCrH@\t0.4\nsOq@@drm\\@@@pJq_DJ\t0.4\nQMhDRVAaWI@\t0.4\nsJPa@erS[T@pfyB\t0.4\nHeTH@@RUYTYZ`@@A`vTwCODa@\t0.4\nHidH@@Rfufz`@`@XDfMwDHP\t0.39\nHiD@@LddUdjjh@XMBXS]rA@\t0.39\nsJQ@@dkU@LLEnJD\t0.39\nsJP`@dfvhAyeb\t0.39\nsGY@DDeej`FDOMMP\t0.38\nqCa@AIMTAaIsAX\t0.38\nHaDD@@IIf]n``@@XEeMwD@`\t0.37\nsOp@DjWkB@@FEbM_I`\t0.37\nRF@QP|R@\t0.37\nsJY@DDefhA`l^R@\t0.36\nqCb@AIj`LINXK@\t0.36\nQMHAIhFD\\Y@\t0.36\n`Ib`hFD\t0.35\nHg|@@DjU_eZx{BAH@@@FBpaUtkiwqLX\t0.35\nsNp`@df^Zj@XKGbk@\t0.35\nHaDH@@RVU[j@@@FAXUMs@c`\t0.35\nQMHAIXGcH\t0.35\nqCaBC[dej@p`xh`\t0.34\nsJY@DDfVhA`enP`\t0.34\nKAxP`PCaSaddbljj`CBdpf{dT\t0.34\nHid@@DjU^nBBH@FCPfEW\\Qa@\t0.34\nHaDH@@RVU[f@@@FAXUMqHx\t0.34\nqCpB@SGj@pTWDL\t0.34\nHk\\@@DjU^ukmLHH@@@A`lHU\\Lki|c@\t0.34\nqCb@AJZ`OME`\t0.34\n`IlQpFD\t0.33\nHg|@@LdbRbtJUB]aAP@@@CAXPjFUt{xf\\\t0.33\nsFp@DiTt@@AaaCsKT\t0.33\nRFDDQFCBNT\t0.33\nsJP@DizhA`Pl^R@\t0.31\nsJP`@dfvhAaCqK@\t0.31\nqCb@AIj`LDIqK@\t0.31\nHidD@@iJ[gxZB@@A`QMp|QJ@\t0.3\nKA|P@HBRjsUT@XTaTw\\RL@\t0.3\nHeVD`Ia@BLdabRwBl@D@C@binx`R\t0.3\nsGP`@dfUj`FDOEj\t0.3\nsFp@DiTt@@A`QC^IX\t0.3\nsJPYHlPDGh}Z@^S@\t0.29\nqCp@AI^`ODl\t0.29\nsFp@DiTt@@A`anFHt\t0.29\nQM`AIXGfR\t0.29\nQOHBNZ`^L`\t0.28\nHidD@@QInUxV`@@AaQTwCrD@\t0.28\nsKX@@eKcRpCAB\\rq@\t0.28\nKAxD@@S\\dsOUU@FCABDw\\SJ@\t0.28\nsJP@DiZhAaaCqK@\t0.28\nQMPARVAaOD`\t0.27\nQM@H~AaWI@\t0.27\nsJQ@@dlu@LEcqQ@\t0.26\nsJP`AdizhAyhb\t0.26\nHid@@DjU^nBBH@FCPfES\\Sa@\t0.25\nHeTD@@QIemQej@@@FETwCOHH\t0.25\nqCr@HILtAaMrH\t0.25\nHid@@LdbRQk``b@AaTHUMqLD\t0.24\nsNq`AjdmsUPC@kF|TX\t0.24\nsGQ`AjdmmT@pruxhP\t0.24\nsGU@E`drmT@pRwDZ\t0.24\nsFp@DiTt@@A`a^FKD\t0.24\nQMHAIhGfR\t0.24\nsJY@DDeVhA`enJD\t0.23\nQMHBN`XUrP\t0.23\nqCa@CISTAyhl\t0.22\nQM@HvAaGFP\t0.22\nsJX@IGuPCAK\\TH\t0.21\nqC`HBHtPdmPFBFxh`\t0.21\nsGX`DJdjmR@pRwEB\t0.2\nqCj@KAIf`ODl\t0.2\nHaDH@@Rfu[j@@@FFXU\\qFx\t0.19\nHaF@@@RYWih@H@FE@fEWHx\t0.19\nsJQ@@eOU@LECqK@\t0.19\nqCr@HIJtAaMqQ@\t0.19\nRF@Q`|R@\t0.19\nsOq@@drm\\@@@pjqky@\t0.18\nQMDa^RVAyP\t0.18\nqC`@IRtA`aNIX\t0.18\nRFPDTNJlNT\t0.18\nRFPDTOJ@\t0.18\nQM`AIhFBOJ@\t0.17\nHidH@@RYm^Fh@@@X\\DES\\OHp\t0.17\nsJY@DDefhAaMqP`\t0.17\nsOx@@drm\\@@@pjqky@\t0.17\nqCcPQXAD]XCB[d@\t0.15\nQMFI@bM`XSr@\t0.14\nsFp@DiTt@@AaaEsXt\t0.13\nRFPDXLJy@\t0.11\nRFQAtXOJ@\t0.09\nRFPDXLHyP\t0.09\nqCb@AIV`LInJH\t0.08\n`H@^P\t0.08\nQMDARZAyd`\t0.05\n`I@`Cr@\t0.05\nsJXA@IczhAahmrD\t0.03\nsJP`@deVhAaMqP`\t0.03\nsJP`@TizhAaCsQX\t0.01\nsFp@DiTt@@A`QEnHt\t-0.02\nQMHBN`^L`\t-0.02\nqCa@CIKTAaAsQX\t-0.03\nQM`AIhFD\\Y@\t-0.04\nsGQ`@ZdruT@pHaoDl\t-0.05\nRG@DXLHyP\t-0.05\nsFp@DiTvjhAaacsQT\t-0.06\nRGAALXLJy@\t-0.06\nsJU@E`dru@OLlP\t-0.08\nHo|H@NrRQRqZZR]j``bh@FDiaMvg[|bP\t-0.09\nRFPDXOD`\t-0.09\nsOp@DjWkB@@FBexUbp\t-0.1\nqCb@AJZ`LHnJX\t-0.1\n`J@OH\t-0.1\nsJQ@@eKU@LECqK@\t-0.11\nsJX`BDdvu@LHnKD\t-0.11\nHk\\@@DjU^ukmLHH@@@AaLHU\\LkoDpp\t-0.11\nsFp@DiTvjhAaQEcrT\t-0.11\nsJP@DjZhAaaEqX`\t-0.11\nsJP`@dfVhAxlP\t-0.11\nqCb@AIZ`LINXK@\t-0.13\nsJPXHlQxQ{T@prq@\t-0.14\nsJP`@TeZhAaaCqK@\t-0.14\nsNq`@VdsMUPCBbHobu@\t-0.14\nQMDARZAaWDP\t-0.15\nHidB@@saRf]ZZjjh@XUEMp|qLH\t-0.15\nQMHAIXFD|h\t-0.15\nsGT@@Tivj`FADqxeP\t-0.16\nsJP@DjVhAahmrD\t-0.16\nqCa@CIKTA`anJH\t-0.16\nsFp@DiTt@@AaQEcrT\t-0.16\nsJP`@TfZhA`QD^S@\t-0.18\nsJPLIDJHtQGuPCCQ[dH\t-0.18\nsFp@DiTt@@A`QE^KD\t-0.18\nsJX@@eMU@LML^HH\t-0.19\nKA|@D@DyIHjjjh@pZDpj[d\\\t-0.19\nqCr@XIJtAaEqS@\t-0.19\nHaDH@@Rfu[j@@@FEIaTwHx\t-0.2\nqCp@AIZ`LDEqS@\t-0.2\nsOx@@drm\\@@@pJqoDF\t-0.2\nRF@Q`pce@\t-0.2\nHcl@@DjYU_egX@@@@@XRfES\\LkrM@\t-0.21\nsGP`@dfyj`FBVxcP\t-0.21\nqCa@AIJtAxi`\t-0.21\nsJX@@dmU@LLl^HP\t-0.22\nQMBBHRZAaGFP\t-0.22\nsGP@LdbKU@LCEcrT\t-0.23\nHaDH@@RVU[f@@@FEXUMwH`\t-0.23\nsNxA@IrRQVjhAaXl[rL\t-0.24\nQM@HvAaOD`\t-0.24\nsOp@DjWkB@@FAakxe`\t-0.24\nsGP@DiVj`FADUxlP\t-0.25\nqCb@AJZ`LInJH\t-0.25\nHaF@@@Rfu[jjj@FGAFES]rD@\t-0.27\nsNyA@qNRRJuU@LKEc^Q`\t-0.27\nKAx@@YIDTjjh@pxHSBknPp\t-0.28\nsFx@@eJf`@@LCEcrT\t-0.28\nqC`@ISTAa`mrP\t-0.28\nHaDH@@RYWih@H@FF@U\\qLx\t-0.28\nqCa@CIKTAaaMrH\t-0.28\nqCb@AIV`LHnJX\t-0.28\nHaF@@@RYWih@H@FAIaUq@x\t-0.29\nsJQ`@ldru@OEb\t-0.3\nsOx@@eJyh@P@pLVOHp\t-0.32\nHaD@@DjWzXHB@AaTIaUrN@\t-0.32\nKA|@@@eSMUU@FCPaTw\\c@\t-0.32\nsJP`@TeZhA`inXH`\t-0.32\nHmt@pHdDdLdLddlRVFZh@H@FATwCrE@\t-0.33\nHg\\@@LddTQRdTwUAAT@@pULJfFUt~QX\t-0.33\nHiF@B@B\\ddURjjh@X]BXUMwHD\t-0.33\nsJP@DiVhAa`mqP`\t-0.33\nQMABHXaIhFBOJ@\t-0.33\nsJP`@TfZhAaaCqK@\t-0.33\nsGP@Dj}j`FFVOEJ\t-0.34\nsJX`BDdvu@OLlP\t-0.34\nHeT@pHdDdLdLddlRPsU@@@C@j[ayD`\t-0.34\nHedHXFbPRPrPzPFPfPrRRqRUUU@CBByC@\t-0.34\nqCpB@ReMPFBfx``\t-0.34\nsFp@DiTt@@A`aNEXt\t-0.34\nHaDH@@RVU[f@@@FFXw\\rBh\t-0.35\nHmtJ@@siWIEDeDZFjjj`AatEWCJ|QF@\t-0.35\nqCa@AIMTAaMsBH\t-0.35\nsJX@@eKU@LLHnKD\t-0.35\nsGP`ATf^j`FADMxe`\t-0.36\nqCp@AIj`LDIqK@\t-0.36\nsGP`ATfVj`FADUxlP\t-0.37\nqCpPXABRu@XJKbF@\t-0.37\nRFB@QADXOD`\t-0.37\nHk\\@@DjU^ukmLHH@@@A`LHUCJ{sLxp\t-0.37\nsJQ@@dju@LDmqP`\t-0.37\nRFTPDQP|`\t-0.37\nQM@HzAaGFP\t-0.37\nQMHAIdFE\\Q@\t-0.38\nsJX`DBdvu@LH^IX\t-0.38\nqCb@AIZ`LEMqA@\t-0.38\nsNp`@dfUZj@XHobu@\t-0.38\nQM``eIhFCOH@\t-0.38\nQMFI@bMP^P\t-0.38\nHaD@@DjUZxHD@AaTIaUqFx\t-0.39\nsNp@LdbJjj@XVKF|PX\t-0.39\nsJP@DjvhA`enJD\t-0.39\nHaD@@DjUZxHH@A`tHUMwHp\t-0.39\nQMHAIhFBWI@\t-0.39\nsJPhHaxIRuPCAX|`@\t-0.4\nsJPXHlQxQ{T@|rq@\t-0.4\nqCa@AILtAxe`\t-0.4\nRGPQFCBNT\t-0.41\nsJPH@DIRuPCCBKbq@\t-0.42\nsJQ`@bdvu@LH^IX\t-0.42\nsGX`BDdwMT@pfydZ\t-0.42\nsGP`@TfZj`FADQxmP\t-0.42\nQMJHBHvAxb\t-0.42\nsFp@DiTt@@AxqEcP\t-0.42\nsGP@DjUj`FFbwDZ\t-0.43\nHmvD`La@BLddlRVFUh@H@FETwBoHp\t-0.43\nsGQ@@dkMT@pvMxh`\t-0.43\nQM``fN`XL|`\t-0.43\nQMHAIhFD|R@\t-0.43\nQMDARVAaOJ@\t-0.43\nHif@@@RUe^Fh@@@XLBES]qHD\t-0.44\nsGP@Djfj`FAVMyH\t-0.44\nqC`@Qv`LDmrP\t-0.44\nqCaPQ@Hck@^HX\t-0.44\nqCa@AILtAaErX\t-0.44\nQMDARVAqUaqd\t-0.44\nQMDARVAxr\t-0.44\nHid@@DjUfaBB`@FCPaTw\\SA@\t-0.45\nqCb@AJZ`LINIX\t-0.45\nHif@@@Rge^Fh@@@X\\DES\\OHP\t-0.46\nHiD@@LdbJTjjh@XBDIaTw\\`P\t-0.46\nsJP`@dfvhA`enP`\t-0.46\nqCaPQCpcm@XQ\\Pp\t-0.47\n`IjqpFD\t-0.47\nsJQ@@dmU@LLL^JH\t-0.47\nHaD@@DjWzXHB@A`THU\\SN@\t-0.47\nKA|H@XGIJsuUPA`pPaMwHh\t-0.48\nsKP`@Ti\\Zj@XR\\Rp\t-0.48\nHeTJ@@qaeJYyzzjjj@FGPUMpsqDp\t-0.49\nsNq`AVeMmUPC@sU|RH\t-0.49\nsGX`DJdrmT@pRoEb\t-0.5\nsGQ@@drmT@pRwDZ\t-0.5\nsGQ`AjdmuT@pHboEb\t-0.5\nHaDH@@RVU[f@@@FAYS]qHH\t-0.5\nqC`@Qv`LHnJX\t-0.5\nqCb@AIZ`LDIqK@\t-0.5\nsGP@LdbMU@LCEcqB`\t-0.51\nsGP@LdbMU@LCEkq@`\t-0.51\nHmvD`La@BLddJfReUj@@@FAMpssLiP\t-0.52\nsJPH@DISuPC@bK\\a@\t-0.53\nHiD@@Dj}Yjf`A`rXU\\OH`\t-0.53\nsGY@DDeYj`FFVOEJ\t-0.53\nsJP@DjVhA`hnKD\t-0.53\nqCr@XIKTA`anJH\t-0.53\nHaD@@DjWjXHB@A`TIS\\SN@\t-0.54\nsJPhHaxISuPCAX|PP\t-0.54\nsOq@@drm\\@@@pHVOEV\t-0.54\nsGP@LdbMU@LJHl^R`\t-0.54\nsJXHL@aIVj@XZK\\PH\t-0.55\nsNx`LFdjmUPCAbHu|f@\t-0.55\nqChPK@HRYhCBCbV@\t-0.55\nsKP@Di\\Vj@XXP|Rp\t-0.55\nqC`@IVtA`aNIX\t-0.55\nsGQhHl@cIHTmPCBGbE@\t-0.55\nsNp@LdbJjj@XLQXwdX\t-0.55\nqCb@AIj`LMEnP@\t-0.55\nHeT@@LddTjPsU@@@CCdpj[ayD`\t-0.55\nsJQhHl@cIHUhAx`P\t-0.55\nsJP@DjZhAahmq@`\t-0.55\nsJXHD@aIYj@XP\\tq@\t-0.56\nsGQ@@djuT@pHVOEJ\t-0.56\nHaD@@DjUZxHH@A`TES\\qLx\t-0.57\nsGY@DDeUj`FFVOEJ\t-0.57\nqCp@AIZ`LLHnS@\t-0.57\nqCb@AIZ`LEEqC@\t-0.57\nQMHAIhFE\\Q@\t-0.57\nsJP`@TizhAaicrH\t-0.58\nHidB@@kaReYrZjjh@XMaTwCqHP\t-0.58\nqC`@IRtA`anJH\t-0.58\nqC`@IVtA``nJX\t-0.58\nRGAALXOD`\t-0.58\nqC`P@HRfhCCBKdp\t-0.59\nsNp@LddUjj@XVKE|e@\t-0.59\nsOq@@drm\\@@@prkyab\t-0.59\nsJQ@@eOU@LEMqH`\t-0.59\nHaDL@@kldRTIMUUP@pKJ[nYBD\t-0.59\nqC`@ISTA`enHP\t-0.59\nsJP`@dfvhAaEqX`\t-0.59\nqC`@Qz`LLHnS@\t-0.59\nsNp@DiWjj@XLQFodp\t-0.6\nqCa@CIKTAaaErX\t-0.6\nsNp`@dfUZj@XKGbk@\t-0.6\nqC`@IVtA`hnHX\t-0.61\nsJP@DjZhAa`cqS@\t-0.61\nsJY@LDejhAa`mqP`\t-0.61\nqCcPQX^D]hCAK\\`@\t-0.61\nsOp@DjWkB@@FEbq_IP\t-0.62\nsGP`@dfyj`FBQxmP\t-0.62\nsOq`AVeJnMUT@pvMyhf\t-0.62\nHef@B@B\\ddURUUUP@pFDpj[ayD`\t-0.62\nHcLF`HSFgP|LdfrbTHjuUUT@LNpjzUwb@H\t-0.63\nHaf@@@RiUkfzjjh@XUBDw\\SJ@\t-0.63\nHeTD@@QIgeQej@@@FATwCqLd\t-0.63\nsNp`@dfWZj@X[U|a@\t-0.63\nQM`AIhFBWI@\t-0.63\nqCb@AIV`LINIX\t-0.64\nHaF@B@BTeeVz`@@A`tXUMwH@\t-0.64\nsF|@AbeJf`@@OMEP\t-0.64\nHmtJ@@pYUJYmVnjjjh@XCBTwCJ|SB@\t-0.64\nQMHAIdGfR\t-0.64\nHiFL@BANR[gfjf@FFIa\\Ra@\t-0.65\nqC`XBHTQhaIj`LLEnR@\t-0.65\nHeTD@@QIemQej@@@FAUpsqBX\t-0.66\nHeTB@@XYRfV~njjj`AatYS\\L|QB@\t-0.66\nHmL@@DjYeVdU@@`@@FGXULLkrN@\t-0.67\nHifDAHAHeNR[e^Eh@@@XHP|rfx\t-0.67\nsGP`@dfuj`FFqoH`\t-0.67\nsJQhHlOAJmj@XP|Pp\t-0.67\nsJQ@@dju@LLl^R@\t-0.67\nHmtJ@@siWIEEMDZFjjj`A`LHU\\LkqLX\t-0.67\nsGQ`@ZdruT@pHbOEj\t-0.68\nsJX@@dkU@LBHmrD\t-0.68\nHidH@@RYm^Eh@@@XTUMp|c@\t-0.68\nsGX`BDdwMT@pRwDZ\t-0.68\nHaDL@@sdigZZjj`A`TYW\\qBX\t-0.69\nqCb@AJZPLINIX\t-0.69\nqC`HBHtPdlpFBFxh`\t-0.69\nsJY@DDejhA`l^HP\t-0.69\nsGQhHlOAJoZhA`c^HX\t-0.69\nHmtJ@@siUJYn~ajjjh@XSBXS\\LkrJ@\t-0.7\nsGPYHlQxGdjmj`FDwIP\t-0.7\nqCbPHABSS@^HX\t-0.7\nHifDAHAHeNR[e^Eh@@@XHU\\qnD\t-0.71\nHid@`@BDiyWaj@@@FC@UMp|RD@\t-0.71\nHid@@DjUfaBB`@FGPfESCrN@\t-0.71\nqCh@CIKTA`anJH\t-0.71\nHidD@@QInUxV`@@AaRDwCrJ@\t-0.71\nqC`P@HRVhCAB\\Rp\t-0.72\nsGY@DDfUjPFBVxcP\t-0.72\nHidB@@qarQQZFnjjj@FCQaMp|QJ@\t-0.72\nHif@@@RYm^Fh@@@X]FES\\OH@\t-0.72\nHidH@@RUe^Eh@@@XTaTw\\RA@\t-0.72\nHidD@@QInUxV`@@A`QMp|QJ@\t-0.72\nsGP`@dfuj`FFVOIP\t-0.72\n`ImAHFD\t-0.72\nHeTL`HS@BLddlRPrm@@@CBlinGdR\t-0.74\nqCb@AJZ`LEErX\t-0.74\nsJQ@@dju@LDcqS@\t-0.74\n`Ld`^P\t-0.74\nRFTQXQ`|R@\t-0.75\nQMhHRZAaGFP\t-0.75\nHaDH@@RYe[hB@@FF@S\\sJx\t-0.75\nsJQ@@eOU@LDmqP`\t-0.75\nsGP`ATeUjPFBVxhP\t-0.75\nHet@@DjYUX^d@@@@AarXS\\L|b`\t-0.76\nHeV@B@B\\dbLRPsU@@@C@XSJ[agd@\t-0.76\nsJP`@TizhA`l^JH\t-0.76\nqC`@Qz`LEEqC@\t-0.76\nsNqDHGBnFRRJuU@LBlWrT\t-0.77\nsJQ@@dlu@LLl^R@\t-0.78\nKA|X@DBLdvluS@FBHWDs`\t-0.78\nqCbPP^BRu@XI[d@\t-0.78\nHid@@DjU^nBAH@FGPfES]rA@\t-0.78\nsOpHAbILkW@@@LJEc^Q`\t-0.78\nqCa@CIKTA`aNIX\t-0.78\nQMDARZA`cr`\t-0.79\nHeVH@IAIe]ZZ@Bh@FG@fEWSrA@\t-0.79\nHidH@@Rge^Fh@@@XLaTwCrH@\t-0.79\nKAxP@Uddbcjf`C@dpfyG@\t-0.79\nRFD@QECBNT\t-0.79\nHeTB@@XYrQQJHuuUUT@LNcB[agbIP\t-0.79\nqCcPQXAD]XCqC@\t-0.79\nsGP`@dfUj`FDwDZ\t-0.79\nHeV@@@rRJEK\\MP@P@LFSBinyD`\t-0.8\nHaDL@@sdigzZjj`A`TXw\\qBh\t-0.81\nsGX@@djuT@phbqyJ\t-0.81\nsGP@Djfj`FAbqyJ\t-0.81\nHaD@@DjYvxH`@A`RDw\\SJ@\t-0.81\nHeTB@@pYRf[vnjjj`AatIc\\L|Pj@\t-0.82\nHidB@@saRf]ZZjjh@XMFTwCqDH\t-0.82\nqCr@PIKLAaAqK@\t-0.82\nKAx@@YIDTfjh@pZDpj{dL\t-0.82\nqCb@AIf`LHnS@\t-0.82\nHeTJ@@qaeJYyzzjjj@F@qaTwCOHP\t-0.83\nHetD@@QIemRhYZ`@`@XDwCOHH\t-0.83\nsJP@DkVhAahmq@`\t-0.83\nsNp`@dfvZj@XXu|Rp\t-0.83\nqCc@AYIf`ODl\t-0.83\nQMBBHRZA`erP\t-0.83\nHklF@@pUt|dTTRbQjZjjjh@XKBESCJz_Hx\t-0.84\nsJPXHlQxQ{T@pRwHP\t-0.84\nHaDH@@RVU[j@@@FEQaTwDC`\t-0.85\nsFp@DjfJjhAaicqJ`\t-0.85\nQMhHchGcH\t-0.85\nKA|X@QS\\eUkUU@FGPfES\\\t-0.85\nsGPXHlQxIU[U@LEcrT\t-0.85\n`IhapFD\t-0.85\nQMFI@bM`^H`\t-0.85\nHmtJ@@pYUJYmVnjjjh@XSBYS\\LkrB@\t-0.86\nHidD@@QInUxV`@@A`RDp|Rn@\t-0.86\nHiDH@@RUUVjj@FGIaTwCrH@\t-0.86\nHaF@@@RYe[hB@@FAXUMq@x\t-0.86\nHeTH@@RUYTYY`@@AarYS\\L|b@\t-0.86\nsOp`@tieujj`FAfkxdP\t-0.86\nsGQ@HeUuT@pLVOIP\t-0.86\nsJQ@@dmU@LBIcrH\t-0.86\nsJY@BDeVhA`d^JX\t-0.86\nHaDH@@RVU[f@@@FFYS\\rBx\t-0.86\nsNp`@dfuZj@XEXwdX\t-0.87\nQM@HzA`sr@\t-0.87\nHaDH@@RVU[j@@@FAAc]sHJ`\t-0.88\nsGX`LJdmuT@phfMyD\t-0.88\nsGX@@eJuT@pHbwDZ\t-0.88\nj`i@`@@HRYYUg\\eIF]yjB@@B@@F@fBdkQkNx`g`\t-0.89\nHet@@DjfehZfjjj`AarES\\L|c@\t-0.89\nHeTD@@EIYWVy`@h@FEXUMOH`\t-0.89\nsGPhHaxISuU@LMc^HH\t-0.89\nsJXHD@aIYj@XXX|d@\t-0.89\nsJP`@deVhAqIXPVOI@\t-0.89\nsJP`@deVhA`l^R@\t-0.89\nqCb@AJZ`LEMrH\t-0.89\nsFp`ATiWFjhAal[qQ@\t-0.9\nsGQ@@dsmT@prqyJ\t-0.9\nQMA@HPaIXFD\\Y@\t-0.9\nsNphHaxIRkUT@pNM_H@\t-0.9\nsOx@@drm]UT@p\\VM_H@\t-0.9\nHiFD@BADf{Ujj`A`rXUMwHD\t-0.91\nqCb@AIY`LEEqC@\t-0.91\nsJQHH@aJuj@XI[bA@\t-0.91\nHmtH@HrRQVIJUjh@@@X\\fDwCOHT\t-0.92\nsJQhHlOBOZ`FBVyB\t-0.92\nsJQ@IGuPCCQ[dH\t-0.92\nqC`P@HRVhCAQ\\Pp\t-0.92\nHidB@@qarQQZFnjjj@FEPS\\OLSJ@\t-0.92\nsJP@DjZhAa`mqP`\t-0.92\nHcND@BADf{YU]Zj@@@F@iaUproHt\t-0.93\nHaDD@@yIe^f`@`@XHe\\HLK`\t-0.93\nsGP@LdbMU@LBHcqZ`\t-0.93\nQM@HzA`erP\t-0.93\nsOx@@drm\\@@@ppUylV\t-0.95\nsJX@@dkU@LLEnJD\t-0.95\nHaDH@@Rfu[j@@@FEIaUwHX\t-0.96\nsJY@BDizhAaicrH\t-0.96\nQMpRRWhuAyP\t-0.96\nQMACD\\QIhFBOJ@\t-0.96\nsJX`LDdvu@OLlP\t-0.96\nsFp`ATiWFjhA`m^ZH`\t-0.96\nsGP`@dfui`FBqxeP\t-0.97\nKAxYDPf@DYvRRUIZj@LLJfyG@\t-0.98\nsOpHAVILkW@@@LJEc^Q`\t-0.98\nsFx@@eJfuU@LJHl^R`\t-0.98\nqCa@CIKLA`anJH\t-0.98\nsJP`@TizhAahmrD\t-0.98\nsKQ@@eKcUPCAX|TP\t-0.98\nJoBD@@IS\\rjzv[uP@@@@@XKS\\Lkiv\u007FDa@\t-0.98\nsNy@DDf]Zj@XYZ|QX\t-0.99\nRFTPxQP|`\t-0.99\nqCp@AI^PODl\t-0.99\nHcLF@@siWTify^ajjjj@FBpaTwCJ{rL@\t-0.99\nqCpB@SGj@ptVy@\t-0.99\nHaD@@DjUZxHH@AaTHS]qLh\t-1\nKAx@@IVjmUPAapPfES\\c`\t-1\nqC`@IWtAaEqS@\t-1\nHaD@`@BDimVz`@@A`pXUMwHP\t-1\nsJPhHaxIVmPCAX|`@\t-1\nsOx@@drm\\@@@prqyaV\t-1.01\nqCa@AIJtAaMrP\t-1.01\nsJPLIHJPtaGuPCrD\t-1.01\nsNy`BDtfuZi@XYX|e@\t-1.02\nRF`dS]ECr@\t-1.02\nHaD@@DjUZxHD@A`tIaUwHX\t-1.02\nqCb@AIj`LLEnR@\t-1.02\nsOp@DjWkB@@FAfMxdp\t-1.03\nqCr@XIKTAahmr@\t-1.03\nKAxY@PfAxIUjmUPA`QTw\\`@\t-1.03\nsJP@DkVhAa`mqP`\t-1.03\nRFTQxQ`pkd@\t-1.03\nsNp`@deUZj@X[F|TX\t-1.04\nqC`@IStAxe`\t-1.04\nsOq`AVeJnMUT@pNM_ED\t-1.04\nsJQhHl@bOV`FDwHP\t-1.04\nHmvLAHAEbT{Hihdh^Eh@I@AaaTODzP\t-1.05\nHmL@@DjYUVGi@@@`@FGIaUpsrE@\t-1.05\nHev@@@RfUWJzZABH@FE@aMwDFP\t-1.05\nHiF@@@rRHiJjj`AatIaTw\\`P\t-1.05\nHaD@@DjUZzjj`AapPaTw\\c@\t-1.05\nHaFH@DAIYUnZjh@XEaTwDc`\t-1.05\nHaFH@NAIe^f`@`@XTfEW\\`@\t-1.05\nHiFD@BADf{Yjj`A`rYS\\OHH\t-1.05\nsJPXhlQx[ct^u@LH`\t-1.05\nsFp`@TivJjhAahmrT\t-1.05\nsGQHL@aJYihA``nHt\t-1.05\nsJU@DPdru@LDcrX\t-1.05\nsJQ@@dju@LEMqH`\t-1.05\nQMABlZqIhFBOJ@\t-1.05\nRFHEFCqH\t-1.05\nHaDH@@RVU[j@@@FAAeMsHK`\t-1.05\nKA|X@DBLdvluS@FFIa\\c`\t-1.05\nqCbPXQBRt`XP\\TP\t-1.05\nqC`@ISTA`aNIX\t-1.05\nJoBD@@ILrkNx{[mA@@@@@XJU\\Lkiv\u007FHp\t-1.06\nHeTD@@QImeQej@@@FCIeMp|bP\t-1.06\nHeTD@@QIemQej@@@FEES\\OHd\t-1.06\nHidH@@RYm^Eh@@@XUaTw\\`P\t-1.06\nsJT@@TizhA`QL^Q@\t-1.06\nQMBCpRZAaOD`\t-1.06\nRGPQECr`\t-1.06\nqCbHXRBdDfj@|Tp\t-1.06\nHaDL@@[die|jjj`AaTYS]qDH\t-1.06\nqCr@XIJtAaMqQ@\t-1.06\nQMFIOBM`XSr@\t-1.06\njhi@`@@^Rfuue]Yrsj@B`@h@@XJQ`eFJtZp^PD\t-1.07\nJoBD@@ISZzrk][t@E@@@@XRfESKiwqA\\\t-1.07\nHiDNPhSB[aCpkpsb{nGlddqdZih@P\t-1.07\nHiDD`HPOAJUfjjh@XMaTwCr@@\t-1.07\nsOp@DjWkB@@FAbqxjp\t-1.07\nQM@HvAaWDP\t-1.07\nsJXA@IRej`FAbqy@\t-1.07\nsGP`@dfuj`FBQxmP\t-1.07\nHmuD`IVD@HrRFIKKaV@BP@XDUMwDAP\t-1.08\nsKP@H~Jj`FBqxh`\t-1.08\nsGX`DJdjmT@pRoMLP\t-1.08\nsKQ@@eKcUPCB[fbH\t-1.09\nsJY@LDefhA`enJD\t-1.09\nsGP`@dfuj`FBqxeP\t-1.09\nsJXHB@aIVj@XZH|Pp\t-1.1\nHeTD@@QInYQehB@@FEISCOHx\t-1.1\nHaE@@@yIe^f`@`@XXfEs@k`\t-1.1\nHid@@LdbbQxXF@@A`RTw\\sBD\t-1.1\nQMhXNiRZAxr\t-1.1\nsJP`@dfvhAaMqH`\t-1.1\nKAx@@IRzmUPAapPaTw\\c@\t-1.11\nsNx`BDdw[UPCCXobU@\t-1.11\nsFp`@dfTujhA`m^P`\t-1.11\nsJQ`@bdvu@LHnKD\t-1.11\nsJQ@@dkU@LMEnHD\t-1.11\nsJQ@@eKU@LEMqH`\t-1.11\nsGXA@IRiZj@XFHwbF@\t-1.11\n`Ic@hFD\t-1.13\nHmtJ@@XYUJYY^njjjh@XSFES\\LkrD@\t-1.13\nsGP`@dfUjPFDwDZ\t-1.13\nsJQHL@aJVf@XHKba@\t-1.14\nQMB@HRZAaGFP\t-1.14\n`Ig`hFD\t-1.14\nHif@@@RYWZZ@BP@XTfEW\\PA@\t-1.14\nsNp`@teUZi@XYX|Th\t-1.14\nsGQ`@jdvmR@pbxlP\t-1.14\nsGPP@cTfyi`FBU@\t-1.14\nHidB@@kaReYrZjjh@XUaTp|rDx\t-1.14\nqC`@IRtAaaErX\t-1.15\nsNx@@eLuUPCAqXu|`@\t-1.15\nsGP`@df]j`FBMyL\t-1.15\nHcMB@hdDWXeNf]yInU^xV`@f@A`aCqJX\t-1.16\nHeTH@@ReyTYj`@@A`tYS\\ODRP\t-1.16\nHaE@@@aJyUnh@@@XLFES]rH@\t-1.16\nHidD@@yJ[VXZ@H@A`RXp|Pn@\t-1.16\nHaDD@@YIeZn`B@@XXf]s@i`\t-1.16\nKA|@@AddQRjj`CAhSBinQp\t-1.16\nsNp@DkUZj@XLKFod`\t-1.16\nsGQ@@eJuR@ptuxdP\t-1.16\nsGY@JDiVj`FAbuyB\t-1.16\nsJQ@@dmU@LBEcrP\t-1.16\nsFy@LDigVfXAydZ\t-1.16\nQMJAhHzAxr\t-1.16\nsGQ@@eNuT@pLqoH`\t-1.16\nsGPXhlQx[crUVuPCrT\t-1.16\nHaDH@@Rfu[j@@@FFXw\\qBh\t-1.16\nJoBbPCCk`DISJj{nTLuUUSSP@XZDIaTwBi|bp\t-1.17\nHk]H`FGPBDieU]rafjjjZ`A`hPfES\\J|`p\t-1.17\nHmMH`FEPBLdTRRQrafjjih@XBDIaTp|c`\t-1.17\nHcmH`FGPBDieU_JFZjjih@XRDIaTwCrE@\t-1.17\nHeuH`FzPBDiewrnFjjf`AapPfTw\\``\t-1.17\nHiUH`JXPBDiU_B[jjih@XLDXUMrD@\t-1.17\nsFp@DiTujhA`Pm^JD\t-1.17\nsGP@DiUj`FEDVOIP\t-1.17\nsGQ`@jdsmT@pfxcP\t-1.17\nsGQ`@bdwML@|Rh\t-1.17\nsGX`LDdwMT@|rM@\t-1.17\nqC`@IVtAaaMrH\t-1.18\nHidD@@QInUxV`@@AaRES]rA@\t-1.18\nHeTD@@eIYe^E`HD@FAEWCqHX\t-1.18\nqCb@AIj`LLHnS@\t-1.18\nHidH@@RVY^E`H@@XDU\\ODa`\t-1.18\nQM@HvA`erP\t-1.18\nsNqDHG@nFRQFUU@LBmWq@`\t-1.18\nqCqAPJRnRYhCrX\t-1.18\nsNp@DkUZj@XLQXwdX\t-1.19\nqCp@AIZPLDEqS@\t-1.19\nsGQLKHaQFbLddmU@LCEcrT\t-1.19\nKAxH@@bSZkUT@XTfES\\c`\t-1.19\nsJP`@dfvdAaMqH`\t-1.19\nRGPp]SFCr`\t-1.2\nHmLH@@RYUUiiV@@@@@X\\BEproHp\t-1.21\nHiV@@@RfU|kahDB@AaPHS]qJD\t-1.21\nqCbPXQBRu@XHSbV@\t-1.21\nHcN@@@rRJIIGRmPTA@@pELJfxYyF`\t-1.21\nHeTD@@eIYWVy`@d@FAXUMqHD\t-1.21\nQMB`HIchFCOH@\t-1.21\nHaDH@@RVU[f@@@FAXU]qHX\t-1.21\nQM@H~Axr\t-1.21\nQMHAIhFCOH@\t-1.21\nQM@HvA`cr`\t-1.22\nqCr@XIKTAaaMrH\t-1.22\nHg|H@@RYfUWd}nh@@@@@XJUMprng_DB@\t-1.22\nHg^J@DCiV|bbtTVaTzuUUUP@pULJnE]N~Pp\t-1.23\nHcNB@DCiUIgf^gVjjj`AarXU\\J|Qm@\t-1.23\nHidD@@QInUxV`@@AabEwLia@\t-1.23\nHaDH@@RYVih@H@FFAW\\qLX\t-1.23\nsOpHAHILkW@@@LBlWrT\t-1.23\nsGT@ATivj`FEDqoH`\t-1.23\nqCaPQCpRmhCB[d@\t-1.23\nsOp@DjWkB@@FFbwLZp\t-1.24\nHiDD@@QImUZjh@XLfES]rA@\t-1.24\nQMJH|HvAxb\t-1.24\nsJP`@dfVhAaCrX\t-1.24\nHklNaHSBySCpsb{IILUDhYujjjj@FCTpki|bP\t-1.25\nHcMB@hDDWXeNF]yInUnzV`BF@AaaUOHx\t-1.25\nHg^L@BAAR[eVuNvji@@@FDiaUJz]|a`\t-1.25\nHmtD@@eJ[WUnh@J`@XTfES\\PU@\t-1.25\nHmtD@@QIn[VUZh@@@X\\fMproHh\t-1.25\nHidD@@QInUxV`@@A`RDw\\Ra@\t-1.25\nsNp`@tfUZi@XI[bM@\t-1.25\nQOHBNZ`XUrP\t-1.25\nsNp`@dfWZj@X[F|c@\t-1.25\nsNpXhlQx[cvRRXmU@LI^Q`\t-1.25\nsOx@@drm\\@@@pHVOEV\t-1.25\nHeVDAHAHeNR[e_aZ@B@A`QTw\\bP\t-1.25\nsJX@@dmU@LCEcr@\t-1.26\nqCbPPABUm@XQ\\Pp\t-1.26\nsJP@DiVhAaaEqX`\t-1.26\nsGX`DJdjmT@prqxiP\t-1.27\nsJP@H~j@XZK\\a@\t-1.27\nHeV@B@BTiVuFZh@@@XCBES\\L|`@\t-1.28\nHidH@@Rge^Fh@@@XTaTw\\RA@\t-1.28\nsNx`LDdvkUPCCXwbS@\t-1.28\nRGDQxQ`|R@\t-1.28\nHaF@@@Rfu[j@@@FCAaTw\\a@\t-1.28\nsJP@H~j@XJKbq@\t-1.28\nHmN@@@rRJIKEaxZjjj`AaLIaMproHh\t-1.29\nKA|I@Dw@DIJwUTpAaTIa]r@@\t-1.29\nsFx@@eLzuU@LJHm^P`\t-1.29\nqCr@PIKTA`anJH\t-1.29\nsJQ@@dru@LDmrD\t-1.29\nsNq`@jdvsTpCB[bS@\t-1.3\nRGAALXLHyP\t-1.3\nHaD@@DjWZXHB@A`TXw\\QB@\t-1.3\nsNp`@dfUZj@XXu|f@\t-1.31\nHeTD@@QImeQej@@@FCIS\\L|c@\t-1.31\nsJQ@@eOT`LML^Q@\t-1.32\nHidD@@QInUxV`@@A`REp|Rf@\t-1.32\nsJPYHlQxGh}j@XQ\\f@\t-1.32\nqCb@AIZPLMEnP@\t-1.32\nsOp@DjWkB@@FEbqoHp\t-1.32\nj`aAb@FBdAprRQUJYQIN}UUM@@@@peAFBTtZsnP|\t-1.33\nHg^DIItxdDdLdKbId~rRRqISQBmT@Pt@CCJZ~PH\t-1.33\njhip@@@P\\eKLjo}HvkM@@@EP@@pTRcARUjw`|ap\t-1.33\nHo}DHL^D@HVPvPNPrRRqQJQN[V``jH@FAXporI@\t-1.33\nHo}DHLzD@HNPnP^PrRRqQ[JRUV``Jh@FAXwCrI@\t-1.33\nj`qa@@H@RVYUg^Qg^f``@@@@FF`TXJRmFl{wHD@\t-1.33\nj`aP`@N^`sIIEYdhdmYuUMP@P@CBRLDxIVcVyCp\t-1.33\nHaD@D@|HBDBLBBBJ|FBNBLddqfTuSP@pJDqnHW@\t-1.33\njdi@H@@HMYw`qR[fYf[iZZ@hafX@A`QEoQxbV`\t-1.33\nj`aa`@APYsIIEYdeDd{uUML@@@C@RLDtZso\\aH\t-1.33\nJgRF@EM^RRJJHjkNHRd]gZijjfj`@pyLKae^PX\t-1.33\nJoB@QGTG]ddRtJbReL]a@T@D@@XKBYSRiv\u007FHx\t-1.33\nJoBBQBzG`DiSJkvoS[tDDUTP@XZfES\\Lg[|`@\t-1.33\nj`a``@APYddbltrbR]zjfj@@@AaiFB\\EKQkN|\t-1.33\nJoBe@BX@j[yEEDeEeJw[[USSUT@F@xU]Iv\u007FHt\t-1.33\nHknDqItxdDdLdId~rRRqISPUj`BF@AaeLoHD\t-1.33\nHg]HpMLhdDdLdLddjbRrTkUP@T`@pinF]yE`\t-1.33\nJcBB@E[ddbTQJqbxazQfjZjZj@CAdpfx^IB`\t-1.33\nJcBB@EMddbbbJsbDiGYvjZjjj@CCdpnFUyF`\t-1.33\nHo}BbFvDqpBL{IIJeDedeuZjjjh@XRU\\Lj]n\t-1.33\nJoC@QDAxAJTsMjnmf}T@@@@@FFeS\\Lkiv\u007FH@\t-1.33\nHg|@b@|@eJYUg_eNzB@@@@A`jXS\\Lki|PK@\t-1.33\nHkmHpMLhdDdLdLddjbRreZj@B`@XTwCN|`p\t-1.33\nHg|DbEtO@IRfUWmzSn``bjH@XJfES\\Lg_H@\t-1.33\nKA|PPPrPFPAPGPFRRJrjf@NBQ@q@I@y@yD@\t-1.33\nJoBb@EFNyIHiTlhepv}USUMT@FGPfDtwrG@\t-1.33\nHklHHCbPRPrPvHrRRqISPUj`BI@A`ULn|`P\t-1.33\nHg\\L`HS@\\LddRtRRIwUTE@@@pffxYWSodX\t-1.33\nJoC`@@PJUrnljwV{@AP@@@FBxULJz]orH@\t-1.33\nHklDpMNIAICICIIKDeMAVj@HX@FATrgrA@\t-1.33\nHcL@HHdDdLdMqLddlRTHsU@AP@LJiiW^PH\t-1.33\nJoBd@AonRRJUKJICMoU@ASP`AarXUMp|bp\t-1.33\nHcL@HHdDdLdMbLddlRTHsU@AP@LJiiW^PH\t-1.33\nHklDpJZIAICICIIKLhdmNjZ@@@FACJ{rC@\t-1.33\nHg\\D`HlGCIIEUEDeiuU@PP@LISBhYWSyE`\t-1.33\nHo\u007FD@Nux[IIEYdeDk^jfjeh@XBfEtg[|`P\t-1.33\nHg\\HpJbPRPrPrRRsQIYKmTp@T@CBnFUOdV\t-1.33\nHeT@H@\\HbDbLbLddlRPsU@@@CChSJ[ayD`\t-1.33\nHo~DANW^g^rRQRqJJJSj``Xh@FEIaV\u007FDXp\t-1.33\nHmt@HHdDdLdEbLddlRVFZh@H@FFTr|Sa@\t-1.33\nJoB`@@Ndrnlktqvtlp@T@A`hIaUJz[|c`\t-1.33\nHmt@HHdDdLdIqLddlRTEZh@`@FATtkrA@\t-1.33\nHo~DB@XPgIIETihdk^jjX@@@XKBXUKivx\t-1.33\nHklD`HPGCIHdbhdmNjj@@@FDxULLki|`@\t-1.33\nHo|F@@RUm|bfbbTRlGV``Xf@AabJ\u007FLkm@\t-1.33\nHo~D@NalddUfRbRmzjZ`@@AaJXWBng_H\\\t-1.33\nHcL@PH\\JqLddT\\RR{UT@@@LISBkae]yD@\t-1.33\nHe\\H@JrRQJHxq\\P}HsUMUUP@pYLInGbQH\t-1.33\nHmtFaHSBEP|LxnrRSFJJUZjjh@XYSCqNd\t-1.33\nHklHhCBPJPZPFPVPRYf{\u007Fi`bHj@AaA|cP\t-1.33\nHg^HB@xIrRQRpqIG]UU@@@C@xSBine]Np\t-1.33\nHmTLpHZP\\J\\A\\LddUbtjjj`AarXU\\J|bP\t-1.33\nHiDLpJXT`L`B`LddTZjfd@\\XbAb@RArN@\t-1.33\nJoBF@@IrSJzljUV{UUP@P@FDxULng[|`P\t-1.33\nHedLHJ[h|DbLbBbLddTcuTuP@piLHYyG@\t-1.33\nHmtLbHS@|@gIIEXdiVj`@@AapDwCJ|QL@\t-1.33\nHmtBaHSBCpsb{IILXdiUj`@@A`S\\L|aP\t-1.33\nHmTBHJXUbPRPrPxJrRRsIJjZi@FD]rI@\t-1.33\nHcLL`HS@\\LddQbRr{UPP@@LNinFUwbI`\t-1.33\nHcLHpMbPRPrPrRRqIPcMT@D`@pJfe^PH\t-1.33\nHmTBHJXUbPRPrPytrRRsIJjZi@FD]rI@\t-1.33\nHg\\L@HSLddRtRRIwUTE@@@pffxYWSodX\t-1.33\nHeT@p@BJBFBLdTTRHsPT@@CC`SBxYxeL\t-1.33\nHklH`HAprRQUQQKSjj@`@A`JXUCNgrE@\t-1.33\nHg^L@NakrRQVZJIZ]UMUS@CCdpagSyB`\t-1.33\nsOpNAxPHPQ``aE|FDYIIcjuS@LCEgqF`\t-1.33\nHcLHbMCpBTieV_ehHJH@FBiaTwCJ{r@@\t-1.33\nJoBB@@Ndrnlktqvtlp@T@AaJXURnf\u007FHx\t-1.33\nHmtHPHApIprRHjIJUjh@@@XBBXwCJ|``\t-1.33\nHcvJ`FaLWPBLddjRbRuUU@C@\\JfFUwd\\\t-1.33\nHk^@B@BTjU^ukmLHH@@@A`lHU\\Lki|c@\t-1.33\nHk^D@D@\\bbbbRQImMj`XB@AaU\\ngrI@\t-1.33\nHg\\L@NN|ddTlVVdkUA@S@@pyLInV~PH\t-1.33\nHmTDHJVIAICi@YCIICELZijd@XPwDJP\t-1.33\nHmv@pIbPRPrPrRRqIPUj`B@@XESRoHD\t-1.33\nHiD@XHdDdLdJdNdAdLddltjjh@XPWHX\t-1.33\nHmv@pEbPRPrPrRRqIXYj`@`@XES\\OHT\t-1.33\nHmt@P@\\HbLddqbReZj@@@FGPc\\LkqLH\t-1.33\nHk^D@D@\\bbbbRQImMj`XB@A`WKi|Ri@\t-1.33\nHmv@pIbPRPrPrRRqIPUj`B@@XESCOHT\t-1.33\nHmv@pIbPRPrPrRRqIPUj`B@@XEProHX\t-1.33\nHk^H@IOIIEJeLaWSj`BFH@XLfESCrM@\t-1.33\nHcLH`HAprRQUQIKmUP@@@pELJne]y@`\t-1.33\nHg]L@NvFOIIEYhddnuTuTt@LJSBF~QH\t-1.33\nHcN@bACpBTieU{ihHHh@FDiaTwRn|`@\t-1.33\nHmt@P@\\HqLddqbReZj@@@F@qaMproHP\t-1.33\nHo|H@KrRQRiQHpYj`@jX@FCIaKk|Rf@\t-1.33\nHedHHNbPRPrPYprRRiQUUU@C@bhYyC@\t-1.33\nHg~D@DAdfygUQd{ZB@@@@FCUrz]|QF@\t-1.33\nHmtBbHSB@HKlddNbReUj@@@FAMpsrE@\t-1.33\nHmVBcNaLHOCNKngIILTiFjjh@XUSCJ`\t-1.33\nHmtB`HSBCprRSFIJUZh@@@XTwCJ|SB@\t-1.33\nHet@b@|@gIEEDcSCMAPD@C@Tpj[agd@\t-1.33\nHiDDHJZHaHcH`oCIIKLjij@FEPf\\OHH\t-1.33\nHctBPHZUApGprRQVYSUTtt@LJSB{^PH\t-1.33\nHid@p@bH\\DbDfyWaj@@@FG@aTwCrH@\t-1.33\nHmLB@HSerRQHiJfFZjjj`A`vESBoHd\t-1.33\nHklD@NOIIEKEeiVjB@d@FGIaMr{rA@\t-1.33\nHcLL@HSLddQbRr{UPP@@LNinFUwbI`\t-1.33\nHo|F@@Q]mtfyfYZSZBHXX@FFIk|siL\t-1.33\nHo|F@@SSmtfyVV{iZ@HfX@FFH[|pjd\t-1.33\nHk\\@@DjU^ukmNeZ@B@AaLHUCJz_Ds`\t-1.33\nHklH@ErRQRqJJ]j`Hb@AarXSKi|PY@\t-1.33\nHklH@IrRQRiPhUj`BJ@AarXUMI|P]@\t-1.33\nHmTBHJXUbPRPrPzlrRRsIJjZi@Gb]H\t-1.33\nHgnD@NalddUfbreUMUT@LASBe]N~PH\t-1.33\nHaL@@DjYtkBqjynjjjj@FCABDw\\SJ@\t-1.33\nHklD@NwIIEKEeiVjB@h@FGIaMrgrA@\t-1.33\nHknBAHAEvISdfyW[xV`@f`@XDUMwHL\t-1.33\nHkL@bLhLyJgnYzjjj`AaXPfES\\Lki`\t-1.33\nHeTL`HS@|LdbLRPsU@@@CA`inF^Hf@\t-1.33\nHeTDbHPO@IrRHqICMT@@@LFrfxYx`H\t-1.33\nHid@p@qH|DqDfyWaj@@@FG@aTwCrH@\t-1.33\nHcM@@@wIHhdd]JuAPD@C@Tpj[agdZ\t-1.33\nHko@@@`Tke]e[f@Bj`@XCaTtki|b@\t-1.33\nHeTDPHP@bYCIHXdmpk@A@@pHj[nP@\t-1.33\nHiDLHJXXdDdLdNBlddlrjfh@XQwHD\t-1.33\nHctDPJEYCYCIIEFiiUMMP@pILInQH\t-1.33\nHmTLRHS@BN\\L{IIJdiEjjh@XLULLj\t-1.33\nHkND@NalddUfRbjijj@F@ia]Jz_HD\t-1.33\nHiDLHJXXdDdLdN]Lddlrjfh@XQwHD\t-1.33\nHeT@P@bH\\Df^UFZh@@@X\\BTwCODc@\t-1.33\nHklD@@qJYVu}NjhH@@FDiS\\Lki|b@\t-1.33\nHkLB@NFUrRQRqSFjifh@XMBXVgrE@\t-1.33\nHmtB@HRUrRQHiJfjjjh@XMaTtkqND\t-1.33\nHiDLpJXXbDbLbLddlrjfd@XYBYrN@\t-1.33\nHmt@`NqLddTlVeZhH@@FCIaMr|PY@\t-1.33\nHmt@`EqLddTjVFZh@H@FCIaTr|Sa@\t-1.33\nHmVLB@ZUB\\ddUfTjji`AatIaUr|a`\t-1.33\nHmVB`FaLT@cIIJiDUjjd@XUaMp|bP\t-1.33\nHiFDHNihBDdLdBdLddTgjeh@XQwHD\t-1.33\nHiDDpJzHaHcHcIIJeJjf@FAPfErA@\t-1.33\nHmv@`JVYrRQRiHYj`@`@XLfDtkrN@\t-1.33\nHefDpFFXdDdLdLddjfeUTp@prfgdD\t-1.33\nHmT@bLhLyJgnVjjj@FBaBXU\\LkrF@\t-1.33\nHedLpJFXdDdLdLddlReUTp@pSiyA@\t-1.33\nHk\\D@@iIYgU^GSXH@@@@XLSRngqHh\t-1.33\nHctB`HZUGprRQVYSUTtt@LJSB{^PH\t-1.33\nHmTH`HAprRIIPjjje@FDiaTwCJ|`@\t-1.33\nHcLD@NwIIEKEeJuPPD@CCdpfx]yB`\t-1.33\nKAxXPTwQxIDYDEDIUMUSPAabXWD[`\t-1.33\nHedLPH[`\\J|LddUbuUUP@pYLJagd\\\t-1.33\nHmTLPHeP\\JBDjeUjjjT@XLfDwCrJ@\t-1.33\nHcM@@@wIHhdd]JuAPD@CBTpj[agP\t-1.33\nHcLH@NrRQRqYRmTDA@@pyLInW^PH\t-1.33\nHkLFb@RfgRhByIgeYfjjY`A`VDi`\t-1.33\nHctB`HY]AprRQVYJUTwL@LLSByF`\t-1.33\nHid@P@BJBDie_ahH@@FCAaMp|QJ@\t-1.33\nHeT@`@|LdbLRPsU@@@CC`bfxYxbX\t-1.33\nHmtD`AdOAJYUgedHF@@XLfDwKq@d\t-1.33\nHeTD`HDGCIIEXituTuP@piLKiy@`\t-1.33\nHiDLHJXXdDdLdNkLddlrjfh@^Ip`\t-1.33\nHcvL@NamrRQVYQUTuR@LFSBxUyD`\t-1.33\nHcvB@JFUwIIEJhiUULp@pZDpjgdZ\t-1.33\nHeTH`HAprRSPiCMT@@@LNpj[agdP\t-1.33\nHmtH`HApRi[WeZhH@@F@xUMproH@\t-1.33\nHetL@HR\\ddRJULLuUUT@LBpj[b\\H\t-1.33\nHefDpFjXdDdLdLddltUTuH@p`^Qp\t-1.33\nHedDjAeICI@iBiAiB^rRQR^UUS@B\t-1.33\nHcND@JADfUyU]Zj@@@FGEWCJ{rL@\t-1.33\nHmTH`HAprRQIIVjji@F@iaMproHh\t-1.33\nHmVH`DTGCIIEHizjjd@XMaTwSrA@\t-1.33\nHiDLPHS@BB\\LdaTRVjh@XES\\OD@`\t-1.33\nHedLPHZP\\A|LddUfUUMH@pILKnPH\t-1.33\nHmw@@@`Tke]Vy`@j@A`vESRoD`P\t-1.33\nHiFHb@E``IrRQUJjjPA`tIaTwHx\t-1.33\nHedJb@RfDj@nRYyyjjX@XYaSrF@\t-1.33\nHcuB@NFFW\\ddUfReULs@CCDpnQh\t-1.33\nHmv@@JrRQRiHYj`@`@XLfDtkrN@\t-1.33\nHmtD@@qJY{VUjh@@@X\\eMproDc@\t-1.33\nHctL@NF\\ddTlVuUSM@CAhSBW^Pp\t-1.33\nHmtD@@yJUe^Uj``@@XCATwCJ|a@\t-1.33\nHiDDpJzIAICICIIJeJjf@FDErA@\t-1.33\nHcLB@@RUR[fVQuhHF@@XHb|JnGP\t-1.33\nHcLB@@Q]R[e[neh@a`@XHf|HjzP\t-1.33\nHcLB@@Q]R[e]nEh@I`@XHf|Hj[`\t-1.33\nKAxA`@xPD\\DYIHjjjh@pZDpj[d\\\t-1.33\nHidD`HRgCIIEYLJjjh@XUaMp|a@\t-1.33\nHiFDpFhXdDdLdLddlrjfd@XPwHx\t-1.33\nHmVL@JFUrRQRjJjjY@FCPfET|c`\t-1.33\nHmvD@LADfueYUj`@@A`vMproDP`\t-1.33\nHmTH`DAprRQRKrjjf@FCXUMr|bP\t-1.33\nHmNH@BAIfUmaeX@@@@A`pXproHx\t-1.33\nHmVD@Aa\\ddUbajjjPAarXUMp|bP\t-1.33\nKAxX`Ia@tQDYIIcJjh@phHinxfD\t-1.33\nsGQBK@aPF`C@IUMT`NLdCHAPNR@\t-1.33\nsGQLJhCPA`Djfj`GFRAd@hDI^R@\t-1.33\nHcL@@LddTjTPsU@AP@LJSBW^YSR\t-1.33\nHmvD@DClbbbRJIfjVj@FEYSCOHT\t-1.33\nKAxX`Pf@xTxYIHkJjh@pHj[nHf@\t-1.33\nHmLH@@RYVuiiV@@@@@XLe\\Lkq@p\t-1.33\nHmtH@ErRQRiXYj`@`@X\\fTwBoHH\t-1.33\nHiDDPNDGA`cIIEUJje@FEPfEWHX\t-1.33\nHiDLPJxP\\FBLddUTjjT@XEBXWHX\t-1.33\nHiDL`HS@\\LddQTjih@XTUMp|QL@\t-1.33\nKAxY`DwQDIDYDYIIUJjX@prDsd\\\t-1.33\nHiDDPHXGBoCIIETjjj@FEIa\\OHh\t-1.33\nHmtB@HPYrRQHzZfjjfh@XEaToHX\t-1.33\nHiDDPFxGBoCIIEYjjZ@FEPfDOHh\t-1.33\nHid@b@|@eJYWxZB@@AarXUMp|`@\t-1.33\nHeTH`LCDRVyTYi`@@A`pQS\\OHd\t-1.33\nHeU@@@eJYW~F``H@FCIaTw\\PI@\t-1.33\nHmtD@@eIfUTUZBA@@XMaUpkrF@\t-1.33\nHaD@P@bFbDfUjz@H@AaPIc]rJ@\t-1.33\nsGXCJJAPF`C@CdjfjpGFRAd@h@\t-1.33\nHmUL@NFFWIIEYdjjYd@XXfErI@\t-1.33\nKAxQ`TdQdPTPeTuUU@FEIaMwHh\t-1.33\nHeV@@IrRQRiCMT@@@LJSBinIp`\t-1.33\nHmTL`HYP\\LddUfRjinPAabXWHd\t-1.33\nHctB@HY]rRQVYJUTwL@LLSByF`\t-1.33\nHmTB@NFUrRQRqRjif@FAPfEqAT\t-1.33\nHid@`@|DiyWaj@@@FCPUMp|PD@\t-1.33\nHiDD`HPGCIHdcJjj@FCXUMp|`@\t-1.33\nHmtD@@QIee^UZ``@@XLU\\LkrF@\t-1.33\nHidD`@RJAInUxZ`@@A`pHULOHx\t-1.33\nHeTD@@IJUmQfj@@@FCQS\\L|QB@\t-1.33\nHmvH@AAIYWYnh@J`@X\\FESRoH`\t-1.33\nHidH`H@HRme^Eh@@@XLaTwCr@@\t-1.33\nHiFL`FaL@HrRRjQZj`A`VEp|a`\t-1.33\nHidH`HAprRQVKajij@FEIa\\OHX\t-1.33\nHedD`HDGCIIEXiUTr@LJSBz^PH\t-1.33\nKAxX`Pw@xUxYIHkJjX@pILJnQp\t-1.33\nHmVL@NaerRQVYJjfi@FEIa\\OHd\t-1.33\nHmVD@NalddUfTjij`A`rXWRoHD\t-1.33\nsNqBbJBRBdMHCcNRRUUU@LEWrD\t-1.33\nHedH`DAprRQRKuUU@CAlJfz^PH\t-1.33\nHmVB@NaiWIIEYeJjZX@XDfEwHT\t-1.33\nHiD@bLhLyJgmjjh@X\\DIa\\ODi`\t-1.33\nHiD@bLhLyJgyjjh@XBDHUMp|c@\t-1.33\nHmtL@@SDfUmYUj`@@A`S\\L|Re@\t-1.33\nHiDDPNDGA`cIIEUJke@FFIa\\a`\t-1.33\nHiDH`HAprRIIFjjPA`rYS\\OD@`\t-1.33\nHedD`HdGCIIEXmUSR@LJSB[nPH\t-1.33\nHiDL`HXP\\LddUfjjX@XDfEwDXP\t-1.33\nHiDL`HxP\\LddUTjih@XDfEWDxP\t-1.33\nHcvD@AalddUffeUMU@C@TpnFUt\t-1.33\nHeTH@@RUYTYj`@@A`HQeMpsrD@\t-1.33\nHmTJ@H[iWIIEYeJjZX@XXfErE@\t-1.33\nHid@`@\\Die_ahD@@FGIaTwCr@@\t-1.33\nHidH`DBHR[e^FX@@@XDATOD[`\t-1.33\nHiDLpJXXdDdLdLddlrjfd@^Qp\t-1.33\nHiDBb@RnJ`Kdf]Yji`A`fCrB@\t-1.33\nHig@@@idiUWajjj@FDaBXUMp`\t-1.33\nKAxHPTtRBRFRGNfRRVZjZ@OHx\t-1.33\nHeTL@@SDf^UFVh@@@XES\\ODrP\t-1.33\nHiDH`HAprRQURjf`AaRXWCrJ@\t-1.33\nHedL@NF\\ddTlUUSH@pJDpnHp`\t-1.33\nKAx@`@xQDYIIcJjh@pjDinxfD\t-1.33\nKAxH`MpNEAFRRJrje@LBaLKdT\t-1.33\nKAxH`TpND^FRRJNjZ@LBSB{dP\t-1.33\nHaDH`@b`RVU[j@@@FEXUMwDB@\t-1.33\nKAxI`UtRBRFRFRRUJji@LHKdL\t-1.33\nHaDH`HAprRQVMuUU@CBdpf{dT\t-1.33\nHiUH@HPDiUWBxZVeh@XIp|Rf@\t-1.33\nHiFH`DDGAJiijjh@XLfES]rA@\t-1.33\nHiDH`HAprRQVJjiPAaRXWCrF@\t-1.33\nKAxHPTtRBRFRGUfRRVZjZ@OHx\t-1.33\nKAx@`PD\\DYIHjjjh@pZDpj[d\\\t-1.33\nHefD@Nb\\ddUTuUUH@pYLJf{dB\t-1.33\nKA|I@YBAxYICDjjh@phBinxdH\t-1.33\nHefL@NairRQVYUTt`C@dpny@`\t-1.33\nHiD@`F\\LddUTjjh@X]BXU\\OHX\t-1.33\nHiE@`HZoCIIEUJij@FAIgCqBh\t-1.33\nKAx@`AxPDYIIcFjh@pjDinxfD\t-1.33\nsNpbJyDJHtP_CIIKJi`FFbwDV\t-1.33\nHiFH@HGIIEKjjj@FCIaUp|a`\t-1.33\nHiDD`NDGCIIEUJkf@FFIa\\`P\t-1.33\nHidD@@EIYW[f@B@AaTXUMrA@\t-1.33\nHidH@@RYm^Fh@@@X\\FES\\OH`\t-1.33\nHefL@L@YRUeVjjh@XRDHUMpr\t-1.33\nHid@@DjUfaBB`@FCPfMp|Pj@\t-1.33\nHidD@@EIYW[j@B@A`pQaTwHD\t-1.33\nHmTL@HY\\ddUfRjinPAabXWHd\t-1.33\nHedD@F{IIEYeUST@LJSBF^Q@\t-1.33\nHedB@HXYrRQVZUTs@CCDpnQH\t-1.33\nsNpB@DQppPPaJYjj`FEDU_Eb\t-1.33\nKA|A@TCaddbjjj`CAhSB[nQP\t-1.33\nKAzA@QsAFRRJrjf@LJaLInQp\t-1.33\nKAxY@Pf@DYIIUEjh@pKB[nQP\t-1.33\nHiFH`APGAJiijjh@XUaTw\\`P\t-1.33\nKAxI@QpNFRRJjjj@LJSBinQp\t-1.33\nHiFD@Jx\\ddT\\jjX@XUBXP|b`\t-1.33\nHiFH@AcIIEXjjj@FCIaUp|a`\t-1.33\nsNxBB@bRBdMHYIIUUT@ppkyB\t-1.33\nHaF@B@BTjWzXHB@AaTIaUrN@\t-1.33\nsJXLHhCPA`H~j@\\YHFPB`\\d@\t-1.33\nHiDH`HApRjUZjj@FCIaMp|b`\t-1.33\nKAxI@QpNFRRJjjf@LBSB[bUp\t-1.33\nKAxI@QpNFRRJjjf@LBSBkbMp\t-1.33\nKAxI@P`NBUNuTt@XUaTw\\PD@\t-1.33\nKAzQ@Qs@xYIHkJjX@pJDpnQp\t-1.33\nHedB@H[irRQVYUTt`CCDpnPH\t-1.33\nKA|P@QvRRJjjj@LFSBinyD@\t-1.33\nKAxX@AaYHhdZjh@pjDinyA@\t-1.33\nHaF@@@Rfu[j@@@FC@fES\\c`\t-1.33\nHidH@@RYm^Eh@@@XUaTp|c`\t-1.33\nHaDH@@RVU[j@@@FCAFTw\\``\t-1.33\nHiDD@J{IIETjjj@FAPfEqFD\t-1.33\nHiDL@JX\\ddTZjfd@XXfErN@\t-1.33\nHaf@@@RiUkfzjjh@X]BXUMp\t-1.33\nHiED@NFFrRQVZjfPAabXWHD\t-1.33\nHiDD@NGIIEJjje@FAPfEqJX\t-1.33\nKAxQ@\\CaddbjjjPCBhSBkdL\t-1.33\nHiDL@HX\\ddUfjid@XXfErN@\t-1.33\nKAz@@QvRRJjjj@LFaLJfyG@\t-1.33\nKA|I@]C@xYIHkJjX@pqLKd\\\t-1.33\nHaF@@@RnU[fjj@FCIaTw\\b@\t-1.33\nHigL@Ds`XTfUfn`BH@FF@p`\t-1.33\nHigL@Ds`XTfUfnZif@GfIg@\t-1.33\nKAxQ@PCaddblji`C@dpnyC@\t-1.33\nHaDD@@yIe^f`@`@XDBUwDp`\t-1.33\nHaDH@@RYWifjj@FEIaTwHx\t-1.33\nHaDH@@RVU[f@@@FEQeMwHH\t-1.33\nKAz@@ArTjmUT@X\\DIaUwHX\t-1.33\nKAxH@UvRRJjjj@LBaLKbMp\t-1.33\nHiDL@HSDjeVjj`A`QTw\\`P\t-1.33\nHiDD@H[IIEYjj[@FFIa\\`P\t-1.33\nKA|A@PC`eTsUU@FEIaTwHx\t-1.33\nKAxA@@xYIHjjkh@pILInQp\t-1.33\nsNtDHpxTHrRQVjTA`Uc^Q`\t-1.33\nKAxH@QvRRJjjj@LJSBknPp\t-1.33\nKA|P@]FRRJjjj@LJSBknPp\t-1.33\nHaD@@DjUZxHD@A`tHUMwHp\t-1.33\nsJPJaPRPTai@r@zSGuPCrP\t-1.33\nKA|X@XB\\drzuU@FFXw\\Pj@\t-1.33\nsNxLGIARFdLddjjj@XQ|Qh\t-1.33\nKAxP@@djkUU@FG@fES]rH@\t-1.33\nKAxH@ArRjmUT@XLBXU]qHX\t-1.33\nHaF@@@RYe[hB@@FA@UMqLx\t-1.33\nHiDH@@RYuVjj@FCIaTw\\`P\t-1.33\nsOpIAVBTfUk`@@FEVM_H@\t-1.33\nHiDB@@QnRY^vjZ@FBDOHp\t-1.33\nsNpY@jsPGdf^Zj@XKGbk@\t-1.33\nsNpDADQpRYzjhAaPcWqS@\t-1.33\nsNpD@xTJrRQVjhAaXl[rL\t-1.33\nsJQBBGBQBbMDQ}T@ptVyB\t-1.33\nsGPL@DQppPeKUT@pHQoEL\t-1.33\nsFqlAcJATEhDiTvjhAyhj\t-1.33\nsNpD@xPHrRQVjhAaXl[rL\t-1.33\nKAxA@@xYIHjjkl@pqLKd\\\t-1.33\nsNyLGYHJPtaddeeUHCqU`\t-1.33\nsNpdE`tRHRkzjhA`XZ~HX\t-1.33\nsNpyhlZqVGkszOvj`FBV@\t-1.33\nsNqhHlGCIHcJj`FFM_LE`\t-1.33\nsNqhHlOCIHcJj`FAAkxe`\t-1.33\nKA|@@@djkuU@FEIaTwHx\t-1.33\nsNxHHGCIIEZj`FEVM_I@\t-1.33\nsNqhHlKCIHcJj`FFM_Dl\t-1.33\nsOpHAbILkW@@@LBEgqF`\t-1.33\nsNpH@xYIDYUT@phakxe`\t-1.33\nsJPLADQpTPdwU@LBEcrP\t-1.33\nsGXDHGByBUSUPC@kF|d@\t-1.33\nsGPDADQpRUjj@XTKF|d@\t-1.33\nsNqhHjGCIHcJi`FBsx``\t-1.33\nsJPlLQxRHTQGtpCAQ\\a@\t-1.33\nsGQhHlGCIIKUPCAF|tV@\t-1.33\nsNxHHpcIIEZjPFEbq_IP\t-1.33\nsNyHHpxYIHkUR@pJqoI@\t-1.33\nsGQdEbOBQFRRFZ`FDgDZ\t-1.33\nsGXhMDOCIHUUPCC@wfbp\t-1.33\nsOxA@IRi^lH@@XVKF|c@\t-1.33\nsNpIEPKdkejj@XFJofFH\t-1.33\nsNpIMP[dj_jj@XTQE|Vh\t-1.33\nsGXDHGBxBUSUPC@kF|d@\t-1.33\nsGXDHGBzBUSUPC@kF|d@\t-1.33\nKAzD@XPHwIMkMLpAxb\\\t-1.33\nsNqHHGCIIEZjPFAVMyF\t-1.33\nsOpHAHILkW@@@LLc^KL\t-1.33\nsNpHADIJkUT@pXbqoHp\t-1.33\nsNy@iYMyJijjPFFVOHp\t-1.33\nsGXLKIARFdLddmU@LDZ\t-1.33\nsNqhAVjAJYjj`FFfOEV\t-1.33\nsGQi@jZ@|dsmT@pfxcP\t-1.33\nsNpIEPKdkejj@X\\QXu`\t-1.33\nsNpH@xIRkUT@p\\VM_H@\t-1.33\nsGPH@xYIDZi@XVKF|`@\t-1.33\nsGPD@xTJRjZj@XFKGdh\t-1.33\nsNxHGGCIIEZjpFFVOI@\t-1.33\nsNphIPxYIDYUL@prsy@\t-1.33\nsGPhH`xYIHZj@XKGfbh\t-1.33\npnH@BTlzemUR@X\\QXu`\t-1.33\nsGXDHGBaBUSUHCCKGdh\t-1.33\nsNyHEAxIWKUT@pNM_DH\t-1.33\nsNyHEAxIR}UT@pNM_DD\t-1.33\nsFxHHOAIeMjj@XEXwd@\t-1.33\nsGPM@xRPT`Nc}U@LMcP\t-1.33\nsNtHHpxYIHkUJ@prqyF\t-1.33\nsGQ`HmddejhA`c^ZK@\t-1.33\nsNq@KddbmUPC@qX|c@\t-1.33\nsGY@iYMyJijhAaecrP\t-1.33\nsGPhAUPISMU@LML^IT\t-1.33\nsGPhHaXISuU@LMc^HH\t-1.33\nqCcRAZTDyIj`LINXK@\t-1.33\nsGPhCQbILuS@LLI^S@\t-1.33\nsJQDDH`qBS\\t@pPOI`\t-1.33\nsJPIAxBTiZhA`Xl^P@\t-1.33\nsNq`AbeLuUPC@qU|VH\t-1.33\nsJPBHxJPt`YBOj`Gd`\t-1.33\nsNphH`xIUKUT@pvkyB\t-1.33\nsGQHHGAJejhAaecqB`\t-1.33\nsJPdE`DPHRie`FDgDB\t-1.33\nsNy`LDtfuZj@X[U|RH\t-1.33\nsGQHBLQIejXAaaMqJ`\t-1.33\nsJQdEbOBQD_M@LINKD\t-1.33\nsOp`Jtjeqje`FBqxh`\t-1.33\nsGPhCQbILuS@LDIqZ`\t-1.33\nsNt@H|ddUje@XYX|c@\t-1.33\nsJYHEAxIVuPCAX|pb@\t-1.33\nsGXHHOAIejhA`Uc^P@\t-1.33\nsJPIEPKdkZhA`h^XK@\t-1.33\nsNqhHlOAJeZj`FAAk@\t-1.33\nsNx@GddbmUPCBqXodh\t-1.33\nsNpH@xIUMUT@pjqkyH\t-1.33\nsGPhH`xISuU@LMc^HH\t-1.33\nsNq`@jdrkTpCAE|f@\t-1.33\nsKRAADQJTnKT`OLE`\t-1.33\nsNx`JDdrkUPCCFodp\t-1.33\nsNy@DDeeZj@XZX|UX\t-1.33\nsGP`@TfVj`FEDVOIP\t-1.33\nsJPhH`DIVmPCAX|`@\t-1.33\nsNp`@dfUZf@X[F|c@\t-1.33\nsJPH@xISuPCCSGbB@\t-1.33\nsNp`@df^Zj@X[U|a@\t-1.33\nsGPH@xIUMT`LBl[rP\t-1.33\nsGPhHPxISuS@LDkr@\t-1.33\nqC`HAps`dmPFBbxa`\t-1.33\nqC`HBHTPduPFFBwI@\t-1.33\nsNx@GddbmUXCCKGd`\t-1.33\nqCcRQXV@zOTAaAsQX\t-1.33\nsFy@LDigVfhAaEqF`\t-1.33\nsJPLAHJPL`dwS@OI`\t-1.33\nsJPDAxPHcvhAaIsKD\t-1.33\nsGPH@xIUMT`LCEcrT\t-1.33\nsNu`LPfdvkSHCqJ`\t-1.33\nsJPXHlPjQzt@pfyB\t-1.33\nsNqP@mVdssTpCqY`\t-1.33\nsGQ@JeTuT@pLVOIP\t-1.33\nsGY@LDizj`FFQoI@\t-1.33\nsGP`Adinj`FFQoI@\t-1.33\nqCpPP^BRu@XI[bD@\t-1.33\nqCqPZH^D^hCBCfbp\t-1.33\nsGY@DDeYj`FFqoED\t-1.33\nsJQIH@`SGmPC@aX`\t-1.33\nqCpRPA@eIj`LDmr@\t-1.33\nqCpRP^@eIj`LDmr@\t-1.33\nqCcPQXJd]XCB[d@\t-1.33\nsJQ@@eKU@LML^Q@\t-1.33\nsJP@DkVhA`Pl^R@\t-1.33\nqCbPXUbRt`XHSdP\t-1.33\nsGPP@mTf]jPGbq@\t-1.33\nqC`PBHRZhCCA[d`\t-1.33\nqCpHXRBdDfj@|Tp\t-1.33\nsBP@DiVj`FBFxh`\t-1.33\nsJPXHlQiQzl@|a@\t-1.33\nsJTILPyAdiZ|Ay@\t-1.33\nsJPHIxQ}T@prqyH\t-1.33\nqC`R@HDqz`LBHl\t-1.33\nsJP@DjVhAaecrP\t-1.33\nsJT`LPdfvdAxdP\t-1.33\nsGP`ATeUnPGba@\t-1.33\nsJPYHlQiGh}V@P\t-1.33\nsJPYHlPZGh}V@P\t-1.33\nqCr@XISRA`enR@\t-1.33\nQMA@HoAIXFD\\Y@\t-1.33\nQMACp_AIhFE\\Q@\t-1.33\nqC`@ISJAaaErX\t-1.33\nQMJd|BTf`XL|`\t-1.33\nQMFIDjN`XQqd\t-1.33\nQMFajByIhGfR\t-1.33\nqCh@CIRrAxe`\t-1.33\nQMFiK@zN`^YH\t-1.33\nRFQALXOD`\t-1.33\n`IGPCr@\t-1.33\nHiT`@@rnRfUjEnBB`PA`bMqLD\t-1.33\nsGY@DDf]j`FBVxcP\t-1.34\nQMdXZWde`^T\t-1.34\nRFDDaFCr`\t-1.34\nHev@@@rRIIRus]UUUP@pFDJfxYyF@\t-1.35\nHaE@@@aJyUnX@@@XEeMwD``\t-1.35\nsOpHAbILkW@@@LJEb~R`\t-1.35\nsKX@@eKcUPC@bX|b@\t-1.35\nsGP`ATf]jPFDwDb\t-1.35\nHcND@LCTeme^UZh@H@F@iaMpsoH`\t-1.35\nHiT@@LddTQQnFjjj@FGPfESCrN@\t-1.35\nHeT@@DjU]k``b`@XMBXUMqAd\t-1.35\nsKP`@Tixjj@XZK\\a@\t-1.35\nsJX@@dmU@LBEcrP\t-1.35\nQMA@HPaIhFE\\Q@\t-1.35\nQMBBHRZA`cr`\t-1.35\nsJX`BDdvu@LInID\t-1.35\nHeTLbHS@BL{IIKDdLkP@@@pragbMp\t-1.35\nsOq@@drm\\@@@pHM_Dl\t-1.35\nHidH@@RYfVF@b@@XXAMsJxP\t-1.35\nsGP@DiVj`FADMxe`\t-1.35\nsJPhH`DYIBm@LInHD\t-1.35\nsJQhHlOAJmj@XS\\PH\t-1.36\nqCaPQCpRfhCsAX\t-1.36\nQMJH|Df`^H`\t-1.37\nHeTB@@xYRfUVfjjj`AatES\\L|QL@\t-1.38\nsGP@DkUj`FEDVOIP\t-1.39\nsFt@AdiTt@@AaIsHt\t-1.39\nqCaPQCpRfhCBKbF@\t-1.39\nHg}B@DpAV|bbbRbrK]imUMMU@CAlJnN~IN`\t-1.4\nHcnH@LAIYe_x^fjjjj@FDhUMprn|PH@\t-1.4\nHeTD@@QIVUnEjjj@FCXS\\L|RJ@\t-1.4\nHidH@@rJJIEn`HH@FA@UMqLD\t-1.4\nKAxP@@dsjuU@FEIaTwHx\t-1.4\nKAxH@@bSJkTt@XEaTwHx\t-1.4\nsOx@@drm]UT@pXbM_I`\t-1.4\nsOq`Afdlm[UT@pbyhZ\t-1.4\nsOp`@dfUMZj`FFqoHp\t-1.4\nsJPHAVIJuPC@bK\\a@\t-1.4\nsGPhLQDIK]M@LDKrP\t-1.4\nsFy@LDi_FfhA`d^JT\t-1.4\nsJQHBGAJ^i@XJKbF@\t-1.4\nsKY@LDi\\ZZ@XQ\\TH\t-1.4\nqC`PCDRVhCCB[dP\t-1.4\nsGQ`@bdwML@pbyJ\t-1.4\nqC`PCDRZhCCBKdp\t-1.4\nQMaDbKpRZAaGFP\t-1.4\nqC`@IRtAa`mrP\t-1.4\nQMbh|BTf`XSqH\t-1.4\nqCr@XIKVAxa@\t-1.4\nRFTEPQP|h\t-1.4\nRFTPbQP|`\t-1.4\nsOq@@drm\\@@@ppUylV\t-1.4\nHidH@@RUe^Fh@@@XLATwCqHP\t-1.4\nHifDAHAHeNR[e^Eh@@@XHW\\rfD\t-1.41\nHidB@@kaReYrZjjh@X]FES\\OHP\t-1.41\nHeT@@DjU_k``b`@XCBXUMp|bP\t-1.41\nsGQhHlOAJmZhA`c^P@\t-1.41\nsOq@@drm\\@@@prsy`Z\t-1.41\nQMFIKBN`^YH\t-1.41\nsNy@DDeUZj@XEXwbc@\t-1.41\nHmtJ@@siUJYn~ajjjh@XCBDwCJ|SJ@\t-1.41\nqCaPQCpRmhCqC@\t-1.41\nsGX`LDdvmT@pVODj\t-1.42\nqC`XBPTaiAIj`OI@\t-1.42\nsNqiHl@c]ddeRuPCAF|c@\t-1.43\nQMRIOBN`^YH\t-1.43\nsGQ@@djmT@prqxiP\t-1.43\nHeT@H@bBbFbAbDfYnzXHb@@XDALODzP\t-1.44\nHif@`D@HRUe^EX@@@XTATw\\`P\t-1.44\nKAxH@@bSZsUT@XTaTw\\c@\t-1.44\nsNy@aAL{IILZj`FAVMyF\t-1.44\nsGY@aALyJkjhAaecrT\t-1.44\nsNp`@dfUZf@XHwbs@\t-1.44\nsJY@aALzOj`FBVyB\t-1.44\nqC`@IVtAa`mrP\t-1.44\nQMPBchFBWI@\t-1.44\nsJP`@TeZhA`QL^Q@\t-1.45\nsJQiHlOC\\eVu@ODB\t-1.45\nsJT@@TeZhA`QEnP`\t-1.46\nqC`@IWtA`hnS@\t-1.46\n`IoQHFD\t-1.46\nHidD@@QInUxV`@@A`RES\\Sa@\t-1.46\nHet@@DjYUX^d@@@@AarXUMp|bP\t-1.46\nqCb@AIZ`LLInQ@\t-1.46\nsOpH@DILkW@@@LJlZ~P@\t-1.46\nj`iA`@@V|eLsJnmARMiu@A@DT@@LFDYqQg\\pJw`\t-1.47\nJoBAQDIg@DYvRRVJKIIRmnmA@@@@AaS]ioqAl\t-1.47\nJoBAQDIc`DYvRRVIJYKCIvm@@@P@AaULJoqML\t-1.47\nHknJcEaLtp|LyYOdj~UvFVh@I`@XES\\OH@\t-1.47\nJobB@@VeLsJntLgkt@D@Q@@FCDwJ[|p]l\t-1.47\nHmL`B@aBB\\dRabRpiUj`@P@XTwCJ|PL@\t-1.47\nHk^@@@rRJFIQDYtz`@@@@F@ie\\Lz_HT\t-1.47\nHmtBbHSBCpKlddqbReVj@@@FAMpsqLT\t-1.47\nHcNBAHAEvISdfyW[aZ@BX@FAES]rM@\t-1.47\nHknJ@D@ae\\bbbRtJifjiZh@XYc[rC@\t-1.47\nHcLBbHSBCpKdjm{U]Zj@@@FA\\LkqFt\t-1.47\nHid@p@bFbAbDfUfn`BH@FE@fLOHx\t-1.47\nHeVD`Aa@BLdabRgRl@P@C@bkix`B\t-1.47\nHid@P@bH\\Df[Waj@@@FGAaTwCrH@\t-1.47\nHcNJ@LANF\\bbTRbILuTmT@LH[bKh\t-1.47\nHmMH@LhDigm\\jUid@H@FAMpsqDT\t-1.47\nHeT@`@\\DieUzZBA@@XBfES\\L|`@\t-1.47\nHmtH@@RYeY[hBBh@FGAaTLkrN@\t-1.47\nHifDAHAHeNR[e^Eh@@@XXwCqBh\t-1.47\nHmvJ@LANFTfVVRYjiZ`AaC\\RU@\t-1.47\nHeTH@@RYe\\YjB@@AapES\\L|RL@\t-1.47\nHcLH@@RYeYVz@`j`@X\\FEProHt\t-1.47\nHaD@b@|@eJ[Unh@@@XLfES]r@@\t-1.47\nHidH`ACDRYWZZ@BP@XTBXU\\c`\t-1.47\nHiTH@@RgeXSaj@B@A`tES\\OHP\t-1.47\nHaDH`HApRnU[j@@@FCQaTw\\`@\t-1.47\nHeTD@@YIfUqehH@@FEEWCODq`\t-1.47\nHid@`@qDee]nh@H@FCAFES\\`P\t-1.47\nHid@@LddRL[jjj`A`tIa\\ODi`\t-1.47\nKA|Y@]BX@QddeRVj`C@lJfyG@\t-1.47\nHeTD@@QIemQej@@@FAESSqND\t-1.47\nHifH@AAJ[W[j@B@AaRXUMrA@\t-1.47\nHif@@@RfU~F``@@X\\BXS\\OHh\t-1.47\nHif@@@RUe^Fh@@@X\\DHUMwHD\t-1.47\nHidH@@RUe^Fh@@@X\\DHUMwHD\t-1.47\nHaD@P@dNdDfUZZ@B@AabDwHp\t-1.47\nHif@@@RYWZZ@B`@XLBXUCrN@\t-1.47\nHeVB@LANEIeeRYjiZ@FDMqJd\t-1.47\nsJQL@hCPA`DijhAqd`Y@JArP\t-1.47\nHaDH@@RVU[f@@@FEQaUwHX\t-1.47\nHaF@@@Re]J`@h@FEPfDwHx\t-1.47\nHaD@@DjWzXHB@A`TYW\\QB@\t-1.47\nHaFL@LAFRYUJfjZ@FDUqLx\t-1.47\nKAxD@@c\\dwLuS@FFIg\\b`\t-1.47\nKAxH@@rTzmST@XDfEwDb`\t-1.47\nKA|P@HBSJsTt@XTfTw\\``\t-1.47\nqC`XBHTQhAIj`NIdBBByH\t-1.47\nsOpHAxILkWUU@LGEcWr@\t-1.47\nsGQHHGCIHcUPC@kF|`@\t-1.47\nsOx@@drm]UT@pXbqoHp\t-1.47\nsGQHHGCIHcUPCCHwbF@\t-1.47\nsNqHHGCIHcJi`FFV_DJ\t-1.47\nsGPH@DIJmU@LJHl^R`\t-1.47\nsNp`Jtjejj@XVXu|b@\t-1.47\nsJPDAbGDRUj`FADqyD\t-1.47\nsGXA@IrRHuR@plVMy@\t-1.47\nsGPhCQDILmR`LDIqK@\t-1.47\nsJQDDGBQD_U@LML^Q@\t-1.47\nsNy`LDtfuZi@XKGbU@\t-1.47\nsNx@@djoUPC@kF|c@\t-1.47\nsNp`@dfvZj@XEZodH\t-1.47\nqC`HBHtPdmPFFDwH`\t-1.47\nsGQHDHaInfhAa`[rX\t-1.47\nsNq@@dssUPC@hu|f@\t-1.47\nsJPHAXIRuPC@qX|`@\t-1.47\nsGPP`cUuyInZXA@\t-1.47\nsJQ@@eMU@LLl^R@\t-1.47\nsGQ@@dsMT@prMyL\t-1.47\nsJQ@@dmU@LBHmrD\t-1.47\nQMCCDoAqDf`XH|h\t-1.47\nsGT@ATeWfpGbV@\t-1.47\nQMABHWAIhFBWI@\t-1.47\nQMBcpIchFCOH@\t-1.47\nqC`@IS^Axe`\t-1.47\nRG@aMt\\OH@\t-1.47\nQM`PfzO`^R\t-1.47\nQMFIOBMP^P\t-1.47\nQMPARUAxb\t-1.47\nHaFH@BAIf]n``@@XYe]s@I`\t-1.48\nHg|H@@RfUYwySn``@@@@XJfDwCNg_HT\t-1.48\nHet@@DjYUX^d@@@@A`rDwCODr`\t-1.48\nsGX`BDdvmT@prqyJ\t-1.48\nHifDAHAHeNR[e^Eh@@@XXUMqND\t-1.48\nRF@Qp|R@\t-1.48\nsJPXhlQx[ct^u@OH`\t-1.48\nsJPhHaxIRuPCsCD\t-1.48\nQMFIOBN`^YH\t-1.48\n`IK@Cr@\t-1.49\nHaD@@DjWjXHB@A`f]phRf@\t-1.49\nsGQ@@eNuT@ptqxeP\t-1.49\nqC`XCDVIlQIj`LLEnR@\t-1.5\nHif@@@rRJEKaj@@@FCIaTw\\`P\t-1.5\nHaDH`H@HRnU[f@@@FEXUMwH@\t-1.5\nsGP`AdeUj`FBMxh`\t-1.51\nsGQ`@jdrmR@|VH\t-1.51\nsNx`DFdkKUHCCKWba@\t-1.51\nqCh@CIKTAaaErX\t-1.51\nQMBCDRYAaGFP\t-1.51\nHiT`@@rnRfUjEnBA``AabMwHD\t-1.51\nsJY@DDfvdAaMqH`\t-1.51\nsJQ@@dkU@LCEcr@\t-1.51\nRF@QPpce@\t-1.51\nJoBA@@YviSJwZ}C[uUTDA@@XSA\\LkiwsDip\t-1.52\nJgBD@@YHhhhddZFSvz@B@B@@LFInT{y`sh\t-1.52\nHcND@BADf{YU]Zj@@@F@ie\\LkoHH\t-1.52\nHid@`@BDiyWaj@@@FGAATwCrD@\t-1.52\nHeTD@@eJ[WVz`@h@FEIaTwDFP\t-1.52\nHidH@@RYWZZ@B`@XLBXUCrN@\t-1.52\nsOx@@drm\\@@@pHM_Dl\t-1.52\nsNp`ATfUjj@XTQF|VX\t-1.52\nsKP`@Ti\\Zj@XJ[dP\t-1.52\nsJQ@@dmU@LML^HH\t-1.52\nqCcRQX^FzNtAaEr@\t-1.52\nsNx`BDdw[UPCCKWbK@\t-1.52\nHmtL@@QTfyeQehBA@A`ULL|QN@\t-1.53\nHk^H@DAInUm^GSZ@@@@@XLWRngqDh\t-1.54\nHmNh@DkaTDfVYVzUZiYj@FFHL|aP\t-1.54\nHmLD@@QIe[VfeVi@B@A`U\\J|SI@\t-1.54\nHaF@@@RYe[hB@@FEAaTwHx\t-1.54\nKAxD@@c\\dr|uS@FDYqBx\t-1.54\nsNu@EPdrkUPCCKWbK@\t-1.54\nqC`HBHc`dmPFFBwI@\t-1.54\nsGQ`@bdwMT@pvMyD\t-1.54\nsGP`@dfUjPFBuyB\t-1.54\nsJP@Dk^hAahmrD\t-1.54\nqCa@BIWTAaMqQ@\t-1.54\nQMABHcAIhFBWI@\t-1.54\nqCaPSAXRVdCqA@\t-1.54\nsJP`@dfvlAxdP\t-1.54\nqCb@AIV`LEMrH\t-1.54\nqCp@AI^`LInQ@\t-1.54\nRFBFIFDXLHyP\t-1.54\nQMBBpRZA`sr@\t-1.54\n`IC`Cr@\t-1.54\nqCq@XIIf`ODl\t-1.54\nsJP@DiVhA`QD^S@\t-1.56\nsGP`@TfUi`FDwDj\t-1.56\nqCj@KAIV`LDmrP\t-1.56\nHev`@@rfeJY{ZxYBBFH@XXfCrI@\t-1.58\nHmN`@@rfeJY{^nFP`ah`AabXOHT\t-1.58\nsGP`ATiVj`FAbuyB\t-1.58\nJoBD@@ISZzrk][t@E@@@@XJfESKiv\u007FHD\t-1.59\nHg|@@DjU^Uzx{BB@`@@FDpaTLki|SK@\t-1.59\nHeV@@@RfU\u007FahHB@A`rXUMwDBP\t-1.59\nHie@@@aJVuxV`@@AaQTwCrL@\t-1.59\nHaE@@@aJyUnX@@@XUaTw\\b@\t-1.59\nqCb@AIVPLInJH\t-1.59\nHg|H@@RYfUWd}mh@@@@@XBUMprn|P{@\t-1.59\nj`qAB@G@bDfYeU\u007Fd[Uj`@@B`@AahAJLxJRmV}rD@\t-1.6\njdyP@@@^Rfuue]f\\lyjVX@Jh@AaiFBTXkQkAcr@`\t-1.6\njhyQ@@LT@eL}klkbde^CSH@P@P@@pds`iFl{p^PP\t-1.6\njhia@@G@RYfUW~d[Uj`@@Bh@@XFA@eF\\EIVoAyBp\t-1.6\nj`q`@@@YIEBedhdnB]zh@J@@@@XZQ`eF\\tZso\\aH\t-1.6\nj`q`B@K@|DifUW~EiQj@@@J`@AakAJLxJRcN}r@@\t-1.6\nHgl@DLdBdJdFdAdIdCdKdLddTbdYeUUTt@LD{^Ph\t-1.6\nj`q`@@@YIEBedhdnB]zh@J@@@@XZQ`eF\\ehug\\ax\t-1.6\nj`q@`@@LRYfWg^Qg^jB`@@@@FE`PIQ`iJtZso\\a`\t-1.6\nj`qH@@@P]yJVYU_yFuYh@@@d@@XRIQgARUjwH^@\t-1.6\njhiP@@@NRYfUW~d[Uf`@@Bh@@XZ@RcNBdkW`|aX\t-1.6\njhi`@@@ISLwZkf|xMUP@@@@@LO@``cNBdkQkN|D\t-1.6\nj`qa@@D@RYYUg^Qg^Z``@@@@FDdTXkQkN}sAAR@\t-1.6\nHglLxJ[hBLdBdAdIdCdKdLddTbdYeTmUT@LHwdJ\t-1.6\njdyP@@@QrJJJJjIJILEYtADP@U@@CAPBTYqVoAb\t-1.6\nj`q@@@DjYVWWyjwhI@HA@@A`YFJLxJRmFm{dG@\t-1.6\njhia@@G@RYfUW~d[Uf`@@Bh@@XJIQgARUkp^Pl\t-1.6\nHg^BpLaIT@b`aiCIICEELb]kU@h@@LJx]N~QP\t-1.6\njdip@`OAbBPIHJdKRBSLwKKnpTuSMU_h@OHR@\t-1.6\nJoB@Q@^@SddbbaRRrv[uP@@@@@XVFES\\Lkivx\t-1.6\nJoB@P@NFRRJJEIKKYoU@@@@@AanTwCJz]orB@\t-1.6\nj`q@`@@^RVUvUugCVX@H@I@@F@lDhrRmFxbC`\t-1.6\nHkLLHDYP\\HbJ\\I\\LddqdQdjjjj@FGQa\\NgrE@\t-1.6\nJoB@H@^@~BULrj~mf}PA@@@@FAaaTwRng[|`P\t-1.6\nHctHXMRPrPjPZPFPfPrRQRFDuUUT@LLy]yA@\t-1.6\nJoBDH@QPIPIS\\rjzv[uP@@@@@XKS\\Lkiv\u007FHP\t-1.6\nJoB@@FRQQIQKH}moAHA@@@AajYS\\Lz]orE@\t-1.6\nj`aP@`G^AHDdERAiAIf[eWvBfjYjoP@^HyH\t-1.6\nHkn@PHApiprRQJFIGSjhJ@@A`lXS\\Lki|b`\t-1.6\nJoC@PD@xIU\\vjzv[uP@@@@@XZUMprng[|b@\t-1.6\nHg\\BPHSS@HxHrRPqQISKZtD`h@CAjXYV~Qp\t-1.6\nJoBAPDIa`DYIIUHiEEMrzuUUMT@F@xPrnvx\t-1.6\nJoBDPD@xITlvjzv[uP@@@@@XZUMprng[|`@\t-1.6\nHgmL`JULHOCIIEIRdiMUUU@CA\\JfxYT{yF`\t-1.6\nHg\\@x@dBdFdAdEdCdKdDfYn\u007FnfBHbj@AyF`\t-1.6\nHknBPLaFx@``cIHXhhTmMZe@@@FECJz_D@`\t-1.6\nHg\\JPHSAd@b`cIIBhhd]ikTB`P@LJ[ev~QP\t-1.6\nHo~L`LaC@HrRPjJIKG[VhJAH@FGYS\\noqAd\t-1.6\nHklBPHQ]@HxHrRPqIJJSV``T@A`qTpr_Hx\t-1.6\nHklDPDp@b@cIIDXdmevV``H@FGQS]JgrA@\t-1.6\nHklHPHApiprRQJFIGSjhJ@@AaNDwCJz_Hh\t-1.6\nHctLpHS`\\J\\F\\LddRLbeUUU@CClIae]yG@\t-1.6\nHg^L`LaM@HrRPjIIHjZuA@d@CAl[iV~IP`\t-1.6\nHedFIDpnFPBHBJBFBBz^rRQURRuK@CAHh\t-1.6\nHg}@@@aJVYU^Svz`@@@@A`HMprnwphJc@\t-1.6\nHkLLPDtp\\ABLddTbLLjjje@FGXUMtkrA@\t-1.6\nHmTHHDApbHipVHRjvYjjj`A`LXS\\LkrJ@\t-1.6\nHcMB@hDDWXeNF]yInUnzV`BF@AaaUwHd\t-1.6\nHg|H@@RYfUWd}nh@@@@@XJDES\\LkoDNp\t-1.6\nHg\\D@HkIIDhTeDmuTDE@@LIpfzUt{y@`\t-1.6\nHcLDPDp@b@cIIDYdmJrt@D@CAh[ag^QP\t-1.6\nHmTDpHPGBgAgCIIDcEJjjh@X]aUproHX\t-1.6\nHcL@P@|I|DifUl]hJB`@FDiaTwBn|PI@\t-1.6\nHg}@@@mIe]e^ftx@H@H@A`HHUCJwqLt\t-1.6\nHg~@@@RYfUWd}mh@@@@@XJUMprng_H@\t-1.6\nHedLhDp`BJdFdAdIdLddTgdmUP@|Pn@\t-1.6\nHg|H@@RVUvU[cm`@`@@@XCaTtki|RK@\t-1.6\nHmtDPDp@b@cIIKDeafV@B@AaUMpkrH@\t-1.6\nHmvD`La@BLddNbReUjjj@FCTwCJ|SB@\t-1.6\nHk\\H@@RfYU\\]Tz@@@@@FDxUMprn|`p\t-1.6\nHmV@PHApEprRQQMRjjj@F@iaTwRoHD\t-1.6\nHeVD`La@BLdaTRPrmUU@CAbinF^HF@\t-1.6\nHiD@H@\\HbNdAdLddqTjjh@XUBXU\\a`\t-1.6\nHmtJ@@IaUIf[oijjjh@XJDIaTwBoHd\t-1.6\nHmUH`Dr@|LdbTLTjjj`A`HES\\LkrL@\t-1.6\nHmtD`HP@cIICLeaeZ@B@AaUMpkrB@\t-1.6\nHmtD`HP@cIICDeiUZA@@A`qTproHx\t-1.6\nHmUD`LJD@HrRPqIIZjj@FCTwCJ|``\t-1.6\nHk^D@DBTfYYwimMjdDB@A`vMpkoHL\t-1.6\nHiDDpDXGBHbgCIILYJjj@FEQa\\OHX\t-1.6\nHmtD`HP@cIICDeiUZB@@A`qTproHx\t-1.6\nHmU@`HPGCIIEDTjjjh@XCaTwCJ|a@\t-1.6\nHedAQHSBkad@a``nglddNTbuRp@`\t-1.6\nHcNJ@D@ae\\bbbRtMLuUKT@LDqnQh\t-1.6\nHcL@@DjU_VnBAId@FBpfES\\LkrM@\t-1.6\nHeTL`HRPBLdabRwBl@D@CBbingd@\t-1.6\nHmLD@@qJY{WJeZj@B@A`tMproDR`\t-1.6\nHmvH@H[IIEDYNFjjj`AarXSCJ|c`\t-1.6\nsFpF@DSpPQ``~EBDf_EjXA`Pm^JD\t-1.6\nHk\\H@@RfYU\\]Tz@@@@@FBxUMprnf\t-1.6\nHmt@`@|DifUqV`hH@A`JXS\\Lkq@h\t-1.6\nHiDHPDApbHrRPqRjj`AatXUMp|a@\t-1.6\nHg_@@@jteem]YWizV[`A`S\\L|cP\t-1.6\nHiDHPHApIpRiYjjj@FG@fTwCrB@\t-1.6\nHcLH@@RYWeUuiX@@@XBfEprn|b`\t-1.6\nHiED`LJD@HrRPiIZj`A`UMp|SB@\t-1.6\nHaD@R@bNhNyIe^f`@`@XTBXU\\c`\t-1.6\nHeV@B@B\\dbaRwBuRs@CAdpj[nQH\t-1.6\nHmV@@@rRJJIjjjj@FDpfES]J|`P\t-1.6\nKAxUBPfEp^F\\WYIIbfjh@pRnyC@\t-1.6\nHeth@DkiAIeeVxYZiVh@XH`|Pi@\t-1.6\nHiE@`HPGCIIDbjjj@FCXUMp|RD@\t-1.6\nHeU@@@eJYW~F``H@FGIaTwSr@@\t-1.6\nHmtD@@QImYVUZh@@@X\\fMproHH\t-1.6\nHcM@@@uIVUux^ZV[`A`QTw\\QI@\t-1.6\nHiDNPHSB[a@XhXrRPzQZe`A`aB\t-1.6\nHeT`@@`YRnUunX@I@A`VES\\RI@\t-1.6\nKAxD`Pc\\@QPQddJTVYPCALKbAP\t-1.6\nHif@`HApRkm^Fh@@@XMaTwCq@`\t-1.6\nHeTL@HP\\ddRUQ]UUU@CBlJfgd\\\t-1.6\nKAxH`P`NENFRRJrjj@LJpj[nP`\t-1.6\nHif@B@B\\dbaRxVjV`A`rXUMwHD\t-1.6\nHmu@@@UJf]|YjB@`@X\\aTwBoHp\t-1.6\nHaD@P@bH|DeeVz`@@A`pXUMwH`\t-1.6\nHeeH@DhLbbTJcUUP@pyLinF^PP\t-1.6\nKAxY`Pf@DMH]HYIHUEjh@pbnPP\t-1.6\nHiDB`HSACprRSIEjjPAaaTwHx\t-1.6\nHeU@@@gIHhTmpu@A@@pyLJfzP\t-1.6\nHef@@@rRJJIuUU@C@XPj[agdX\t-1.6\nHmvJ@LANF\\bbTReIfjej@GbLh\t-1.6\nHeg@B@J`gIHhXhuUT@LFSFxY@\t-1.6\nKAxAPAHIHEHMH]HIMwTuPAyE@\t-1.6\nHidL@HP\\ddRNijji`AafEWD[`\t-1.6\nHeTD@@eIYWVy`@d@FFXU\\rFD\t-1.6\nKAxQ@PC`eWMUU@FG@fES]rH@\t-1.6\nHeTH@@RYyTYZ`@@A`rTwCOHp\t-1.6\nHifH@AAJ[W[j@B@AaVESCrD@\t-1.6\nHidH@@RUe^Fh@@@X]BES\\OH@\t-1.6\nHaDL@@[die\\jjj`AaTXUMqDx\t-1.6\nHiFD@LADfWYjj`A`VMp|Pj@\t-1.6\nHifH@DAIVUxU`@@A`QTw\\`P\t-1.6\nHiEH@DHDf{Yjj`A`vES\\OHP\t-1.6\nHiD@@LdbJTZjh@X]BXUMwHD\t-1.6\nHaDD@@YJ[tjjjh@XDfEwDb`\t-1.6\nHeVJ@LANFTfVVIfjeh@^Ht`\t-1.6\nKA|H@XBIMkMUPA`VTw\\QB@\t-1.6\nHaFH@FAIe^fZjh@XDfEWHx\t-1.6\nsGPL@xRHTQddajhA`Xl^R`\t-1.6\nKAzP@I`ILsUUPAaTES]r@@\t-1.6\nKA|X@DB\\dwZuT`FAIaUrF@\t-1.6\nHid@@Di[Waz@@@FEES\\OHp\t-1.6\nKAz@@@VRQPrjZ@LBSF{dT\t-1.6\nHifB@LANEIeeIfje`AxeL\t-1.6\nsNxhMDOCIHUJj`FAAkyhl\t-1.6\nHiF@@@RifZjj@F@pfES\\H\t-1.6\nsNpD@DSpRYyjhAaPcWqS@\t-1.6\nsGQDHGCNBTmUPCCHwbF@\t-1.6\nsOpH@DILkW@@@LJEc^Q`\t-1.6\nsNpD@xRHRjzjhAaYcWrH\t-1.6\nsNqhHfOAJmvjPFBqxaP\t-1.6\nsNpH@xIRkUJ@p\\VM_H@\t-1.6\nsNt@@|dTQjj@XTKF|TX\t-1.6\nsGTHHaxISuU@LMc^XQ@\t-1.6\nsNqHHGAJeZj`FAVkx`P\t-1.6\nsOq@@drm[Rt@pJqoHp\t-1.6\nsJPhH`xYICU@LEcsBH\t-1.6\nsGP`ATefj`FEDqoH`\t-1.6\nsJQHHGAJfj@XYX|`@\t-1.6\nsFx@@eJf`@@LEcsQT\t-1.6\nsJQHHGAJfj@XIGbF@\t-1.6\nsOy`LDtfUMZi`Gbu@\t-1.6\nsNxA@NRfZjhA`xlZp\t-1.6\nsJQHHGAJni@XYX|P`\t-1.6\nsJQHHGAJVj@XIGbF@\t-1.6\nqC`H@Hg`dsPFDGME`\t-1.6\nsJYHEAxIWMPCAX|`@\t-1.6\nsGQ@@dsuT@phbqyJ\t-1.6\nqCbPXXbRu@XHSbV@\t-1.6\nsJP`@dfzhA`l^XQ@\t-1.6\nsJQHDHaInZ@XHGdp\t-1.6\nsOq`@fdrikUL@|Vh\t-1.6\nsOq`@fdrikTl@|Vh\t-1.6\nsKQ@@eKcUHCAX|TP\t-1.6\nsGPP@cTfyi`GbU@\t-1.6\nqCcPQXCD]XCB[d@\t-1.6\nqC`PCDRZhCCA[d`\t-1.6\nsJP`@dfvdA`l^Q@\t-1.6\nqC`PApRVdCCQ[d@\t-1.6\nqCcRQXUfzNlAa@\t-1.6\nqCb@AIZPLLEnR@\t-1.6\nQMIDbIpRZAaGFP\t-1.6\nsJY`BEdfVhAxlP\t-1.6\nsGQ`@ZdrmJ@|Rp\t-1.6\nqCb@AIVPLEErX\t-1.6\nQMABlYAIhFD\\h\t-1.6\nsJQ@@djw@OEB\t-1.6\nRFBBYQDXLJy@\t-1.6\nRFTPeQ`|R@\t-1.6\n`II`Cr@\t-1.6\n`II@CB@\t-1.6\nHiFL@BAAR[mVji@FEIaTwHx\t-1.6\nqCaBAKdfZ@xTlMXPf@\t-1.6\nqCaBAKdfZ@pf@\t-1.6\nJoBA@@XNiSJkN}][uUTDA@@XSE\\LkiwsDIp\t-1.6\nHmLL`HS@BLddJfRiiUZ`@`@XEproHh\t-1.6\nHcLB@@RUR[fVQuhHF@@XEWCoDQP\t-1.6\nsGY@JDeUj`FFVoEB\t-1.6\nKA|H@DBIMvmUPAaRXUMrN@\t-1.6\nsGX@JeTuT@prqxiP\t-1.61\nRFTQxQP|`\t-1.61\nsJQ@@eKU@LEEqX`\t-1.61\nqC`@Qz`LMEnP@\t-1.61\nHiDD@@QIeuZjh@XUaTw\\`P\t-1.62\nHidB@@[aRfU\\jjjh@XMFESCqDx\t-1.63\nsGP`@dfuj`FFVoHP\t-1.64\nsGQ@@dkMT@prqxiP\t-1.64\nKAxH@@bSKkUT@XEaTwHx\t-1.64\nHeTD@@QInYQehB@@FATpsqDx\t-1.64\nKA|H@HGIL|mUHAabXWDi`\t-1.65\nsGQ`@ldsmT@paxmP\t-1.65\nHaD@`@bDeeVz`@@A`pXUMwH`\t-1.65\nsJP@DjZhA`Pl^R@\t-1.65\nHmtJ@@XYUJYY^njjjh@XCFEWCJ|QF@\t-1.65\nsJU@E`dru@LH^IX\t-1.65\nsJP`@TiZhA`inID\t-1.65\nHidH@@Rfufzjjh@XMFTwCqDH\t-1.66\nHaGH@Dq`RYVkffi@FFTw\\c@\t-1.66\nHaED@DpFRYVkfjY@FFIW\\c@\t-1.66\nHaDH@@RVU[f@@@FAQaUqFx\t-1.66\nKA|H@LEIRjuUPAaTIaMrN@\t-1.66\nsNx`FJeJmUPCBqXodh\t-1.66\nsNx`FDdvkUPCCHobu@\t-1.66\nsNq`@fdr{UHCAX|e@\t-1.66\nsJY@DDeUhAaMrD\t-1.66\nQMA@HmBN`XI\\d\t-1.66\n`IM`Cr@\t-1.66\nHev`@@rfeJY{ZxYBBJD@XDfLH\t-1.66\nHidH@@RVUvy`@`@XEaTwD`P\t-1.66\nsGY@LDemj`FFVOEJ\t-1.66\nsGQ@@eJuT@ptVxcP\t-1.67\nHeTH@@RYe\\YZB@@A`vEWCOHX\t-1.68\nKA|H@DBIMwMUPAaRXS]rJ@\t-1.68\nqCrAPCiJSU@XI[d`\t-1.68\nRFTQxQ`|R@\t-1.68\nsJP@DkVhA`QEnP`\t-1.69\nqCp@AIZ`LDMqQ@\t-1.69\nRF`EFCBNT\t-1.69\nHmv@B@B\\ddUbReZj@@@FDpfDwCJ`\t-1.69\nsNq@@djkUPC@kF|TX\t-1.69\nHifDAHAHeNR[e^Eh@@@XXS]qJD\t-1.7\nsOpHAbILkW@@@LJDZ~S@\t-1.7\nJoBD@@ILrkNx{[uA@@@@@XFDEWCJz]orL@\t-1.7\nHmtD@@QImYVUZh@@@X\\aMproHp\t-1.7\nsJQ`@bdvt`LHnS@\t-1.7\nHaD@@DjfZzjj`AaVES]qHP\t-1.7\nqCcPQX^D]hCBKbF@\t-1.7\nQMABPYAIhGe@\t-1.7\nsGP`@dfUjPFDoEb\t-1.71\nHmt@HHdDdLdEbLddlRVFZh@H@FATwKrA@\t-1.72\nHo}L@J[SoIIEKEELQfjjjYh@XMBXPkrG@\t-1.72\nHg]L@J[SoIIEKEELLuUUSL@LFaLHUyA`\t-1.72\nHcl@@DjYU_egX@@@@@XRfDwCJ{rJ@\t-1.72\nHedLpJFXdDdLdLddlreTtp@panQH\t-1.72\nHcLL@@STfue^UZh@H@FCDwCN|QL@\t-1.72\nHmTB`HZUAprRQVYRjff@FAIa]rE@\t-1.72\nKA|Q`MTQBQFQFRRVJjf@LBaL[d\\\t-1.72\nHefDB@FPgIIETiUUL@LFaLIiyE@\t-1.72\nHeTH`ACpRfUWidHD@A`rXS]ODB`\t-1.72\nHmtH@ErRQRiHYj`@`@XLfETkqFx\t-1.72\nHmtH@IrRQRiPUj`B@@XLfESSqAT\t-1.72\nHmtH@NrRQRqZUj``@@X\\fDwRoHD\t-1.72\nHeVD@DClbbbRILuRu@C@liayD`\t-1.72\nHedL`HZP\\LddUfUUMH@pILKnPH\t-1.72\nHeV@@@Re[TYj`@@A`HPUMpsrD@\t-1.72\nsNplJu@Z@L@eTuTpCcI@r@TCdX\t-1.72\nHiDD`HX\\SIIEYjjZ@FAIaCrN@\t-1.72\nKAxI`MtRBRFRFRRURjf@LH{dD\t-1.72\nKAxP`PCaSaddbjjjPCBdpf{dT\t-1.72\nHeTD@@yJ[VvF`BH@FEIaCOHX\t-1.72\nHaE@@@aJyUnh@@@XLDXUMrN@\t-1.72\nHefD@NalddUfeUMP@pILH^QH\t-1.72\nsNyLGYDJHtQddeeUHC@qY|e@\t-1.72\nHiDD`HXGCIIEYjj[@FFIa\\`P\t-1.72\nHiFD@Jx\\ddUTjjX@XDfEWD`P\t-1.72\nHidH@@RUe^Fh@@@X\\BES\\OH`\t-1.72\nKAxI@MpNFRRJrjf@LBaLKd\\\t-1.72\nKAxI@P`NBUKmUT@XUaTw\\`@\t-1.72\nHiFD@NalddUfjih@XDfDOHx\t-1.72\nsGPJHBJPt`YBubUSUPCBWd`\t-1.72\nHiFL@AanrRQVZjf`AabXWHD\t-1.72\nsNpBHxJPt`YCIIEZj`FBkyH\t-1.72\nKA|Q@QpNFRRJVji@LBSB[dT\t-1.72\nsNqLKHaQFbLddljf@XZK\\c@\t-1.72\nKAxI@PpNFRRJrji@LLSBxc\\\t-1.72\nsNplByDJHtQddeUUHCCQ[dh\t-1.72\nKAxQ@\\CaddbjjnPCCDpnPp\t-1.72\nHaD@@DjWjXHB@AaTIc]rJ@\t-1.72\nsGXLKHaQFbLddmT`LCEkrD\t-1.72\nHif@@@RVUv{`@`@XEaTwHD\t-1.72\nHaDH@@RYe[hB@@FA@UMqLx\t-1.72\nKAxD@@c\\dvlts@FBXWDS`\t-1.72\nKAxH@MvRRJrjf@LLSBxd\\\t-1.72\nKA|@D@DiUJuUPAatIaTw@\t-1.72\nsNpD@xTHrRQVjhAaXl[rL\t-1.72\nsGPdE`DRHrRRrt@pTOIP\t-1.72\nsJQLCHaQFbH~j@XZK\\a@\t-1.72\nsO|HEfHaIeZx@@AaAqF`\t-1.72\nKAxX@PfIUJuUPAaeMwHH\t-1.72\nsNqHFGCIIEZj`FEbqoHp\t-1.72\nsNqhHl@cIIJej`FFqoHp\t-1.72\nsOpH@bILkW@@@LJlZ~P@\t-1.72\nsNyHHpxYIHkUL@prqxjp\t-1.72\nsNyHG@xYIHkUR@prqxjp\t-1.72\nsJXBB@bRBdMHQ}T@p`yB\t-1.72\nsOpHAxILkW@@@LBl[qA`\t-1.72\nsFxIHO@RdrfuU@LBl[r@\t-1.72\nsGPhH`xYIDZj@X[F|PP\t-1.72\nsGQDHGBtBUSUPCCKGdh\t-1.72\nsGPD@xPHRjZj@XFKGdh\t-1.72\nsGQhHlOCIIJuPCCKWdH\t-1.72\nsOy@FDfUkZj`FAVMyF\t-1.72\nqC`XBlTaiAIj`LHNR@\t-1.72\nsJPIEPKdkZhA`QL^Q@\t-1.72\nsGXHJGAJijhA`Xl^R`\t-1.72\nsGQHJGAJijhA`Xl^R`\t-1.72\nsGQ`AbeLuT@ptUxlP\t-1.72\nsJPH@DIJuPC@bK\\a@\t-1.72\nsJPhLQDIKTpCA@|d@\t-1.72\nqC`D@Hd`iCRD^hCr@\t-1.72\nsJPH@xIKUPC@qX|`@\t-1.72\nsGT@A\\dbMU@LMC^IX\t-1.72\nsNqA@IRiVjhA`xlZp\t-1.72\nsGQHI[AJijhAaecrT\t-1.72\nsJPH@xITuHC@qX|`@\t-1.72\nsGP`ATeejPFBQxi`\t-1.72\nsGP`ITjfj`FFVOIP\t-1.72\nqC`DBPe@iCRBSU@P\t-1.72\nsJQIHGB]GuPCCKD\t-1.72\nsGX@HeTuR@prqyJ\t-1.72\nQMCAxhabHzAad`\t-1.72\nsGT@ATegipGdH\t-1.72\nRFTE@Q`xi@y@\t-1.72\nQMBCpchFCOH@\t-1.72\nQMHAIdFD|R@\t-1.72\nQMbDBDfp^P\t-1.72\n`IDhCr@\t-1.72\nsNq`@VdsMUPCBbKGbk@\t-1.72\nsGQ`@ZdruT@pHboEb\t-1.72\nsGQ@@dsMT@pQoDl\t-1.72\nsNq`AVdlmUPCAbXu|b@\t-1.72\nQMFIOBN`XUqD\t-1.72\nqC`HBPT`duPGbf@\t-1.72\nHeTD`HP@cIHXdmpk@A@@phj[iy@@\t-1.73\nsOx@@drm\\@@@phVKyJ\t-1.74\nsNx`DFdskUHCAK\\Qh\t-1.74\nHaDD@@yIe^f`@`@XHa\\HLk`\t-1.74\nHet@@DjYUX^d@@@@A`rXS\\ODjP\t-1.74\nsGPYHlQxGdjmj`Gbu@\t-1.74\nHifDAHAHeNR[e^Eh@@@XXUCqFx\t-1.75\nHaF@@@Rfu[i@@@FAXUMqDx\t-1.75\nsGQ`@bdwMT@pVODj\t-1.75\nHeTJ@@qaeJYyzzjjj@FCPUMt|qLD\t-1.76\nsJX`BDdvu@LDmrD\t-1.76\nHmVBaNaLHO@nz\\ddqRdZjj`AaULLkrL@\t-1.76\nHidD@@QInUxV`@@AaRESCrN@\t-1.77\nHaDH@@RYVijjj@FCPaTw\\PL@\t-1.77\nsJPhLQbIKTpCBCba@\t-1.77\nsJPhI`xIKTpCBGd@\t-1.77\nsGY@DDfUjPFFVoHP\t-1.77\nsGQ`@jdvmT@prqyJ\t-1.77\nsGP`ATf]jPFBVyB\t-1.77\nsGX@@dj}T@prqyJ\t-1.77\nqC`PBlRVhCCBKdp\t-1.77\nqCaPS@pRVdCqA@\t-1.77\nQMhDRYAaOD`\t-1.77\nsOx@@drm\\@@@pHU_Eb\t-1.77\nsOp@DjWkB@@FAfkxdP\t-1.77\nHidH`H@HRnUvy`@`@XUaTp|`@\t-1.77\nHiDD@@QIeWZjh@XUaTw\\`P\t-1.78\nRFDBIFCqH\t-1.78\nHidD`HP@cIHXdnEX@@@XTUMp|`@\t-1.78\nHaD@`@BDfUzZ@B@AaRXU]r@@\t-1.78\nHeTH@@rJJJJlLADP@LL@fye]H\t-1.79\nHidH@@rJJIEn`HH@FF@S\\sJD\t-1.79\nRFDFIFCbkBD\\h\t-1.79\nRFDFIFCBNT\t-1.79\nHmtL`HS@BLddlRVFUh@H@FCYS\\J`\t-1.8\nHeVH@DAInU~Eh@H@FAES]qDd\t-1.8\nsGP`Adifj`FFboEb\t-1.81\nqCb@AIZ`LLHnS@\t-1.81\nKA|@@@djkUU@FGABXUMrN@\t-1.82\nKA|H@XGIKZmUHA`RXU\\RF@\t-1.82\nsGQhHjOCIHTuHCB[fBH\t-1.82\nsGP@Di]j`FADQxmP\t-1.82\nHg]L`FvDp@cIIJeDednkUUUT@LApjxYT~QX\t-1.83\nHkmL`FvDp@cIIJeDeeuZjjj@FGXU\\LkrC@\t-1.83\nHcML`FvDp@cIIJeDdnkUUU@CClJnFUyF`\t-1.83\nHg}@@@mJYeU|]Tz@@@H@A`hXUMpri|bp\t-1.83\nHmL@`EhLddTQaQnFjjjh@XSBXUMpsrE@\t-1.83\nHknHBFAiRYfUnSjB`h@AahQaUprngrF@\t-1.83\nHg^B@D@agHhhdmBeSMURuT@LJqet~QX\t-1.83\nHmL`B@aBBTiWe\\JUZh@D@FEMproDA@\t-1.83\nHknBAHAIvISdfyWVxV`@i`@XDUMwHL\t-1.83\nHmwDAHePRISdfyW^Eh@I@A`QTw\\aP\t-1.83\nHid@P@bDbDfyWaj@@@FG@aTwCrH@\t-1.83\nHeT@b@|@eJYU^f```@FGIaMpsq@h\t-1.83\nHiDLbHS@BL{IIJdejj@FAES]rA@\t-1.83\nKAx@PQHIHUHMHYIHXjjh@p`nIg@\t-1.83\nHmvJ@D@aeTfYWjYjjV`A`fMrE@\t-1.83\nHmtH@@RYvUeVf@@@FGHS\\LkrL@\t-1.83\nHidH`HApRem^Fh@@@XMaTwCr@@\t-1.83\nHid@`@|Die_ahH@@FCXUMp|PD@\t-1.83\nHidD@@QInUxZ`@@AaTES]s@PP\t-1.83\nHeTH@@RYWZf`@j@A`pIaTODzP\t-1.83\nHid@@DjUfaBB`@FEQc\\OLPj@\t-1.83\nHaD@`@bDeeVz`@@A`pQaTwHx\t-1.83\nHie@@@EJ[W[j@B@AaVESCq@P\t-1.83\nHmt@@DjYnvDHbJ@AaVUtkrA@\t-1.83\nKA|A@PCaddTJjj`CAdpj[nP@\t-1.83\nsNqdHjG@nFRQFUU@LBLWqJ`\t-1.83\nKAxX@@gILjmUHAafDwHh\t-1.83\nsNpXHlpDYIBdmT@pQoDF\t-1.83\nqCbPXXbRt`\\JVBlHHSdP\t-1.83\nsNyHHpxYIECUR@prux`P\t-1.83\nsGXHHGCIHcUPC@kF|P`\t-1.83\nsOq@@drm[UT@pJqoHp\t-1.83\nsGPHADIJmU@LJHl^R`\t-1.83\nsJQDBIARBS]T@pVOI@\t-1.83\nsNxHHGAJijj`FAVkyB\t-1.83\nsOu@E`drm[SR@pU_Eb\t-1.83\nsJPdEaDSpR[f`FDGEB\t-1.83\nsGP@LdbMU@LJHm^P`\t-1.83\nsNq`@fdr{UHCBGbu@\t-1.83\nsJPhHaIIRuPCAX|`@\t-1.83\nsJPHAXIKUPC@qX|`@\t-1.83\nsJPYHlPLGh}Z@^S@\t-1.83\nsJT@@TizhAaicqA@\t-1.83\nsJQA@IRej`FAbqy@\t-1.83\nsJTHH`xQ}T@pVOED\t-1.83\nqCbPXXbRt`XHSdP\t-1.83\nsJPXhlPL[ct^m@H\t-1.83\nsJQ@@dsU@LBHcrX\t-1.83\nsKQ@@eKcUPC@qX`\t-1.83\nqCp@AIZ`LLInQ@\t-1.83\nRFB@QZDXLJy@\t-1.83\nRFBDQFDXLHyP\t-1.83\nRFaDSGCr@\t-1.83\n`InqHFD\t-1.83\n`IoApGd\t-1.83\nsGQiHl@c]ddeVhAaCrT\t-1.83\n`IO@Cr@\t-1.84\nsNx`BDdw[UPC@kF|c@\t-1.85\nHefLa@XY{pBY{IIETiUUL@LJaLJnQp\t-1.86\nsGQiHlOC\\eWmT@paxaP\t-1.86\nsNp`@dfuZj@X[F|RX\t-1.86\nsJX`LDdru@OEb\t-1.86\nsNp@LdbLjj@XTQU|VH\t-1.87\nQMFIOBM`^H`\t-1.87\nHiDH`DBHR[UYjj@FCAaTw\\`P\t-1.88\nqCb@AIVpOEL\t-1.88\nRFDGaFCBnP\t-1.88\n`IE`CB@\t-1.88\nHet@@DjYUX^d@@@@A`rXwCODJ`\t-1.88\nsJP`@dfvdAaCqK@\t-1.88\nsJP@DiZhAaaEqX`\t-1.88\nHmv@@@rRJIIFUjB`@@XBfES\\L|aP\t-1.89\nsNx`LDdr{UPCAZ|QX\t-1.89\nsJQ@@djs@LDcqS@\t-1.89\nsJX`BDdvu@LDcrX\t-1.89\nsNx`JDdrsUPCAZ|QX\t-1.9\nqCb@AIZPLDmqB@\t-1.9\n`IDPCr@\t-1.9\nHmvLAHAIbTyInUwaZ@B`@XDUMwHT\t-1.9\nsNq@JeTuTpCCKGdX\t-1.9\nsGX`BDdwMT@prqyJ\t-1.9\nqCcPQX^D]hCsAX\t-1.9\nHcLFbHSFgP|L{IIlheBJmUUU@CAbkiW^H@`\t-1.91\nqCb@AIi`LInHH\t-1.91\nQMB@HRZA`erP\t-1.91\nJgB@@FRQQIIEIixSl@@@@@@CBTpnE]N~QP\t-1.92\nJgBD@@YHhhhddZFSvzdI`R`@LFInT{y`sh\t-1.92\nHk\\L@@PtfUe]ieujP`J@AaV\\L{rC@\t-1.92\nHefHA@`IclddqRUUUP@pyLJfx^QH\t-1.92\nHmt@@DjU_ZxHDf@AaLIaTwCOHT\t-1.92\nHeT@@DjU_k``RP@XCBXUMp|bP\t-1.92\nHidD@@QInUxV`@@A`RMp|Pj@\t-1.92\nHaD@`@qDeeVz`@@A`pQaTwHx\t-1.92\nHidD@@QInUxV`@@Aaa\\OLRf@\t-1.92\nHifD@DCdfYTfZiZ@FFYS\\`P\t-1.92\nHidH@@RVUvy`@`@XUFES\\`P\t-1.92\nHiFD@LADfuYjj`AaVTwCqDH\t-1.92\nHaD@@DkeV{`@@AaVES]rH@\t-1.92\nKAx@@YIDTjjh@pxHSBinQp\t-1.92\nsJQBDHbNF\\CDIJuPCA@|Tp\t-1.92\nsNpD@xRHrRSFjhAaXl[rL\t-1.92\nsOpH@fILkW@@@LJlZ~P@\t-1.92\nsOpHMPIToVD@@LKDZ~S@\t-1.92\nsGPhH`xITmU@LMc^P@\t-1.92\nsNpP@jtfvZf@XIOdp\t-1.92\nsNpP@jtfvZf@XS\\RX\t-1.92\nsJPHAbIJuPCCA[ba@\t-1.92\nsGT`EaTeUfPGba@\t-1.92\nsJY@DDfvdA`l^Q@\t-1.92\nsJP`ADkjhA`l^JH\t-1.92\nqCaHK@H`QGe@|P`\t-1.92\nqC`PApRfdCCQ[d@\t-1.92\nQMFICBN`XQqd\t-1.92\nRFTPXQ`|R@\t-1.92\nqCa@CIJrAxh`\t-1.92\nHcL@h@dBdFdAdEdDfYn\u007Fi`bHh@GdZ\t-1.94\nHaD@`@BDeeVz`@@A`tXUMwH@\t-1.94\nKAxH@@bSJkUT@XEaTwHx\t-1.94\nsNpXHlQxIUZuT@paxjp\t-1.94\nsNx`BDdw[UPC@kE|e@\t-1.94\nsGP`Adinj`FFboHP\t-1.94\nQMBBlRZA`erP\t-1.94\nsGP@DjZj`FADQxmP\t-1.94\nsGP`@dfUj`FBqyJ\t-1.95\nQM`AIxGd`\t-1.95\nqCcRQX^FzNtAx``\t-1.95\nsGP@LdbMU@LBEcqR`\t-1.96\nsNx`BDdvsUPCAK\\UX\t-1.96\nHifDAHAHeNR[e^Eh@@@XDUMwHD\t-1.96\nqC`@ISTAaaErX\t-1.96\nHefB@LANEImfVjf`A`VDL|QF@\t-1.97\nsNx`JDdrsUPCCXwdX\t-1.97\nsJPH@xISuHCCSGbB@\t-1.97\nqCj@KAIVPOEL\t-1.97\nRFTPlQ`|R@\t-1.97\nHifDAHAHeNR[e^Eh@@@XDS\\OHh\t-1.97\nsNq`@fdr{UHCB[bM@\t-1.97\nHaFHAH@NbTeeVz`@@A`VES\\RN@\t-1.97\nQMFiOAzM`^P\t-1.97\nHeTD@@eIYe^E`HD@FFEW\\rFD\t-1.98\nqC`H@HpPdmPFBbxa`\t-1.98\nqC`PBHRVhCCBKdp\t-2\nsGQhHl@cIIJmPCAX|e@\t-2\nHaFH@FAIeZn`B@@XXe]s@q`\t-2.01\nsGX`BDdvmT@pRwDZ\t-2.01\nsJQiHl@c]GkPCrD\t-2.01\njhyQ@@LT@eLw\\jobaoNCSL@@@P@@pEpTeZMYwD^B\t-2.02\nj`a``@NPYddblrRbR]zjfjjj@AaiFB\\dkQkN}r@`\t-2.02\nHmt@hL`J`N`I`E`DjYnvDHbJ@AqVHEHGHDhBhGHD\t-2.02\nj`qQ@@G^@dsLjolcZmT@@@T@@LM@IQgARUiwnP`\t-2.02\nj`q@`@@^RYWYm^cKN`@j@B@@FF`HpRdeZM[wH\\@\t-2.02\nHo|LHJXXdDdLdNkLddlrfbRmzij`@@A`wKiv\u007FHD\t-2.02\nHgnHxCuICI@iAiCi@YBYCIIEIYGMUUUM@CBKyA`\t-2.02\nKAxQPTdAd@T@tAtAddbejj`CbdPLPBPFPNPNQ@\t-2.02\nHg]DHNff`HRPrPJPrRQR]QIG]Tm@@@C@et{yG@\t-2.02\nJoB@I@Xb^BRdlsJkkYoU@@@@@A`XQS\\Lkiv\u007FHP\t-2.02\nJoB@I@QB^BRdlsJkkYoU@@@@@A`XQS\\Lkiv\u007FHP\t-2.02\nJoB@H@AEABTrj}ktv}AA@@@@FAaaTwRng[|a@\t-2.02\nHg|@R@kH|HeIfYU^Svz`@@@@AahES\\LkiwrH@\t-2.02\nHg\\DHJZIAICICjsIIKLeDd]uST@@@LJe]N~Qp\t-2.02\nj`aaP@NPXkSlddUfbbaVcUTuMSP@LJHpRWnPl\t-2.02\nJoC@QDAxAJUKMjnmf}T@@@@@FFeS\\Lkiv\u007FDB@\t-2.02\nHgnHhFjIAICi@YBWCIICEMCIUMUU@C@et{yA@\t-2.02\nJoB@Q@^@ReSLjokYoT@H@@@A`^ES\\Lkiv\u007FH@\t-2.02\nHo\\DHJvIAICi@YCIICELXiJijij@FAL]orB@\t-2.02\nHg\\DpIVIAICICIIKEEDb]uU@p@@LFxUt{yA@\t-2.02\nHklLPHS@\\J\\LddQbbQtzjB`@@XCS\\Lki|QL@\t-2.02\nJoC@HGAbObILkkJ}L]p@T@D@@XJBXURnf\u007FHx\t-2.02\nJoB@X@^ErCrBTsJkzNj]@@@E@@F@xUMproH@\t-2.02\nJoB@P@^BTkLjnmf}T@@@@@FApeMprng[|`@\t-2.02\nHg|@P@BH\\DfYeUyO[j@@@@@FF`UMprng_H`\t-2.02\nHcm@RDpLROBIRYfUQe]jjjj`AaHES\\LkoH`\t-2.02\nHcm@RDpHbOBIRYfUQe]jjjj`AaHES\\LkoH`\t-2.02\nJoBDPD@xIU\\vjzv[uP@@@@@XZUMprng[|b@\t-2.02\nHkLJHJ[mN@aICI@iCIIEIuEJiZi`A``krC@\t-2.02\nHg|@P@SHbDjyeUyO[j@@@@@FFpUMprng_H@\t-2.02\nHg|@b@|@eJYUg_eNzB@@@@AajXS\\Lkiwr@@\t-2.02\nHg|@P@QH|DfYeUyO[Z@@@@@FF`UMprng_H`\t-2.02\nHg|@P@BH|DfYeUyO[Z@@@@@FF`UMprng_H`\t-2.02\nHg}@A@LINTjWvUz[SB@`@`@F@paTLkoDrp\t-2.02\nHg|H`HALRkfUWd}nh@@@@@XJUMprng_DB@\t-2.02\nJoCD@D@e\\sJkkYoU@@@@@AalTwCJz]orD@\t-2.02\nHklHPHipZHrRQVZIKSjj`@@AaJXSCJz_Hh\t-2.02\nHklHpJbPRPrPrRRqJYKSjj`@@A`wCJz_HH\t-2.02\nHcLDPHhG@gCIHcMDdnuU@@@C@PSNFUwbEP\t-2.02\nHg~@`HApRkfUWd}nh@@@@@X[ATwCJz]|`@\t-2.02\nHcmHbDr@|@eJnYUFUvjjjj@F@eS\\LkoDB@\t-2.02\nJoB@P@^BTsJkzNj]@@@E@@FBxUMprmor@@\t-2.02\nHg|@`@bDfYeUyO[j@@@@@FB`UMprngqHl\t-2.02\nJoBD@@IS\\rjzv[uP@@@@@XZeMprng[|b@\t-2.02\nHg|@`@BDfYeUyO[j@@@@@FF`UMprng_H`\t-2.02\nHg|@`@qDfYeUyO[j@@@@@FF`UMprng_H`\t-2.02\nHklD`HPGCIHdhdd]Nj`h@@FDeS\\Lki|`@\t-2.02\nHcNHpMJIAICICIIJhdmJuT@D@C@fxYyF`\t-2.02\nHg|H`@b`RYfUWd}nh@@@@@XJUMprng_H@\t-2.02\nHed@H@BH\\LBBBLdTTUeUUP@pdHSJ[agdD\t-2.02\nHklHpJbPRPrPrRRsQIY]jf@B@A`WCJ|`p\t-2.02\nHmTLpHS@\\J\\F\\LddRLTjjj`AaqTwCJ|c@\t-2.02\nHco@`HS@|DjyeTYWVjjjh@XBUMprn|PH@\t-2.02\nHctBHJ[m`HRPrPJPrRQR]QURuR@LDE^QH\t-2.02\nHcL@PH\\JbLddT\\RR{UT@@@LISBkae]yD@\t-2.02\nHg|H`HBDRkfUWd}mh@@@@@XJUMprng_H@\t-2.02\nHcLHpMbPRPrPrRRqIPcMT@D`@pJfF^Ph\t-2.02\nHg~@@@RYfUWd}nh@@@@@XZATwCJz]|b@\t-2.02\nHk\\@b@k@eJYeUquSh@@@@@XZFES\\Lki`\t-2.02\nHk\\@`@sDifUWGUN`@@@@A`nES\\Lki|`@\t-2.02\nHeT@pHbDbLbLddlRPsU@@@CChSJ[ayD`\t-2.02\nHk\\@`@rdifUWGUN`@@@@A`nES\\Lki|`@\t-2.02\nHctLhAgTdLdBdJdFdLddTgfUUTt`CrE@\t-2.02\nHg}@@@aJnYU^Svz`@@@@A`iTwCJz]|b@\t-2.02\nHkn@`NaprRQRjYKSjjp@@A`JXPrngrH@\t-2.02\nHcvHPDDGBWCIIEHniUUUP@pELJfy]yD`\t-2.02\nHcvHPAPGBWCIIEHniUUUP@pGBinE]yD`\t-2.02\nHcmH`Dr@|DjyeTYWVjjjh@XBUMprn|`@\t-2.02\nHg|H@@RYfUWd}mh@@@@@XRUMprngrK@\t-2.02\nHmv@PHApIpRi[WeZhH@@F@qeMproD@`\t-2.02\nHed@hH\\DdLdAdIdLddTaeUUP@prf{dD\t-2.02\nHiFDhNihBDdLdBdApLddTgjeh@XPOHx\t-2.02\nHg}@@@mIe]e^ftx@H@H@A`JXUCJ{q@l\t-2.02\nHglF@HXUN|ddUfffeUMUL@LNSBxYWdV\t-2.02\nHmtD`HPGCIHdhTiVj`@@AaqTwCJ|`@\t-2.02\nHiD@p@BH\\LBLdRbLjjh@XBBXUMp|b@\t-2.02\nHg}@@@mIe]e^ftx@H@H@A`hIaTLkm`\t-2.02\nHkmD@J[SrRQRqQXYjjje`A`tIaBoHx\t-2.02\nHknB@D@agHhhdmBjYjjVj@FAXr{rC@\t-2.02\nHknD@NalddUfbRtzjZjh@X\\fDLkoHL\t-2.02\nHmTLPDp`ZHbDjmyjjjh@XBDTwCJ|``\t-2.02\nHcvD`Naj|LddUfbUUUU@C@TpjFUwdL\t-2.02\nHmtHPHApIpRi[WeZhH@@FGYS\\Lkq@H\t-2.02\nHk^@@@RYWYVftx@H@@@F@iaTLkoD@p\t-2.02\nHedLHJ[hBDdLdBdLddTgUTmP@pb^PH\t-2.02\nHkLJ@HXUOIIEYiijjZjPAarXWCJ|cP\t-2.02\nHmTLPDeP\\ABLddTb|jjiPAaVES]rA@\t-2.02\nHcL@@LdbRbceBDEEP@LIaL[ae]yE@\t-2.02\nHmtD`HPGAJge^Uj``@@X\\UMproDA@\t-2.02\nHcND@NalddUfbR{USUT@LFSBE]yD`\t-2.02\nHedDhAeICI@iBiAiCIIEIyUUL@OHd\t-2.02\nHmT@P@\\HbDjmyjjjh@XSBYS\\LkrB@\t-2.02\nHcNH@JGIIEJeipuUMU@CChSBY]yD`\t-2.02\nHkLDbMLG@~rRQRVPrjjjX@X\\fDwRh\t-2.02\nHmT@P@|HBDjmyfjjh@XSBYS\\LkrB@\t-2.02\nHctL`HS@\\DjevZjij`A`ITwCJ{rL@\t-2.02\nHmvLAHAEbTyInUwaZ@BP@XDUMwHd\t-2.02\nHcvD@AklddUTReUUU@CChSBY]yD`\t-2.02\nsJPKAPR@TAh@p@zOj`GAb@dCHAP@\t-2.02\nHcvD@NalddUfReUMU@CCdpne]y@`\t-2.02\nHeT@`@\\LddTjPsU@@@CCdpj[ayD`\t-2.02\nHeTL@HSLddlRPsU@@@CBj[agfPf@\t-2.02\nHed@P@bH\\DfUfjjj@FD`fES\\L|b@\t-2.02\nHmvDAHAHeNR[e]xV`@h@FEES]OHT\t-2.02\nHmuH@JY\\ddTlRzZjjf@FCPfDL|a`\t-2.02\nHedDPDpFbHaJmYjjj`AatES\\L|`@\t-2.02\nHefD`Naj|LddUfUUUP@pYLJagbMp\t-2.02\nHmVH`FboCIIEUDjjjh@X\\fEproHh\t-2.02\nHeTB@@HYRYgUnjjj`A`HPaMpsqLh\t-2.02\nHclH@@RVUmVy]X@@@@A`vLLkoD``\t-2.02\nHeTL`HS@|Dj~UFVh@@@XUS\\L|PL@\t-2.02\nHmVD@AklddUTtjjj`AatIaMJ|c`\t-2.02\nHeTH`HApRee\\YjB@@AavES\\L|`@\t-2.02\nHedL`@idhLdTTNeUUP@pZLKagdL\t-2.02\nHiDBPHSB@HiprRPzQZj`Aac\\OHH\t-2.02\nHmtH@NrRQRqZUj``@@X\\fDwBoHd\t-2.02\nHmtD@@yJUe^Uj``@@XCBTwCJ|``\t-2.02\nHcl@@DjYU_egX@@@@@XJfES\\Lkh\t-2.02\nKAxQ`HCaDQSaddfLjj`CBhpf{dT\t-2.02\nKAx@PQHIHYHLxYIITjjh@ppj{dL\t-2.02\nHmTD`HPGAJeVZjfj@F@xUMproH@\t-2.02\nHedL`HS@\\DjWfjjj@FCES\\L|PL@\t-2.02\nHmTB@HXUrRQVZZjfj@FCIa\\J|bP\t-2.02\nHedL`HS@|DjWfjjj@FG@UMpsrL@\t-2.02\nHeTD@@gHeDeBwL@D`@psB{fPp`\t-2.02\nHiDD`@iJCIEDUJjj@FCQaMp|a@\t-2.02\nHedL`HS@lDjWfjjj@FCES\\L|c@\t-2.02\nHed@`@\\DjWfjjj@FDaBES\\L|c@\t-2.02\nHmTB@HZUrRQVYRjff@FAIa]rE@\t-2.02\nKAx@`@xTxYIDUJjh@pZDpf{bAP\t-2.02\nKAxH`PpNENFRRJrjj@LJSBknPp\t-2.02\nHeT@@LddTjPsUUU@C@XSBinGdR\t-2.02\nHedL@@pldTTUeUUP@pzDinF^PP\t-2.02\nKAz@dI`Xd^DRdsMUU@FE@UMwH`\t-2.02\nHmVD@NalddUfbjij`A`RXT|PU@\t-2.02\nHiFH`FboCIIETjjj@FEIa\\ODi`\t-2.02\nHeVH@JGIIEJepuUMP@pjDpfyD`\t-2.02\nHefH`D`OAJVyjjj`AavES\\L|`@\t-2.02\nHef@`HkprRQJrUSU@CAdpnF^QP\t-2.02\nHed@`@\\LddTeeUUP@pyLInF^QP\t-2.02\nHefH@NkIIETiUUT@LFaLHYyC@\t-2.02\nKAxHPTtRBRFRGAVRRVZjZ@OHx\t-2.02\nHedF@@PfFTf{nZjf@FAXUMqDd\t-2.02\nHiDD`HPKAJUfjjh@XMaTwCr@@\t-2.02\nHeTH@@RYWZf`@j@AapIaTL|c`\t-2.02\nsNpJIHJPtaZqrFRRVUU@LI^P`\t-2.02\nHefD@NalddUfeUUP@pYLJagdL\t-2.02\nKA|I@]CUxYIHkJjh@pILJnHw@\t-2.02\nsJPBHDJ@t@XBOj`GFRAd@hGI@\t-2.02\nKA|Q`MTRBRFRFRRVZjZ@LH{dL\t-2.02\nHiDD`HPGAJUfjjh@XMaTwCr@@\t-2.02\nKAzQDIdAxAJU\\uUT@XDUMwDB@\t-2.02\nKA|A@PCaddbljj`CAdpj[nQ@\t-2.02\nHiD@`H\\DjfZjj`AarXUMp|b@\t-2.02\nHiD@b@|@eJiUjjh@XRDIaTwB\t-2.02\nKAzA`TtRBRFRFRRUJjY@ODK`\t-2.02\nHaD@`@|DimVzP@@AaVES]q@P\t-2.02\nHiD@`@\\DjVZkj`AarXUMp|`@\t-2.02\nHiDD@JGIIEFjii@FAIaMrN@\t-2.02\nsJPK@ERlUYjpu`ZOj`FEDVH\t-2.02\nKAxA@@xYIHkJjX@pjDpfyG@\t-2.02\nsJPKAVRHTQh`q@zOj`FEDVH\t-2.02\nHiFH@DAIYfZjh@XMaTwCrH@\t-2.02\nsJPKA\\SDVIlPx`ZOj`FEDVH\t-2.02\nsJPK@\\SDVIlPx`ZOj`FEDVH\t-2.02\nsJPKA\\RHTQh`q@ZOj`FEDVH\t-2.02\nsNpLAbWD^HdlmUPCAbXu|b@\t-2.02\nKA|P@UvRRJjji@LBSBkbPp\t-2.02\nHiD`bJ[aArAdjeVj\u007FpAa@@\t-2.02\nKA|P@UvRRJNji@LJaLJnPp\t-2.02\nsNtDHpxTHrRQVjhA`Uc^JL\t-2.02\nKAxX`EWAHIHIMvuMHAxdL\t-2.02\nsNtLJyHJPtaddeUTpCqE`\t-2.02\nsJQBbJBRBdMHCcL_U@OHP\t-2.02\nKAz@@AVRQERjj@LFaLIn@\t-2.02\nsNyhGE`DYIITmT@pvMyF\t-2.02\nsNpIAxBTjUjj@XNKFod@\t-2.02\nsGYhKEaxYIIVj@XYX|e@\t-2.02\nsNyHHpxYIHkUR@pLVOIP\t-2.02\nsNyHGKxYIHkUT@prqxjp\t-2.02\nsNphH{xYIHkUT@prqxjp\t-2.02\nsGXDHGBaBUSUPCCKGbe@\t-2.02\nsNphHpxYIHkUT@prqxjp\t-2.02\nsNqHFGCIIEZjPFAbqyJ\t-2.02\nsOpAMJUKua@@CBqFodp\t-2.02\nsNpH@xYIHkUT@plVMyF\t-2.02\nsNpHAxISMUT@p\\VM_H@\t-2.02\nsGPD@xTHRjZj@XFKGdh\t-2.02\nsNpHHDYIHkUT@plVMyF\t-2.02\nsNyA@qNRRJuT`LCEcrT\t-2.02\nsNtA@QNRQFMU@LBmWrP\t-2.02\nsNpHAXYIDYUT@pLM_Dl\t-2.02\nsGPH@xISuU@LCL[qA@\t-2.02\nsGPHAXISuU@LCL[qA@\t-2.02\nsJPhLQbIKTpCA@|d@\t-2.02\nsNq`@fdvkSHCAX|Rh\t-2.02\nsNp`Adimjj@XYF|TX\t-2.02\nqCaHKBHg`dtpFDGEL\t-2.02\nsGQHI_AJijhAaecrT\t-2.02\nsN|ICVGHLeRmWxCr@\t-2.02\nsJPIAxBX~j@XTQX`\t-2.02\nsGP@DjZj`FEBqoI@\t-2.02\nqCcPQXND^hCBCfbp\t-2.02\nsJQ@@dkU@LBHcrX\t-2.02\nsJPXHlPDQzt@|VH\t-2.02\nqC`PBpcu@^ZK@\t-2.02\nsBQ@@eLUT@pTWI`\t-2.02\nsGY@LDemj`FFVoEB\t-2.02\nHiDL`HS@|Djmujj`A`UMp|``\t-2.05\nHeTD@@QInU~Eh@H@FEHUMwHd\t-2.05\nsNqhHlOCIILVj`FBkxlP\t-2.05\nsGPhHaxIU{U@LD[qC@\t-2.05\nsGQLKIARFdLddmU@ODZ\t-2.05\nHifDAHAHeNR[e^Eh@@@XXSCqJx\t-2.06\nHiDJ@@PfEIn{Zjd@XEaTwDS`\t-2.06\nKAxH@@bSKKTt@XXUMrN@\t-2.06\nsJY@LDejhA`QEnP`\t-2.06\nqCbPPNBUu@XI[bD@\t-2.06\nHcN@B@B\\dbRTJR{UT@@@LEaLinFUwd@\t-2.06\nKA|@D@DiUJuUPA`tIeMwHH\t-2.06\nqCcRQXAFzNlAy@\t-2.06\nQMFi@azM`^P\t-2.06\nHefB@BAAeImgfji`A`RYT|a`\t-2.07\nHiFB@BANEImVZjX@XDfTOHH\t-2.07\nsNx@AddQeUPC@pu|Rp\t-2.07\nsGX@@eOUT@pLqoDD\t-2.07\nQMBCDRZAaGFP\t-2.07\nsGX`JDdrmT@pexlP\t-2.08\nsJQ@@dmU@LLInID\t-2.08\nRF`EECqH\t-2.1\nHkmLAHdDvISdfyWVxV`@i`@XTUMt|`p\t-2.11\nHcMLAHdDvISdfyWWaZ@Bd@FEES]OHT\t-2.11\nHklD@@QIVYYQujjjj@FBxUMprngrH@\t-2.11\nHeTLbHS@|LyJoeQej@@@FATwCq@d\t-2.11\nHcL@@DjU_VnBBJh@FBpfES\\LkrM@\t-2.11\nHiFHA@`IclddqRjjh@XLfES]rA@\t-2.11\nHidL@HP\\ddRNijjj`A`VEP|Qn@\t-2.11\nHidD`HPOAJvUxV`@@AaQTwCr@@\t-2.11\nHidH`DBHR[e^FX@@@XTATw\\`P\t-2.11\nKA|PBA@SGYIIbjjh@piLJfyG@\t-2.11\nKAxQ@HDPdvkMU@FEAaTwHx\t-2.11\nsNyDJQHJPR[nihAaGqR`\t-2.11\nsJPL@xRHTQGuPCCQ[dH\t-2.11\nqCbXXQD|EDIMRA``nR@\t-2.11\nsOx@@drm\\@@@pHV_DZ\t-2.11\nsJPDADIpRVj`FADqyD\t-2.11\nsGPH@DISMU@LBEcqR`\t-2.11\nsNp`ATiujj@XYY|TH\t-2.11\nsJQHHGAJ^j@XYX|`@\t-2.11\nsGX`JDdrmT@pQoI`\t-2.11\nsGP`@deUjPFFqoI@\t-2.11\nqCaPQApcu@XH[bb@\t-2.11\nqCaPAR`RZhCAA\\f@\t-2.11\nsJPdE`DRPcyXAyB\t-2.11\nsGP`@dfUj`FBMyL\t-2.11\nqC`PApcu@XXQ\\f@\t-2.11\nqCx@AYIf`LLE`\t-2.11\n`IE`Cr@\t-2.11\nsJQ`@bdvt`ODl\t-2.11\nHaD@`@bDeeVz`@@AaPXUMqHx\t-2.11\nsOp@DjWkiV`FAbMxlp\t-2.11\nsGX@@djuT@pHVOEJ\t-2.11\nsNxA@IRiVjhA`xlZ~P@\t-2.11\nHcLF@@siWTify^ajjjj@FDpaTwBn|SI@\t-2.13\nHmtLbHS@BL{IIKDeaeZ@B@AaUMpj\t-2.14\nHeUDAHdDbTyInU~Eh@H@FAES]rI@\t-2.14\nqCcQQX^B{]GZ@|`@\t-2.14\nHaF@@@RYWih@H@FA@aUqLx\t-2.15\nsJPhEaDIMsPCBCdH\t-2.15\nRGDQxQP|R@\t-2.15\nsOx@@drm\\@@@phQkyL\t-2.15\nsOp@DjWkjj`FEDVOEV\t-2.15\nQMBCpRZAaWDP\t-2.15\nsJP`@TfVdAxe`\t-2.15\nQMHAIXFE\\d\t-2.15\nqC`@IRtA`enHP\t-2.16\nHevH@DAIemRhYZ`@`@XDwCODP`\t-2.18\nJoC`@@PzTlrj\u007FI{[P@@A@@FDeS\\Lkm|ap\t-2.19\nHk\\D`HP@cIHXheDQgSV@@@@@XMWCNgq@T\t-2.19\nHk]@A@LINTjWvUimLHB@@@A`LHUCJ{qLL\t-2.19\nHclL@@STfUmVfeVi@B`@XEgBoDkP\t-2.19\nHmt@`@\\DjYuQVhHD@A`JES\\Lkq@p\t-2.19\nHmtD`HPOAJUyVUjh@@@XMc\\Lkq@H\t-2.19\nHcnD@D@TfYg\\fWVjA@`@XEtkoHh\t-2.19\nKAxPP]D`daT`taddabjj`CAB{dX\t-2.19\nHidH`HCpRUe^Fh@@@XLaTwCr@@\t-2.19\nKAxX`Pf@DDxYIBeEjh@prf{b@P\t-2.19\nHiFD@L@TeYVjj`AapPaTp|c`\t-2.19\nHid@@DjUfaBB`@FGPfES]rA@\t-2.19\nsNqLKIARCdLddJjj@XKWdH\t-2.19\nsNqDHG@nFRQFUU@LJEj~P`\t-2.19\nsNpdIPxEprRHrjhA`Pl~Ht\t-2.19\nsNpXHlqxYIITuR@payej\t-2.19\nsNt@A\\dbJjj@XVKE|e@\t-2.19\nsNqhHlOAJkVj`FBkyB\t-2.19\nsJPDADQpRVj`FABqyH\t-2.19\nsNp`@dfvZj@XEFodp\t-2.19\nsJPhHaxIUmPCBGbF@\t-2.19\nqC`HCDvHdmPFBFxh`\t-2.19\nsJPhHQLiJuPCCA[dH\t-2.19\nsJXHHOAIZj@XYX|P`\t-2.19\nsJPYHlQxGh}j@XIX\t-2.19\nsFx@@eJf`@@LKEcP\t-2.19\nsGP`@Tizj`FAfMyD\t-2.19\nsGP`ADiVj`FBMxe`\t-2.19\nsJPhH`xQ}T@pfyhb\t-2.19\nqCaPKBHRZXCBCdp\t-2.19\nQMCALhabHzA`sr@\t-2.19\nqCaHK@HdaGe@|`@\t-2.19\nqCbPPNBTu@XI[d@\t-2.19\nsJPH@xQ}T@pTWEb\t-2.19\nRFDEaFCqH\t-2.19\n`IlaHFD\t-2.19\nsNx@@djmUPCAbKF|c@\t-2.21\nqCpPHABRk@XP\\f@\t-2.21\nsJQ@@dsU@LBHmrD\t-2.21\nsJX@@dmU@LLL^JH\t-2.21\nsJXaBDnR[Z`FDwHP\t-2.21\nsFp@DiTt@@AaacsQT\t-2.21\nHeVD`La@BLddlRPrm@@@CBj[agdX\t-2.22\nHif@@@RfU~F``@@XLfES]q@D\t-2.22\nsJX`LDdvu@LInID\t-2.22\nsJP@DiZhA`QEnP`\t-2.22\nsJP`@TfZhA`QEnP`\t-2.23\nsNp`@dfVZf@XJodH\t-2.23\n`IK`Cr@\t-2.23\n`IG`Cr@\t-2.23\nHmVD@NalddUfRjij`A`rXWCOHT\t-2.25\nsGP`@dfui`FBuxdP\t-2.26\nsFp@DiTujdA`QC^IX\t-2.26\nHed@H@\\HbDbLbLddjRUUUP@pFDpj[ayD`\t-2.27\nHmtF`HSBEP|LddqbbeVjjj@FEXULODzP\t-2.27\nHeT@pHdDdLdLddlRPsU@@@C@jxYyC@\t-2.27\nHiDHHNbPRPrPYprRRiJjj`AaaTOHX\t-2.27\nHcND@LADfuYU]Zj@@@FGYWCJ{qDH\t-2.27\nHeTHbACpBTieUzZBA@@XLfDwSq@h\t-2.27\nHeTD@@YIfUqfhH@@FG@UMpsqHp\t-2.27\nHefD@NalddUfUUMP@piLKiy@`\t-2.27\nHifH@JGIIEJnFjih@XUBXS\\c`\t-2.27\nHiFL@BAAR[eVji@FAIa]qJX\t-2.27\nsNplJyDJHtQddeeTpCCQ[dX\t-2.27\nKAxQ@PCaddblji`C@dpfyG@\t-2.27\nKAxQ@PC`eUkUU@FCIaTw\\b@\t-2.27\nsNpBIHJPt`wCIIJjj`FBMyF\t-2.27\nHid@@DjWffB@h@FEPf\\OHX\t-2.27\nKA|P@XBRvsUT@XLfES]rH@\t-2.27\nsNqLKIARFdLddjjj@XQ|a@\t-2.27\nKAxH@MvRRJrjf@LBSB{dP\t-2.27\nsGPLIHJPtaddejhAaCrT\t-2.27\nsNqHHGCIIEZj`FAVKyJ\t-2.27\nsGPD@xRHRjzj@XFKGdh\t-2.27\nsNpH@xYIHkUR@plVKyJ\t-2.27\nqC`DAhdPhcQD^hC@bK@\t-2.27\nsJQdHl@`nFRPkPCq@`\t-2.27\nsJQHHGAJzj@XYX|P`\t-2.27\nsGXHI_AJijdAaecrT\t-2.27\nsGPH@xIUMU@LBl[rP\t-2.27\nsGX@JeTuT@pJqoI@\t-2.27\nqC`PApcu@XP\\tV@\t-2.27\nsJPYHlQLGh}V@P\t-2.27\nsJQa@bnR[ZPGdp\t-2.27\nsJPYHlQVGh}V@P\t-2.27\nRFBCQQDXLJy@\t-2.27\nQMBBpchGfR\t-2.27\nRFDAaFCqH\t-2.27\nKAxP@@djkUU@FGABXUMrN@\t-2.27\nQMbDBDePXQrP\t-2.27\nsJP`@deVhAaCqS@\t-2.27\nRFD@QFCBNT\t-2.28\nsJP`@TizhA`inID\t-2.28\nsGP`@Tivj`FBfxeP\t-2.29\nsNx`BDdw[UPCCXwbS@\t-2.31\nsGP`@dfyi`FBMyD\t-2.31\nsJPD@DDHRgj`FFDwDb\t-2.31\nqCaPQCpcm@^XK@\t-2.31\nqCa@CIKTA`hnHX\t-2.31\nqCb@AIj`LDmqB@\t-2.32\nsGP`@TeYf`FBUxhP\t-2.33\nHeVLAHAIbTyInU~Eh@H@FAES]rI@\t-2.33\nsNpYHlQxGlddqZj@XR|VX\t-2.33\nHeTLbHS@BL{IIKDdLkP@@@pJfx^QH\t-2.33\nRG@D\\OJ@\t-2.33\nsGP@DiVj`FADVxcP\t-2.33\nj`q@@@DjU_VUiqw``RX@@@A`ZBBTYpVcV]{bLR@\t-2.34\nHkLLxJ[hBLdBdAdIdMdCdBbHaHaHbXjejj@Gb\\h\t-2.34\nJoBDH@QPIPIRlrjzv[uP@@@@@XKS\\Lkiv\u007FDa@\t-2.34\nJoB@H@^E^BTrj}ktv}AA@@@@FFiaMtkiv\u007FDB`\t-2.34\nHg\\BPHP{@HTHrRPqIIZSjtBDJ@CCbinF]yB`\t-2.34\nJoBF@DHNRRIGSQITuoUUT@D@AaNEPkiv\u007FD[`\t-2.34\nHctDHDXGBHbgBWCIILYDYUUUP@pzLKaW^QH\t-2.34\nHmTHHDBHapIpzHrJIQRrjjj@F@aeMproD``\t-2.34\nJoB@P@NBULrj~mf}P@`@@@FAxUMprng[|`@\t-2.34\nJoBDPDAxIU\\vjzv[mP@@@@@XKS\\Lkiv\u007FHH\t-2.34\nHk\\@b@B@eJYeUquSh@@@@@XKaTwCJz_H@\t-2.34\nJoBD@@ISJlzoJ[tD@@P@@XJfDwCNf\u007FHl\t-2.34\nHknD`La@BLddJbRvWUjB@`@XMc]JgqJD\t-2.34\nHcNBAHAIvISdfyWWaZ@Bd@FAES]rE@\t-2.34\nHedLPHFP\\JBLddRaeUTh@pKB[nHe@\t-2.34\nHefD`La@|LdabbeUUP@pxBinF^IB@\t-2.34\nHcLD`HP@cIHXhhjJk@QD@CBfxYwd@\t-2.34\nHmUH`Dr@|DjUyjjjh@XBATwCJ|QL@\t-2.34\nHcNB@D@agHhhdmCSMURu@CCLY^Qh\t-2.34\nHed@P@BH|DfUfZjj@FD`fES\\L|b@\t-2.34\nHmv@@@rRJEKKaj@BP@X\\fES]OHT\t-2.34\nHcLD@@QIe^UWVj`@@AavEprn|b`\t-2.34\nHiDHHJbPRPzPFPrRPqRjZ`AxaL\t-2.34\nHidH`HApRkm^Fh@@@XMaTwCq@`\t-2.34\nHmt@@DjU_ZxHHj@AaLIaTwCOHT\t-2.34\nsNpJIVJPtaYArFRRVUU@LINP`\t-2.34\nHiE@`HPOAJ[fjjh@XMaTwCq@P\t-2.34\nHiDD@@kIEDUJjj@FCQaMp|a@\t-2.34\nHiF@`HApRjYjjj@FCXUMp|a@\t-2.34\nHaDH`HCpRVU[j@@@FEXUMwH@\t-2.34\nHaE@@@yIe^f`@`@XTBXU\\c`\t-2.34\nKAz@@AVRQERij@LBSF{bPP\t-2.34\nKA|H@DFIS[UUPAadMwDR`\t-2.34\nsGPdH`xEprRHuT@ppuxdP\t-2.34\nsNqTHmV@`aFRPjKR`ODB\t-2.34\nsNx`BMdbaeUPCCRobV@\t-2.34\nsNqHDHaImYj`FAFMyF\t-2.34\nsJQHBLQIfi@XHSbV@\t-2.34\nQMIDbKpRYAaGJ@\t-2.34\nsJT@@Te^lAyD\t-2.34\nQMBCDRZA`cr`\t-2.34\nQMBApRYA`sr@\t-2.34\nsJPYHlQxGh}j@^IX\t-2.34\nsGP@Djyj`FFbwDZ\t-2.35\nHaE@@@yIe^f`@`@XDfEWDC`\t-2.35\nsNx`DJdrkUPCCHwbs@\t-2.38\n`IaphFD\t-2.38\n`IiApD\t-2.38\nsGPXHlQxIU[U@LInHt\t-2.4\nHet@@LddTQbMpuUUT@LAaLJfz^PH\t-2.41\nHcLH@@RfyY^Vjjjh@XKFES\\LkoHP\t-2.41\nHet@@DjYUZ^D@@@@A`rDwCODr`\t-2.41\nHidD@@QInUxV`@@A`REP|Qn@\t-2.41\nHiDB`HRnApReYjif@FBLOH@\t-2.41\nHaDH@@Rfu[j@@@FAIa]qJX\t-2.41\nHaF@@@RVU[n@@@FAXUMrN@\t-2.41\nsGQHHGAJmjhA`Uc^HP\t-2.41\nsJQDHG@nBUMT@ppVyB\t-2.41\nsJXHHOAIYj@XKGbB@\t-2.41\nqC`P@HRVhCCBKdp\t-2.41\nqCbPPND^hCCA[d`\t-2.41\nsJQ@@dls@LEcrP\t-2.41\nQMJDbDf`XQqd\t-2.41\n`Ld`\\WaB@\t-2.42\n`Ld`XP\t-2.42\nHeT@@LdbbQwCUUU@CBPaBinF^Q`\t-2.44\nsGP`ATf]jPFDWEb\t-2.44\nHclD@@QIeYWieujP`H@FEYpsoHT\t-2.45\nKAxA@AbIJjuUPAapPfES\\c`\t-2.45\nqC`PCDRVhCAA\\Tp\t-2.45\nsJX@@dku@LDmrD\t-2.45\nQMPBcXGcH\t-2.45\nsGQ`@jdvmR@pfxdP\t-2.46\nj`aA@@NYIHiXihYIFj``hj@@XRQ`gIFl{wHJ@\t-2.48\nHg\\@xHdDdLdNdIdMdKdLddlTTtrgUADQ@@|`p\t-2.48\nj`aaP@NPXkSlddUfbbaVcUTuMSP@LJHpRUnP|\t-2.48\nh`\u007FC@@B`qVRRJsSQZRjfjjh@LEHpS`iZl{p\t-2.48\nHmTLhJ[hBDdLdBdA\\LddTgRjej`A`dkrA@\t-2.48\nHcLHPHApiprRQUQIKmUP@@@pELKae]yE@\t-2.48\nHo|H@NrRQRqZZR]j``bh@FDiaMtg[|cP\t-2.48\nsNpJE@Z@LAXApBUSUU@NJdCHAPJ`M@yH\t-2.48\nHg}@@@aJVYU^Svv`@@@@AaITwCJz_Hl\t-2.48\nHiD@H@\\HbDbLbLddjRjjh@X]BXUMwHD\t-2.48\nHo^L@AaerRQVZYJRjfjj`AaJXWCIvx\t-2.48\nHmu@`HZoCIIEUDiVjZj`AarXWCJ|b`\t-2.48\nHiD@HH\\DdLdBdLddTjjjh@XES\\OH`\t-2.48\nHkND@NalddUfRdjijj@FGIa\\ngrA@\t-2.48\nHedLPHS@\\J\\LddQTeUMP@phfxYxbX\t-2.48\nHcvL@NamrRQVYQUTuT@LNSBxYwdJ\t-2.48\nHiDLPHS@\\J\\LddUdjjh@XDS\\ODS@\t-2.48\nHiDDPHXGBgCIIEYjji@FEIa\\ODi`\t-2.48\nHeTD`HPOAJ]eQfj@@@FCYS\\L|PD@\t-2.48\nHmLH@ARjZYjZVjjZh@XMaTwKrI@\t-2.48\nKAxI`UtQBQFQFRRUJji@LBaLKdL\t-2.48\nKAxX`Pw@xTxYIHkJjh@pILJnHw@\t-2.48\nHiFL`FaL@HrRRiQZj`AafEWDxP\t-2.48\nHeV@@JrRQRiCMT@@@LJSBZ^Ip`\t-2.48\nKAxX`Pw@xUPYIHkJjh@pqLKbMp\t-2.48\nHmVD@AalddUffjij`A`rXWBoHd\t-2.48\nHefL@H[aRjUVjjh@XCBXUMp|bP\t-2.48\nKA|Y@MBX@QddeRVj`CCLJnHw@\t-2.48\nHiFL@H[aRjUZjj@FGPfDwCrJ@\t-2.48\nqC`FB`d@hCP@zOTAqh`I@rAr@\t-2.48\nKA|AP\\dad`TaT`t`eTuUU@Gd\\\t-2.48\nsGQLJhCPA`DjfjPGFRAd@hGI@\t-2.48\nHiE@`HZoCIIEUJij@FEIa\\OHh\t-2.48\nHid@@DjUfaBB`@FEPfMwLJa@\t-2.48\nKAxY@PwUXYIHkJjh@pqLKbMp\t-2.48\nHiF@`DApRjZZjj@FCXUMp|a@\t-2.48\nKA|H@XCIKJuUPA`pQaTwDS`\t-2.48\nKAxH@AfRJIFjj@LJaJ[nPP\t-2.48\nsNpdHpxV`rRQVjhAaecqU`\t-2.48\nKAxX@PwYIHkJjX@pqLKd\\\t-2.48\nHaD@@DiUVyjjPA`HPfES\\\t-2.48\nsNplJyHJPtaddeeTpCBL\t-2.48\nsNphH{XYIHkUT@prqxjp\t-2.48\nsGQDHG@nFRQFj`FABuyB\t-2.48\nsNx`LFdlmUPCBbXobU@\t-2.48\nsNqHHGCIIEZjPFAVKyJ\t-2.48\nsGXLKIARFdLddmT`ODZ\t-2.48\nsGYHEAxIVuU@LMc^XQ@\t-2.48\nsJPiLQxBTiZhA`Xl^P@\t-2.48\nsGPD@xVHRjZj@XEXwd`\t-2.48\nsGXHHGAJijhA`Xl^R`\t-2.48\nsGQII[B|eTuT@prqyJ\t-2.48\nsGX`LDdwUT@ptMx``\t-2.48\nsJQhHlGBOj`FDOME`\t-2.48\nsJQLCIARFdH~j@^P`\t-2.48\nsJPD@xPHczhAaecrP\t-2.48\nsJQHHGAJnj@XI[fDH\t-2.48\nsNp@Djfjj@XUXu|d@\t-2.48\nsGX@@dkUT@phVMyH\t-2.48\nsFp@DiTt@@A`qEcP\t-2.48\nqCcPQXVD^hCsQX\t-2.48\nsJQAINczhAaeb\t-2.48\nqCcPQXZT]TCr@\t-2.48\n`IB`Cr@\t-2.48\n`IDpCr@\t-2.48\nKAxX@ASIRjuUPAaTIaMrN@\t-2.48\nqCc@AYIj`OBHl\t-2.48\nsNq@@djmUPCAbKF|c@\t-2.49\nHg~@@@RYfUWd}nh@@@@@XJUMprng_DB@\t-2.49\nsJPHADIJuPC@bK\\a@\t-2.51\nsNp`ATf^ZZ@XYY|e@\t-2.51\nQMIDbKpRZAaGFP\t-2.51\nsNq`AVeJmUPCBqXwdX\t-2.51\nsGQDHGBnBUSUPCCKGdh\t-2.52\nHeTH@@Re[TYj`@@AatIS\\L|``\t-2.53\nsJP@DjvhA`inID\t-2.53\nQMH`yIhFD|h\t-2.54\nsGP@DiVj`FEDVOIP\t-2.54\nHmtH@@RYeeZZjjj@FBpfES\\Lkr@@\t-2.55\nHmND@DCdfVUrjUZ`PH@FALLkrD@\t-2.55\nHaDH`HCpRnU[f@@@FEXUMwH@\t-2.55\nHaDH@@RVU[j@@@FCAFES\\c`\t-2.55\nsJPhLQDIKTpCBCba@\t-2.55\nsJXHHGAJZj@XYX|`@\t-2.55\n`Jg@XP\t-2.55\nKA|X@DB\\dwNuT`FFIa\\Rf@\t-2.55\nHmtD@@QIgYVUZh@@@XLS\\LkrL@\t-2.56\nHeTLbHS@BL{IIKDdLkP@@@prf{bCH\t-2.57\nHidH@@Rge^Fh@@@XMATwCrD@\t-2.57\nsGQhHbGAJ^jXAaKqA@\t-2.58\nsGX`BDdwMT@pVODj\t-2.59\nsNy`BDtfuZi@XIGbu@\t-2.59\nqCbPXXbRt`XP\\TP\t-2.6\nj`q@`@@LRYfWg^Qg^jB`@@@@FA`DhpTeZMYwnHa`\t-2.61\njhip@@@P\\eKLjo}HvkM@@@EP@@pTRcNBdkW`|aX\t-2.61\nj`q`@@@IToLjoiuoUT@@@@@C@tDLxJRmFl{wHX@\t-2.61\nJoB@P@^BULrj~mf}PA@@@@FA`UMprng[|c@\t-2.61\nHg|@b@|@eJf[U^Svz`@@@@AanES\\Lkiwr@@\t-2.61\nJoC@@@IRlrjzv[uP@@@@@XGBTwCJz]or@@\t-2.61\nHg|H`HApRkfUWd}nh@@@@@X[ATwCJz]|`@\t-2.61\nHg\\B`HSM@HrRPjIIHjZuAAD@CCj[ae[yF`\t-2.61\nHmT@XDdLdJdFdAdIdLddTaajjj`AaG\\Pe@\t-2.61\nHkmLbFvDp@cNrRRiQIY]Vjjj`A`qUproHL\t-2.61\nJoB`@@Ndrnlktqw@AP@P@A`hIaUJz[|c`\t-2.61\nHcm@A@tIvTiWYTjWP@`@@@XCBXWCJ|PM@\t-2.61\nHcMLbFvDp@cNrRRiQIKjuUUP@pXjxYWdZ\t-2.61\nHk\\@`@|DifUWGUNjjjj`A`\\IaTwCJz_H@\t-2.61\nHedDpHhG@gCgCIHcEiUUT@LNAL[agdD\t-2.61\nHcLD`HP@cIHXhdhLk@PT@CBfxUwfAT`\t-2.61\nHcLD`HP@cIHXdlipk@AT@CBfzUwf@U@\t-2.61\nHcL@@LdbRbUeBDEEP@LAcFxYW^Ha@\t-2.61\nHcL@@LdbRbceBDEEP@LAaL[iW^IP`\t-2.61\nHid@`@\\LdbaRxZ`@@AarXUMp|`@\t-2.61\nKA|P`E@NENFRQERjj@LJSB[nHE@\t-2.61\nHiDNQHSB[a@XhXKa{IICiEjV@D\t-2.61\nHmw@@@`Tke]fy`@j@A`vESRoH`\t-2.61\nHidH`HCpRkm^Eh@@@XTUMp|PL@\t-2.61\nHidD`HPOAJ^UxZ`@@AaQTwCq@P\t-2.61\nHeT@@DjYoIjjjh@XJDIaTwCH\t-2.61\nKAzQ@IdAxIUsMUPA`QTw\\`@\t-2.61\nsNxTMEV@`aFRPjKR`OLLP\t-2.61\nsGXHHGAJfjhA`Uc^P@\t-2.61\nsGQHDHaImfhAaacrT\t-2.61\nsJPDAHJPR[j`FDwEB\t-2.61\nqCbB@\\duPFBVyH\t-2.61\nQMFAjAIhFBOJ@\t-2.61\nHifDAHAHeNR[e^Eh@@@XXWCqJX\t-2.61\nsGQ@@eKuT@ptqxeP\t-2.62\nHaDH@@RVU[f@@@FFXW\\rJX\t-2.64\nKAxX@@gILjmUHA`aMqLh\t-2.64\nsJQ@@dkU@LBHmrD\t-2.66\nJoBBXBwD`dadaddebbTRtj[uTaPI@@XDpz_H\\\t-2.67\nHmtLbHS@BL{IIKEDiUZ@`@A`S]J|`P\t-2.67\nKAx@P@xQDIDYDYIITjjh@pZDpj[d\\\t-2.67\nHcLH@NrRQRqYRmTDA@@pYLInWbCh\t-2.67\nHkLF`HSNgP\\DjUYfjjYf@FFBi|`@\t-2.67\nHmVL@NaerRQVYJjfj@FCIa]J|`P\t-2.67\nHedL@@pdie]jjj@FCIS\\L|b@\t-2.67\nHedL@HZ\\ddUfUUMH@pILKnPH\t-2.67\nKAxX@AaYHhdZjh@pIJ[nIA@\t-2.67\nKA|X@QS\\eUkUU@FCIaTw\\b@\t-2.67\nsNpDHDTHrRQVjhAaXl[rL\t-2.67\nsGPDHDTHRjZj@XFKGdh\t-2.67\nsGQHHGCIHcUPCCKWbA@\t-2.67\nsNx`LDdvkUPCAY|rM@\t-2.67\nsGXHHGAJijhA`Uc^R@\t-2.67\nsGQ`AbeLuT@pRoMLP\t-2.67\nsGX`LJdmuT@pHaoDl\t-2.67\nsGXHHGAJijdAaecrT\t-2.67\nsGPHHDIUMU@LCEcrT\t-2.67\nsGP`Adinj`FBUxhP\t-2.67\nqC`PALcu@XZK\\`@\t-2.67\nsGQ@@dsmL@pruyB\t-2.67\nQMIBhF`chGcH\t-2.67\nqCp@BOTAahl\t-2.67\nRFTQVQP|`\t-2.67\nsNp`@dfUZj@X[E|e@\t-2.68\nHaDH@@RVU[j@@@FCQaTw\\`@\t-2.68\nsNpHAbIJkUT@pXbqoHp\t-2.69\nqCb@AIVPLDmrP\t-2.69\nsGQ`@jdvmR@|rq@\t-2.69\nsGQ@@djuT@phbqyJ\t-2.7\nJoC`@@PzTlrj\u007FI{[P@@A@@F@eS\\LkoDmp\t-2.72\nHeTDPDp@bHcIIKDdLrp@@@pjJ[ayD`\t-2.72\nHiDDHJZIAICi@YCIICEJij@GbDp\t-2.72\nKA|Y@UBXGaddeTZj`CCDpnHw@\t-2.72\nHid@@DjUfaBB`@FGPaTwCrL@\t-2.72\nsNyhKEaxYIITuT@prqxjp\t-2.72\nqC`XBHc`haIj`LLEnR@\t-2.72\nqCcRQXCFzNlAy@\t-2.72\nRFTEPQ`|R@\t-2.73\nsGP`ATiVj`FFbwDZ\t-2.74\nHaE@@@aJyUnX@@@XEaTwDc`\t-2.75\nsJPXHlQxQ{T@pbxlP\t-2.76\nsJX`BDdvu@LH^IX\t-2.77\nHid@`@\\LdbaRxZ`@@A`rES\\ODC@\t-2.78\nHmLD@@QIe[VfeVi@B@A`V\\J|Ri@\t-2.78\nHedJ`HSFDGAJeyjjY`A`gSq@p\t-2.78\nsJPD@DPHRnj`FFFOED\t-2.78\nsNp@Dj{Zj@XFKGbk@\t-2.78\nqCa@CISTAaEqS@\t-2.78\nHefD@NalddUfUUMP@piLKayD`\t-2.79\nHiFL@BAAR[eVji@FAIaUqFx\t-2.79\nsNpP@btf{Zj@X[E|Rh\t-2.8\nHiD@XHdDdLdFdNdAdLddjdjjh@^PP\t-2.8\nsNp@DiUjj@XLQXwdX\t-2.81\nHo|DpHhG@gCgCIHcDhhd[^jjB`@@XZBYprng[|``\t-2.83\nj`q@`@HRRjyeU\u007FSk^j`B@@@@FAhTYpTeZMYwnPH\t-2.83\nHglLXDp`BJdFdA\\IdEdLddTaLTdmUUT@LLt{yG@\t-2.83\nj`q@@@LddTjVTRPiN}U@A@@@@LMHpRcNBtZsnP|\t-2.83\nHo\\LxAgTdLdBdJdFdKdGdLddTgfTtjjffh@^PX\t-2.83\nJoB@Y@RGTCrG\\drnlktqw@AP@P@A`JXURngrN@\t-2.83\nHg^DHNihBDdLdBdLddTgTRQwUKP@@@pjUt{yG@\t-2.83\nHcvBXNiev@aICI@i@Pb\\CIIEIxiUKTh@pb^PH\t-2.83\nJoB@Q@^@ReSLjokYoT@P@@@A`^ES\\Lkiv\u007FH@\t-2.83\nHkLLpHS`\\J\\F\\LddRLbdjjjj@FGXPrngrN@\t-2.83\nHmTHXERPrPjPZPFPfPrRQRFFjjj@FB\\oHH\t-2.83\nHcvFPJaLe]CpGprRRjKRMUTt@LFSBY]yG@\t-2.83\nHmVBhNiiV@aICI@i@PcIIEIuJiZT@XQwHD\t-2.83\nHcLHpMbPRPrPrRRjIKRmU@A@@pinF]yA@\t-2.83\nHmVFPJaLFUCp{prRRiJQjjZ@FCIaMJ|b`\t-2.83\nHo~D@NalddUfRbRmzjZ`@@AaJXWKiv\u007FHD\t-2.83\nHk\\@b@a@eJYeUquSh@@@@@XKaTwCJz_H@\t-2.83\nHk\\@`LhDjU^ukmLHH@@@A`lHU\\Lki|c@\t-2.83\nHmTDHNFIAICIAgCIIJeDjjjh@XDURoHX\t-2.83\nHmTJhJ[iV@aICI@i@PcIIEIuJiZT@^PH\t-2.83\nHmVHHFjIAICICgCIIKMEJijh@XXLkrN@\t-2.83\nHedLjDp`BJdFdAdIdB{IIEIyKUT@OHx\t-2.83\nHedDpHXGBg@WCIIEXmUUT@LFSBkiy@`\t-2.83\nHmM@`HPOAJYeWaeZjjj`A`NES\\Lkr@@\t-2.83\nHeTL`HS@\\LddlRPsU@@@CBj[agfPf@\t-2.83\nHmVD`Naj|LddUfTjjj`AarXUCJ|Qn@\t-2.83\nHmVHpHxGBYAYCIIEKCJjfh@XDfDwHD\t-2.83\nHcw@`H[j|LddUbbeTuU@CAdpYW^HU@\t-2.83\nHkLL`HS@\\DjUYfjjjj@FDeS\\Lki|`@\t-2.83\nHmW@RDp`|Hq@eJkfZjjj@F@aEMprh\t-2.83\nHmWHbDphCpBTjnYjjjh@X\\EMproH`\t-2.83\nsJPKAXR@TAh@p@zOj`GAb@dCHAP@\t-2.83\nKAxX`Pw@xUxYIHkJjh@pILJnHw@\t-2.83\nHefH`FboCIIETiUUT@LFSBxYxeL\t-2.83\nHmTH`HApRjyfjjj`AaJXUMproH`\t-2.83\nHeT@@LddTjPsU@@@CAdpjZ^Ip`\t-2.83\nHaD@P@bH\\DeeVz`@@A`pXUMwH`\t-2.83\nHiD@`@\\DiVZjj`A`LIaTwCr@@\t-2.83\nHiD@`@lDiVZjj`A`LIaTwCr@@\t-2.83\nHed@`@lDjWfjjj@F@paTwCOHp\t-2.83\nHig@PBhPdDdDfyWaj@@@GfPw@\t-2.83\nsGPBIDJHtQXcIIKUPC@qZ|a@\t-2.83\nsJPKAyRHTQh`q@zOj`FEDVH\t-2.83\nHiFH@FcIIEUJjj@FEIa\\OHh\t-2.83\nKAzA@P`^BTlsUT@XDUMwDC@\t-2.83\nHed@@Djffjjj@FGXUMpsrD@\t-2.83\nsNxBLIAREdGHIMmUT@|tq@\t-2.83\nKA|A@HC`eTuUU@FEXUMwHP\t-2.83\nsOpIMP[djWkB@@FEbM_I`\t-2.83\nsNpL@xVP\\`eTuUPCCKGd`\t-2.83\nsNtIH`DBTifjj@XGFod@\t-2.83\nsNtIHaxBTifjj@XGFod@\t-2.83\nsNp`H|ddUjj@XYX|UX\t-2.83\nsNq`AbeMmUPCCJod`\t-2.83\nsJPD@xRHczhAahmrD\t-2.83\nsJPXHlQDQzl@pa@\t-2.83\nsJPYHlQDCh}U@P\t-2.83\nqCcPQXAD]TB\t-2.83\nsNqThmVO@~Fy|eWsTpCq@`\t-2.83\nsGP@Dj}j`FFfODj\t-2.86\nsJQ@@drt`LDmrD\t-2.86\nqCr@HIJlAyL\t-2.86\nsJP`@TeVdAaEqS@\t-2.86\nKAx@@IRjuUPAapPfES\\c`\t-2.87\nHeTLbHS@BL{IIKDdLkP@@@pJnF^Pp\t-2.88\nHeTLbHS@BL{IIKDdLkP@@@pJfz^PH\t-2.88\nsGPD@DDHRgZj@XDSGbU@\t-2.88\nsNpXHjpTIRkUJ@p`xlP\t-2.88\nsJQhHbBaJ^i@XP\\Rp\t-2.88\nsJPYHlPDGh}Z@XS@\t-2.88\nKAx@@YIDTjjh@pXDpj{bPp\t-2.88\nqCaPS@hRVdCqA@\t-2.91\nsJQhHlOBOZ`GfVH\t-2.91\nHk^@@@RfYU\\]Tz@@@@@FBaaTwCJ{rC@\t-2.93\nHeU@@@gIHhTmpu@A@@pYLJf{dR\t-2.93\nHiFD@DBlbbTLZjh@XTfUp|Pf@\t-2.93\nHefD@DBdfV^Zjj@FCIaTL|Qn@\t-2.93\nKA|H@HAIMwMUPA`VDw\\QJ@\t-2.93\nsGPHAbIJmU@LJHl^R`\t-2.93\nsOx@@drm]UT@pxbqk@\t-2.93\nsJQHDHaInZ@XH[dH\t-2.93\nqCcAAY]nRYhCbRp@\t-2.93\nqCcAAY]nRYhB\t-2.93\nQMBApRZA`sr@\t-2.93\nHmt@h@dBdFdAdEdDfYn~fBHb@AyD`\t-2.94\nQMFAjAIhGfR\t-2.94\nQMhDRUAyH\t-2.96\nRFDUAFCqH\t-2.96\nqCp@AI^`LHnS@\t-2.96\nHgnB@NamOIIEYeEEUSUM@CCdpnD{yB`\t-2.98\nHmtD`HHIeIUYVUjh@@@X\\BXwCOHT\t-2.98\nsGPBE@Z@LAXAJijhApT`Y@JATCd`\t-2.98\nHiDDpJFIAICICIIKLjii@FDMrN@\t-2.98\nHaDH@@RVU[f@@@FEQaTwHx\t-2.98\nHaFD@DCdfYRYje`A`fMrJ@\t-2.98\nsNpdHpxWprRQVjdAaecrT\t-2.98\nsNpXHmPxYIDYUL@pcy`l\t-2.98\nsNqHHGCIIEZj`FAVMyF\t-2.98\nsNqHHGCIIEZjPFFVOEV\t-2.98\nsJPD@xRHczhAaIsKD\t-2.98\nsJPD@xQpczhAaecrP\t-2.98\nqCc@AYIj`LINXK@\t-2.98\nqCpB@SGZ@pRwI@\t-2.98\nQMFIJrMP^P\t-2.98\nqC`@Qv`LEErX\t-2.98\nqCbPPSJRu@XHKdp\t-3\nQMJHfTf`XQrP\t-3\nRFTQLq`pce@\t-3\n`IC@Cr@\t-3\nsJQ@@dkU@LLInID\t-3.01\nsGX`LDdrmT@pfxcP\t-3.01\nHed@xDkLdBdJdFdAdIdLddTgeUUP@paNQp\t-3.02\nHg|@`@bDfYeUyO[j@@@@@FF`UMprng_H`\t-3.02\nHidD`HP@cIHXdnEX@@@XHw\\HBhP\t-3.02\nKAx@`ADPxILsUUPAapIaTw\\b@\t-3.02\nKAxQ@PC`eWMUU@FCIaTw\\PH@\t-3.02\nKA|A@PG`dsMUU@FCIaTw\\`@\t-3.02\nHaDD@HSIIDcSUUT@LBpj[d\\\t-3.02\nsNxTMEV@`aFRPjKR`LInHD\t-3.02\nsGPdIPxEprRHuT@ppVxcP\t-3.02\nsOp`Hdjekjj`FFqoHp\t-3.02\nQMFIBbN`XQqd\t-3.02\nRFTPTQ`|R@\t-3.05\nRFDGaFCqH\t-3.05\nHk\\@@DjU^ukmNeZ@B@A`LHUCJ{sLxp\t-3.06\nHeTD@@QIUeQej@@@FEYS\\ODbP\t-3.07\nHig@@@`Tke]nX@H@FAXUMqHD\t-3.07\nsOx`DFdrikTl@pVoH`\t-3.07\nsNp`AdenZj@XEZoba@\t-3.07\nqCbPPNBUt`XI[bD@\t-3.07\nqC`PApRZhCCQ[d@\t-3.07\nsJQDHGCNBT}T@pRODL\t-3.07\nRFTPLQP|`\t-3.09\nHcMLbEvDp@cNrRRqIYCJt@D`@pJfx^Qh\t-3.11\nHg^@B@B\\ddRRTjQwUUP@@@pNDpfxYWSl\t-3.11\nHo~J@Naeu|ddUfbbatzjZfi`A`RXT|bp\t-3.11\nHmtHpBbPRPrPrRRjIJUjh@@@XDwCOHT\t-3.11\nHmt@`H\\LddUbReZj@@@F@iaMproH`\t-3.11\nHkND@NalddUfRTjijj@F@ia]Jz_HD\t-3.11\nHiDHhARPrPJPjPZPrRQR^jjPAyG@\t-3.11\nHiDDPHPGBgCIIEYJjj@FEXS\\OHP\t-3.11\nHmTB@@SarJIIPijij@FADtkrL@\t-3.11\nHeV@B@B\\ddTjPsU@@@C@Tpj[ad\t-3.11\nHefD@AalddUfeUMP@piLKayD`\t-3.11\nKAxH`EpRBRBS]mSR@XQWDa`\t-3.11\nHiDL@HX\\ddUfjih@XDfDOHx\t-3.11\nsNpLHxVH\\PeTuUPCBkFod`\t-3.11\nsNyLGYHJPtaddeeTpCrL\t-3.11\nsNqhHlIIJeZj`FFM_H@\t-3.11\nsNx@@eRmUPCAqXu`\t-3.11\nQM@HvA`sr@\t-3.11\nHiDB@@SaRYUfjf@FBDwDpP\t-3.13\nHo~B@LAEwHhddhhd[]jjA`@@XCaCNg[|b`\t-3.15\nsJPIAxBTeZhA`Xl^P@\t-3.15\nsJP`AdizhA`d^R@\t-3.16\nRFBDQADXLHyP\t-3.17\nj`q`@@@YIEBedhdnB]zh@IiX`@XJQ`eF\\tZsnHex\t-3.19\nHglHHDApbHipVHrRSJJJ^UUUUP@pvLInFUt{yE@\t-3.19\nJoBD@@ILrkNx{[uA@@@@@XZAUprng[|RL@\t-3.19\nHg|@`@\\DjYmUyO[j@@@@@FFxUMprng_H@\t-3.19\nHctHPHApIprRIQQNUUUT@LEALinFUwdD\t-3.19\nHgmH`Dr@|DjV^Yjjjjh@XZATwCJz]|c@\t-3.19\nHiDHPHApqprQJHrjj`A`rES\\ODC@\t-3.19\nHmW@@@H\\dbabLjfj`A`rXwRoHh\t-3.19\nHiE@`HPOAJUfjjh@XMaTwCs@a@\t-3.19\nqCq@XIIV`OEL\t-3.19\nHiF@B@BTjeVjj`A`LIaTwB\t-3.19\nsJPHAbIJuPC@bK\\a@\t-3.21\nHg^J@Naev|ddUfbbNgUSTtp@pqLKdR\t-3.23\nHiFD@AalddUfjih@XDfDOHx\t-3.23\nKAx@@YIHkJjh@pZDpj[d\\\t-3.23\nsGPDHxVHRjZj@XEXwd`\t-3.23\nsNq@GddbmUHCCKGbe@\t-3.23\nsNpH@xYIHkU\\@prqyF\t-3.23\nqCcPAZTBSU@^XK@\t-3.23\nsOx@@drm\\@@@pHQoEf\t-3.25\nRFBDQ^DXLHyP\t-3.25\nHiDDHJZIAICi@YCIICEJij@FDMrB@\t-3.27\nsGPdCQHJPR[jf@XR|d@\t-3.27\n`IJ@Cr@\t-3.27\nHg|@@DjU_eZx{BBH@@@FBpaUtkiwqLX\t-3.31\nHo\\DxM]ICI@iAiCi@YBYCIIEIYGLjjjii@FDGrK@\t-3.35\nj`a``@NPYddblrRbR]zjfj@@@A`iFB\\ehug^y@P\t-3.35\nHo\\BxJX]bPRPrPzlNPnP^PrRRsIJJRjZjZ`AyG@\t-3.35\njhqA@@EYIHiXeDkIKUTADUT@@pTcAFJtYw`|`H\t-3.35\nHiD@g@hHdDdLdBdJhFdNdAd@zIklddUdjjh@P\t-3.35\nHkLLhJ[hBDdLdBdA\\LddTgTRjejj@FARngrA@\t-3.35\nHkLDxMMICI@iAiCi@YBYCIIEIYGJjji`AyA`\t-3.35\nHg\\LPHS@\\J\\LddQbdRiwUPU@@@pEnFUt{xbX\t-3.35\nHg|@P@bH\\DfYeUyO[j@@@@@FF`UMprng_H`\t-3.35\nHklDPHXGBgCIIEYhdmNjj@@@F@iaCJz_Di`\t-3.35\nHko@hAdtdLdBdJdFdLddTgbRtzjh@@@^Hn`\t-3.35\nHmV@XEbPRPrPzPFPfPrRRqRJjjj@FBDoHX\t-3.35\nHo|@`NqLddTlVVdeZhHBj@AaJXS\\iv\u007FHD\t-3.35\nHo|@`NqLddTlVfdgZhHHj@AaJXS]iv\u007FHd\t-3.35\nHg|H`HCpRkfUWd}mh@@@@@XJUMprng_H@\t-3.35\nHkLLPHZP\\A\\LddUbRdjjZj@FGIa\\ngrI@\t-3.35\nHkLLPHXP\\I\\LddUfRdjjZj@FGIa\\ngrF@\t-3.35\nHmVHHFjIAICICgCIIKLiJjZh@XXLkrJ@\t-3.35\nHmVHHFjIAICICgCIIKEiJjZh@XXLkrF@\t-3.35\nHcLHpJbPRPrPrRRsQIKmTp@@@pKae^Qh\t-3.35\nHmM@bHPO@IRfYUxYVjjjh@XCaTwCJ|`@\t-3.35\nHk^@@@RfYU\\]Tz@@@@@FBxUMprngr@@\t-3.35\nHkM@`HzoCIIDbiEJijj`A`JXPrngrJ@\t-3.35\nHctDPHxG@WCIIETeIUSUP@pyLJiW^Pp\t-3.35\nHg^@@JrRQRiKRCMT@ET@C@TpfgSod\\\t-3.35\nHef@HJbPRPrPYprRRiRUSU@CCNF^PP\t-3.35\nHct@PHBNBLddURTeUUU@CAXSBine]@\t-3.35\nHmTDPHXGCgCIIETiJjjh@XLfDLkrJ@\t-3.35\nHctDPHXG@WCIIEXmIUTuP@pYLJUwd\\\t-3.35\nHkLD@J{IIETdiJjjj`A`LIaBngrF@\t-3.35\nHeTD`HP@cIICLdLkP@@@pjfxYyA@\t-3.35\nHiDBPHSBCpiprRSFQjj`Aac\\ODp`\t-3.35\nHidD`HP@cIHXdnEX@@@XDULODC`\t-3.35\nHiDBbJ[aCpBTjeVjj`AaHPfES\\H\t-3.35\nHmVD@NalddUfRjij`AaRXWKqAd\t-3.35\nHmTD@H[IIETiJjjh@XLfDLkrJ@\t-3.35\nKAxYDPwUXUvRRJrjj@LBSBkbMp\t-3.35\nHiFH`D`OAJyfjjh@XMaTwCq@`\t-3.35\nsJPK@zRHTQh`q@ZOj`FEDVH\t-3.35\nsJPK@yRHTQh`q@ZOj`FEDVH\t-3.35\nsNqdIVG@nFRQFUU@LLEnXu`\t-3.35\nsNqiCVO@ReRmUPCAqXu|`@\t-3.35\nsJPJ@tRHTQh`qD_U@LJHlP\t-3.35\nsNpiH{XWlddUjj@XYX|UX\t-3.35\nsJXHHGAJfj@XYX|P`\t-3.35\nqCcPQX^D^hCBCfbp\t-3.35\nsJPXHlPYQzj@`\t-3.35\n`IoAHGd\t-3.35\nHeT@H@dBdAdIdDfYu[hHB`@^Ie@\t-3.39\n`L@LH\t-3.4\nsJXDHGCNBT}T@ptQxa`\t-3.42\nsJTHHaxIWMPCAX|P`\t-3.42\nsJPhLQHIKTpCqP`\t-3.43\nKAxX`Pw@xUxYIHkJjX@pqLKbUp\t-3.45\nsGQLKHaQFbLddmT`LMEnP`\t-3.45\nqC`DBHg`hcQBSU@XXK\\d@\t-3.45\nsGQDHGB~BUSUPCCKGdh\t-3.45\nsGPH@xYIDZj@XFKGbE@\t-3.45\nHo|D`HP@cIHXhheMAUXBHj`@XBwCNg[|PE@\t-3.48\nHo|D`HP@cIHXhdicAeXBBj`@XBwBng[|PJ@\t-3.48\nJoBBPDHG`eJsJkkYoU@@@@@A`mMprng[|`@\t-3.48\nHo|@@LdbRbblThP`hjj@AalIc]Jz]orA@\t-3.48\nHg}@@@aJVYU^Svv`@@@@A`ITwCJ{qCl\t-3.48\nHk^@@@RfYU\\]Tz@@@@@FDaaTwCJ|SS@\t-3.48\nHmTLPHS@BB\\LdaTTTVjj`A`uMproD@`\t-3.48\nHiDDhJZ@aICICi@YCIIECJiZ@GbPP\t-3.48\nHee@@@[IEDTiUUT@LAALJfF^IG@\t-3.48\nHid@`@|Die_ahH@@FCIaMp|PJ@\t-3.48\nKAx@`@DPDIRsUUPA`pPUMwDS@\t-3.48\nKAxH@ABTluUT@XDUMwDS@\t-3.48\nqC`HBHc`duPFFBwI@\t-3.48\nqC`HBHc`duPFDGME`\t-3.48\nHmvB@D@aeIfUzfZjeh@XIc\\aP\t-3.52\nQMCBHoAbDf`XH|h\t-3.52\n`IlAHFD\t-3.52\nHiDHhJbPRPrPzPFPrRRsRjZ`AyC@\t-3.55\nsNpB@xRHTQhcIIJjj`FEbqoHp\t-3.55\nKAxX`Pw@xTDYIHkJjT@pqLKdT\t-3.55\nHaD@`@\\DimVz`@@A`rXUMwH@\t-3.55\nKAxQ@HC`eTuUU@FEXUMwHP\t-3.55\nsNpmJyHJPt`~rRRrjXA@\t-3.55\nsNpH@xITkUT@p\\VM_H@\t-3.55\nsJPH@xIRuPC@qX|`@\t-3.55\nsGQHHGAJWjhAaecqB`\t-3.56\nHcND@LADfUyU]Zj@@@FGXWCJ{rJ@\t-3.57\nsGPeCQHJPoIMuS@OI@\t-3.58\nsJPdE`DRHcyXAaIrD\t-3.58\nsNp`@Ti]jj@XFZodP\t-3.58\n`IjApGd\t-3.59\nj`q`@@@YIEBedhdnB]zh@J@@@@XJQ`eF\\tZsnHex\t-3.61\nKAxLbPfDwAxUxEw]ddfLZi`CqLh\t-3.61\nsNqHHGAJkjj`FAqkxh`\t-3.61\nsNt@@\\dbLjj@XEZod`\t-3.61\nsNpP@jtfvZf@^YY`\t-3.61\nqC`PCDRVhCCBKdp\t-3.62\nKAxI@PpNFRRJrjf@LLSByG@\t-3.64\nsGQhHlGCIIKUHCAF|tV@\t-3.64\nsNq@IddbmUPC@kE|e@\t-3.64\nsNqdHl@`nFRPjKU@LEWq@`\t-3.67\nsJPhH`xQ}T@pVOED\t-3.67\nqCbPPND^hCBKfDp\t-3.67\nsJQHHGAJVj@XI[bA@\t-3.7\nHkLLzJ[hBLdBdAdIdMdCdNxhbHRHRHfJiZj`AyB`\t-3.73\nJoB@Y@QGTCqG\\drnlktqw@AP@P@A`hIaUJz[|c`\t-3.73\nJoBBPDHCaddabbRRrv[uP@@@@@XKS\\Lkiv\u007FDa@\t-3.73\nHknBPLaAd@b`cIIBhhd]MZ`T@@FCXwKi|b`\t-3.73\nJoB@P@ABTwLjnmf}T@@@@@FFaS\\Lkiv\u007FDa@\t-3.73\nHg|@`@|DjYmUyO[i@@@@@FBeS\\Lkiwq@p\t-3.73\nHgnD`Naj|LddUfbbeUUUT@LESBhYWSodL\t-3.73\nHcO@HBhPdDdEdMdLbfbRagCT@EP@OLci@\t-3.73\nHg|@`@BDfYeUyO[j@@@@@FBeS\\Lkiwq@`\t-3.73\nHglLPHNp\\JBDjefYjjjjT@XRfDwCJ{rJ@\t-3.73\nHk\\@`@SDifUWGUN`@@@@A`nES\\Lki|`@\t-3.73\nHmTLHHS@BJ\\FdNdLddJbdVjj`Aa`roHH\t-3.73\nHk\\@BLeJeWmZ{SBB@@@@XKBEWCJz_Hp\t-3.73\nHkNH`FboCIIEUEEJjjj`AaJXWCJz_Hh\t-3.73\nHo\\``HSBCpRjVYffjjjj@F@mprng_H\\\t-3.73\nHk^@@@RfYU\\]Tzjjjj@FEaBXUMprnf\t-3.73\nHef@XLBPRPZPzPFPfPR[fZjjh@^YAT\t-3.73\nHiDLPHQ`\\J\\LddUdjjh@XEa\\ODQ`\t-3.73\nHmTLHBk`dDdIdEdDf{[jffh@^I@`\t-3.73\nHkL@@LddTbdLjjjj@FDxS\\Lki|b`\t-3.73\nHeg@bHS@|@eJfYjjj`A`qTwCOH@\t-3.73\nKAxH`PpNENFRRJrjj@LBSB{bTp\t-3.73\nHeg@bHS@B@eJfYjjj`AapES\\L`\t-3.73\nsGPH@xIUMU`LLl^R`\t-3.73\nHcl@@DjYU_egX@@@@@XBaMprn|SJ@\t-3.75\nHmvD@LADf]eYUj`@@A`qMproHp\t-3.77\nHklL`HS@BLddJbRvWUjB@`@XMS]JgrA@\t-3.78\nsJPhH`xYICT`LEcsBH\t-3.78\nsJPXhlPD[ct^m@H\t-3.8\nHeVB@D@aeIfUzYjjV@FBXwHx\t-3.81\nHaFL@D@nRYgifjf@FBXwHH\t-3.81\nsGQLKIARFdLddmU@LI^P`\t-3.81\nsJPHHDQ}T@prqyH\t-3.81\nQMBcpIRZAaWDP\t-3.81\nsJQHHGBOj`FFVOI@\t-3.81\nHmvLbEaL@HslddlRVFUh@H@FATwCrE@\t-3.83\nHcLL`HS@BLddJfRtjmP@P@LJ[ag^QP\t-3.83\nHiDDpHhG@gAgCIHbeJjj@FC@fMp|``\t-3.83\nHidD`HPGCIHXdnFh@@@XTUMp|pHP\t-3.83\nHef@@@Ri^Zjjh@XCBES\\L|c@\t-3.83\nHiF@@@ReYjjj@F@pfES\\OH@\t-3.83\nqCaHKBH`PdthFDGI@\t-3.83\nQMBCDRZA`erP\t-3.85\nHeTD@@QIUeQej@@@FCYS\\L|b@\t-3.85\nqCbPPNBRu@XI[d@\t-3.86\nQMIDbHHRYAaGJ@\t-3.87\nHo|H@IrRQRiQFPUj`BJh@FDiaTtg[|aP\t-3.88\nHedHHJbPRPrPYprRRiRUUU@CCNF^PP\t-3.88\nHmVHPF`GCgCIIETiJjZh@XLfDLkrJ@\t-3.88\nHiDLhJ[hBDdLdBdA\\LddTgjeh@^PH\t-3.88\nHiDLHJ[hBDdLdBdLddTgjeh@^PH\t-3.88\nsNplJyHJPtaddeeTpCrL\t-3.88\nsNphHpxYIHkUR@prqyJ\t-3.88\nsOx`DFdrikUL@pVoH`\t-3.91\nqCbPPNBTu@XS\\PP\t-3.91\nHctDXAeICIBiAiAYCYCIIEHXYUUMP@pb^Qp\t-3.93\nqCpPP^BSU@XI[d@\t-3.93\nQMABHgAIhFBOJ@\t-3.94\nsNxaBDnR[YjhAaMqU`\t-3.96\nqC`PApRVhCAQ\\Pp\t-3.96\nsGXaBDnR[Vj@XS\\Qh\t-3.97\nsGQHHGAJijhAaecrT\t-3.97\nHaDH`HApRnU[j@@@FEXUMwDB@\t-3.98\nHkLHxMRPrPJPZPzPFPfPrRQRVQrjjjX@^PX\t-4.02\nHmTLhJ[hBDdLdBdA\\LddTgTjej`A`dkrA@\t-4.02\nKA|PtMpFdQBQFQAQEQ@MdddgjjPCA`aLJ`\t-4.02\nHmt@pHdDdLdLddjbReZj@@@FEMproHH\t-4.02\nHcl@@DjYU_egX@@@@@XBfES\\L|Q]@\t-4.02\nHmtL`HS@BLddlRVFUh@H@FETwBoHp\t-4.02\nHiDB`HSBBRRjUZjj@FATwCr@@\t-4.02\nsNxDHG@nFRQFUU@LKEj~HD\t-4.02\nsGP@DjZj`FAVMxa@\t-4.02\n`Jd`^P\t-4.02\nQMJH\\Df`XSr@\t-4.04\nHk|@@LdbbbRQKauS`@`@`@F@ySCJz_Hx\t-4.04\nsGPB@xRHTQhcIIKUPC@qX|e@\t-4.09\nJoB@P@NFRRJJEIKKYoU@@@@@A`mMprng[|SB@\t-4.11\nHcu@`HPOAJYYfjjjh@XSaTwCJ{q@P\t-4.11\nHiD@@DjfZjj`A`vES\\ODa@\t-4.11\nsGPdCQHJPR[jf@^JT\t-4.11\nqCbPPND^hCAK\\P`\t-4.12\nsGQ@HeTuT@prqyJ\t-4.12\nKAxH@PvRRJrjf@LLSByG@\t-4.14\nHet@@DjYUX^d@@@@A`rXULODzP\t-4.18\nHcLBHHQaApIpipUprRIZJJbmUUUP@pKNg^DATh\t-4.2\nsNpdHpxWprRQVjhAaecqU`\t-4.22\nsJPYhlQxKmqzOZ`D\t-4.22\nsGX`LDdsmT@pexlP\t-4.23\nRFPaMt\\H\t-4.25\nQMIH\\EpRZAxb\t-4.25\nHiFB@LANEImfZjX@XYaCqDX\t-4.26\nHmV@xNRPrPJPjPZPfPVPrRQR]Rjjj@Gb\\H\t-4.27\nHcu@XFVIAICICi@YBYCIIKEHkUSUH@|cP\t-4.27\nHgnD`Naj|LddUbRdUUUUT@LISBjUt{yG@\t-4.27\nHmU@XFVIAICICi@YBYCIIKEHjjZh@^Ph\t-4.27\nHedLpHi`\\B\\N\\LdbLVeUUP@pXDsagbDp\t-4.27\nHmU@pHTGBgA`cIIEKHjjjh@X\\fDwCOHT\t-4.27\nHef@XFbPRPrPzPFPfPrRRqRUTu@CrF@\t-4.27\nHcvD`Naj|LddUbteUUU@CCdpje]yG@\t-4.27\nHcw@`H[j|LddUbbeTuU@CCdpae]yE@\t-4.27\nHef@pHApipXHrRQRrUUU@CAdpnF^Pp\t-4.27\nHmTD`HZoCIIETiJjjh@XLfDLkrJ@\t-4.27\nHed@`@\\DjYfjjj@FDpfES\\L|`@\t-4.27\nKA|HP]SPDIHYHEHIUMURpAyG@\t-4.27\nsGPH@xIUMU@LCEcrT\t-4.27\nsJQ@IGuPCCKGd`\t-4.28\nHid@b@|@eJYWxZB@@A`rXS\\ODB`\t-4.31\nHiFB@LANEIevZjX@XI`|Pj@\t-4.32\nKA|D@XBMrSZsUL@XIg\\QB@\t-4.32\nqCbPHQBSS@XP\\f@\t-4.32\nHedL`HS@\\LddRRmUUP@phj[ayD`\t-4.33\nHk^@@@RfYU\\]Tzjjjj@FApfES\\Lki|`@\t-4.35\nqCaPQApcu@XS\\pb@\t-4.38\nsJPHHxQ}T@prqyH\t-4.38\nKAxI@P`NFRQIFjj@LBpj[bAp\t-4.4\nqCsCPKRUvyIf`NIO@\t-4.41\nqCsCPKRUvyIf`H\t-4.41\nHctDxAe@cI@iBiAiAYCYCIIEHhyUUKP@|QN@\t-4.42\nJoB@P@^BULrj~mf}PA@@@@FAaaTwRng[|`P\t-4.42\nsNpH@xISMUT@p\\VM_H@\t-4.42\nsNq`@jdssTpCqY`\t-4.42\nqC`XR@TAhBOTAqh`I@rAr@\t-4.43\nsJPB@xRHTQhbOj`FFbwHP\t-4.43\nsNqLKIARFdLddljf@^Q`\t-4.43\nQMJH\\HzAaOD`\t-4.44\nKAxPPUD`d`tataddaTjf`CqLH\t-4.45\nQMBCpchGfR\t-4.45\nQMJH\\Df`^H`\t-4.48\nHctLhJ[hBDdLdBdA\\LddTgReTmU@CCIW^PH\t-4.49\nHiDL`HS@\\DjeVjj`A`QTw\\`P\t-4.49\nqC`PApRfhCAQ\\Pp\t-4.5\nKAxH`P`NENFRRJrjj@LBpf{bIP\t-4.52\nHg|H`HApRkfUWd}nh@@@@@XJUMprng_DB@\t-4.55\nsNphH`xITkUT@pvMx`p\t-4.55\nsGQDBIARBS]TpCqR`\t-4.55\nRFDTAFCbdCd@\t-4.56\nsGQ`@jdsmR@|VH\t-4.57\nHedL`HS@\\DjUUjjj@FCES\\L|`@\t-4.58\nKAxY@Pf@xITnuUPA`QTw\\PL@\t-4.58\nKAxY@Pf@xIUJuUPAaeMwHH\t-4.58\nsGQhHlGCIHcUPCAF|pV@\t-4.58\nsNpHHxIUMUT@pjqkyH\t-4.58\nQMJDbDfPXQr`\t-4.63\nQMBApRZAaWDP\t-4.64\nsNphH`xIRkUT@pNM_H@\t-4.66\nsJPhH`xISuPCAX|PP\t-4.66\nHglLhJ[hBDdLdBdA\\LddTgTbeTmUT@LBWSodR\t-4.67\nHo\u007F@XBhPdDdMdCdKdGdDfyV^[aj@Bjh@GfQJ`\t-4.67\nh`~@@@YIHiEHYCJjjjj`@pu`cNRUhug^y@P\t-4.67\nHctDpHPGBgAgCIIDcHiUUUP@p{BxYW^Pp\t-4.67\nHmVBhNiiV@aICI@i@PcIIEIuJiZT@^Ip`\t-4.67\nHmW@`HS@|DjVYfjjh@XMS\\Lkq@H\t-4.67\nsOpH@xILkW@@@LJlZ~P@\t-4.67\nQMABHgAIhFD\\Y@\t-4.69\nsGPhH`xITmU@LD[qC@\t-4.7\nqC`PCpcu@XP\\tV@\t-4.72\nQMbH\\HzAaOD`\t-4.75\nsGPH@xITuU@LBl[qB@\t-4.75\nsGPBIHJPtaYCIIKUPCrD\t-4.76\nQMAJ@XBN`\\L`JAr@\t-4.77\nRFBDQNDXLHyP\t-4.77\nsNqhHlGAJeZj`FFM_H@\t-4.79\nsJQ`@bdjt`OEL\t-4.79\nj`aA@@NYIHiXmDiIZj``bj@@XRQ`cNZl{wHF@\t-4.83\nHed@DHADdLdBdFkNdAdIdLddTleUUP@panQ@\t-4.83\nHkLLpHS@\\J\\F\\LddRLbdjjjj@FGEprngrL@\t-4.83\nHkLDpHXGBg@WCIIEXdiJjjZ`A`JXWRngrJ@\t-4.83\nHctLpH[`\\J|A\\LddUbteUUU@CCdpje]yG@\t-4.83\nHctLpH[`\\J\\A\\LddUbteUUU@CCdpje]yG@\t-4.83\nHglLpHS@\\J\\F\\Djevfjjfjh@XBWCJz]|c@\t-4.83\nHctLpH[`\\J|A\\LddUfTeUMU@CAdpiW^QP\t-4.83\nHkLBpHSBCpipYpRjwiijjjh@XMprngrB@\t-4.83\nHctLpH[`\\JhA\\LddUbteUUU@CAdpiW^Qp\t-4.83\nHedHXFbPRPrPzPFPfPrRRqRUUU@CBFyA@\t-4.83\nHmTDpHXGBjCgCIIETiJjjh@XLfDLkrJ@\t-4.83\nHg\\@@LddTjTt`kU@DU@@peLJfWSodB\t-4.83\nHg^@@JrRQRiSRBmT@QT@C@TpfWSodB\t-4.83\nHctL`H[jlLddUbteUUU@CAdpiW^Qp\t-4.83\nHmTD`HZkCIIETiJjjh@XLfDLkrJ@\t-4.83\nKAxX`Pw@xTxYIHkJjh@pqLKfTw@\t-4.83\nqC`DBHc`hcQBSU@XXK\\d@\t-4.83\nsJPH@xITuPCCQ[bA@\t-4.85\nsJPhH`xIRuPCAX|`@\t-4.86\nHid@`@\\LdbaRxZ`@@AaQTwCs@S@\t-4.88\nHct@@LddTbaeUUU@C@\\InFUwbIP\t-4.88\nqCbPXRBRt`^JH\t-5.01\nsOpIAxBTfUk`@@FEVM_H@\t-5.03\nHaD@`@\\DimVz`@@AaVES]q@P\t-5.04\nRFEG`SFCqH\t-5.04\nsGX`LDdvmT@pfydZ\t-5.06\nHo]@`HPOAJYfYfjjjjh@XGaTwCJz]or@@\t-5.07\njhy@`@@HRUYfUWyHwgAZj@@@H@@XJpPRmFl{p^P`\t-5.08\nqC`PApcu@XZK\\`@\t-5.08\nJobD@@IKLrj}\\mf}T@@@P@A`lHprng[|`@\t-5.09\nQMCBHgAbDf`XH|h\t-5.09\nHctHxMRPrPJPZPzPFPfPrRQRVQuUUR@OHT\t-5.11\nHiFD@NalddUfjih@XDfEwHD\t-5.12\nJgB@@BULrjy\\Nk\\@D@Q@@FDyS\\Lkm|`p\t-5.16\nsJXHHGBOj`FFVOI@\t-5.21\nh`\u007F`D@PxJgaddbQbdTTjfjjj@CARLDdkQkN}rE@\t-5.24\nHkLLHHi`\\B\\N\\A\\LdbLRTTjjjj@FD`f\\Lki|``\t-5.24\nHcu@XJZIAICIBYAYCYCIIJdeIULuP@|Pn@\t-5.24\nHo^D`Naj|LddUbRdTjjjjh@XJfETkiv\u007FHx\t-5.24\nHk\\@bLhLyJeWmZ{SBB@@@@XKBEWCJz_Hp\t-5.24\nHmU@pHXGB`cgCIIETiJjjh@X\\fEProDa`\t-5.24\nHiD@P@\\J\\LddUdjjh@XLfDwCrH@\t-5.24\nKA~A@PfAxIUsMUPA`QTw\\PH@\t-5.27\nsNp`H|ddUji@XYX|e@\t-5.3\nHg|@`@\\DjYmUyO[j@@@@@FBeS\\Lkiwq@p\t-5.31\nQMBApchFCOH@\t-5.31\nsNpdHpxUprRQVjhAaecqU`\t-5.32\nRFDCaFCBnP\t-5.38\nRFTPxQ`|R@\t-5.39\n`IG@Cr@\t-5.39\nsJPYHlQDGh}V@P\t-5.4\nqCpPPND^hCAK\\P`\t-5.44\nKAxXPTwPDIHYHEHIUMURpAyG@\t-5.45\nKA|H@]CYIHkJjX@pqLKd\\\t-5.48\nsNqlHl@brCdLddlVj@^KD\t-5.49\nHctDpHXGBg@WCIIEXmIUUUP@pyLKiW^QP\t-5.55\nsKP@Di\\Zj@XTQX`\t-5.55\nHeT@pHdDdLdLddlRPsU@@@CCJZ^Ip`\t-5.6\nqCcPQXQD]TCr@\t-5.62\nQMBApchGfR\t-5.62\nHgl``HSBCpRjVYfZjjj`A`K\\LkiwrJ@\t-5.64\nHk\\@`@\\DifUWGUN`@@@@A`nES\\Lki|`@\t-5.7\nsNtHHaxISMUT@pNM_H@\t-5.73\nsJPLAHJPL`dwU@OEL\t-5.77\nHctLbH[jlJ{IIEXmIUUUP@pyLJiW^Qp\t-5.81\nHmTDbHZkBnrRQUJRjjj@FCIaCJ|b`\t-5.81\nHkNB@NamOIIEYeEJjZi`A`rXWB_HT\t-5.83\nHcvB@NaewIIEYdiUSTp@piLKmy@`\t-5.88\nsGQLKIARFdLddmT`OHP\t-5.94\nHo\\LhJ[hBDdLdBdA\\LddTgTTTjejjh@XMJz]orA@\t-5.96\nHmTDpHXGBoCgCIIETiJjjh@XLfDLkrJ@\t-6.03\nsOpHAxILkW@@@LJlZ~P@\t-6.07\nQMBApchFE\\Q@\t-6.07\nRFDCaFCqH\t-6.09\nsJQDBIARBS]R@|Tp\t-6.14\nsJPH@xQ}T@ptVyB\t-6.14\nJoB@Q@^@ReSLjokYoT@P@@@AanES]Jz]oq@D\t-6.22\nHmT``HSBCpRjyfZjj`A`uMproH`\t-6.22\nsJXA@IczhA`XlP\t-6.33\nHiD@p@\\H\\J\\LddUdjjh@XLfDwCrH@\t-6.38\nKAxXTTwPDIHYHEH]rUSUTl@P\t-6.38\nsNqmHl@brCdMvRRVKU@OHP\t-6.44\nqCcPQXND^hCpbK@\t-6.5\nHaD@b@|@eJ[Unh@@@XUaTw\\PD@\t-6.53\nqC`PApcu@XJKbF@\t-6.53\nsNpO@YRHTQh`qEbGDAh\u007Fjj@X\\QXu`\t-6.53\nsNpO@ZRHTQh`qEbGDAh\u007Fjj@X\\QXu`\t-6.53\nsNpOAYRHTQh`qEbGDBh\u007Fjj@X\\QXu`\t-6.53\nHiF@PHApiprRQVRjj`A`rXS\\OH`\t-6.53\nsNpO@\\RPTai@rEdGHAh\u007Fjj@XP@\t-6.53\nsNpO@JRPTai@rEdGHAh\u007Fjj@XP@\t-6.53\nHeg@`HS@|DjYfjjj@FCES\\L|`@\t-6.53\nsNpOAXRPTai@rEdGHBh\u007Fjj@P\t-6.53\nsNpOABRPTai@rEdGHCh\u007Fjj@P\t-6.53\nsNpyHlZqfCh\u007Fjj@X\\QXu`\t-6.53\nQMFIGBN`^YH\t-6.58\nsGPG@ZRHTQh`qEb@t_jhAaXl[r@\t-6.75\nsNpJIHJPtaYArFRRVUU@OHP\t-6.89\nHaD@`@|DimVz`@@AaVES]q@P\t-6.95\nHk\\@b@|@eJYeUquSh@@@@@XKaTwCJz_H@\t-7.06\nHg\\@XHdDdLdMdCdKdLddlRTt`kU@DU@@prfWdB\t-7.11\nsNpbJxDJPt`YAJijiPGd`\t-7.19\nsGPBAHJPLaYAInjhAxi`\t-7.19\nHefD@NalddUfUUMP@pILKnHL`\t-7.2\nsGPlH`DFPladdLVhAxa`\t-7.25\nsJPKAPRPTai@r@zOj`D\t-7.29\nqC`FBPe@iCRDyIj`OI@\t-7.34\nHctLpH[`\\J\\A\\LddUbteUUU@CAdpiW^IW@\t-7.38\nHg|@`@|DjYmUyO[j@@@@@FF`UMprng_Hp\t-7.38\nsNpOAfRHTQh`qEbGDCh\u007Fjj@X\\QXu`\t-7.53\nsNpOAYRHTQh`qEbGDAh\u007Fjj@X\\QXu`\t-7.53\nsNpOAZRHTQh`qEbGDBh\u007Fjj@X\\QXu`\t-7.53\nsNpO@zRHTQh`qEbGDAh\u007Fjj@X\\QXu`\t-7.53\nsNpO@tRPTai@rEdGHAh\u007Fjj@XP@\t-7.53\nHed@xDdLdBdJdFdAdIdLddTgeUUP@|c`\t-7.62\nHk\\@`@|DifUWGUN`@@@@A`nES\\Lki|`@\t-7.7\nHmTDpHXGBgCgCIIETiJjjh@XLfDLkrJ@\t-7.73\nHo]@HHTGBgA`cWCIIEKHjiJjjjj@FBiaMpr]orE@\t-7.79\nHgl@dDdLdBdFdNdIdEdMdCdLddTedLeUUUT@OHl\t-7.79\nHg|@b@|@eJf[U^Svz`@@@@A`iTwCJz]|PL@\t-7.99\nHkL@dDdLdBdFdNdAdIdMdCdLddTed\\jjjj@GdJ\t-8.08\nsNpOAfRPTai@rEdGHCh\u007Fjj@XP@\t-8.11\nh`~CDHPYreaSlddUbRdTeUUUUP@XJQ`eEZMYw`\t-8.37\nsNpBIHJPl`yCIICJj`Gbq@\t-8.5\nsNpO@yRHTQh`qEbGDAh\u007Fjj@X\\QXu`\t-8.52\nsNpO@xRPTai@rEdGHAh\u007Fjj@P\t-8.84\nHed@XDdLdJdFdAdIdLddTaeUUP@|Pn@\t-9.1\nHkL@H@\\H\\J\\F\\LddTlbdjjjj@FDia\\Lki|b@\t-9.37\nHkL@DLdBdJdFdAdIdMdCdLddTbd\\jjjj@Gb\\h\t-9.57\nHiD@XHdDdJdFdNdAdLddLTjjh@^Ia@\t-9.59\nHct@DDdLdJdFdAdIdEdMdLddTaaeUUU@CqBx\t-10.1\nsNpOAxRPTai@rEdGHCh\u007Fjj@P\t-10.77";

	public static final double cDruglikenessUnknown = -999;

    private static boolean			sInitialized = false;
	private static IncrementTable	sIncrementTable;

	private ParameterizedStringList	mDetail;

	public DruglikenessPredictor() {
		synchronized(DruglikenessPredictor.class) {
			if (!sInitialized) {
		        try {
					sIncrementTable = new IncrementTable(druglikenessNoIndex);
					sInitialized = true;
					}
				catch (Exception e) {
		            System.out.println("Unable to initialize DruglikenessPredictor");
					}
				}
			}
		}


	public double assessDruglikeness(StereoMolecule testMolecule, ThreadMaster threadMaster) {
		ParameterizedStringList detail = new ParameterizedStringList();

		if (!sInitialized) {
			detail.add("Druglikeness predictor not properly initialized.",
								ParameterizedStringList.cStringTypeText);
			return cDruglikenessUnknown;
			}

		detail.add("Found sub-structure fragments and their contributions:",
							ParameterizedStringList.cStringTypeText);
		detail.add("(yellow atoms carry at least one more substituent)",
							ParameterizedStringList.cStringTypeText);
		double nastyIncrementSum = 0.0;
		double incrementSum = 0.0;
		int fragmentCount = 0;
		SSSearcher sss = new SSSearcher(SSSearcher.cMatchAtomCharge);
		StereoMolecule fragment = new StereoMolecule();
		for (int i=0; i<sIncrementTable.getSize(); i++) {
			if (threadMaster != null && threadMaster.threadMustDie())
				return cDruglikenessUnknown;

			Thread.yield();

			new IDCodeParser(false).parse(fragment, sIncrementTable.getFragment(i));
			sss.setMol(fragment, testMolecule);
			if (sss.isFragmentInMolecule()) {
				double increment = sIncrementTable.getIncrement(i);
				if (increment < -1)
					nastyIncrementSum += increment;
				else {
					incrementSum += increment;
					fragmentCount++;
					}

				detail.add(sIncrementTable.getFragment(i),
								   ParameterizedStringList.cStringTypeIDCode);
				detail.add(Double.toString(increment),
								   ParameterizedStringList.cStringTypeDouble);
				}
			}

		if (fragmentCount == 0)
			return -1;

		double druglikeness = nastyIncrementSum + incrementSum / Math.sqrt(fragmentCount);

			// correct cut-off by also treating molecules
			// with more than 50 found fragments as more drug-like
		druglikeness = druglikeness + 0.0625 * (fragmentCount - 40);

		mDetail = detail;
		return druglikeness;
		}

	public String getDruglikenessString(StereoMolecule testMolecule) {
		if (!sInitialized)
			return "Druglikeness predictor not properly initialized.";

		double incrementSum = 0.0;
		int fragmentCount = 0;
		SSSearcher sss = new SSSearcher(SSSearcher.cMatchAtomCharge);
		StereoMolecule fragment = new StereoMolecule();
		for (int i=0; i<sIncrementTable.getSize(); i++) {
			new IDCodeParser(false).parse(fragment, sIncrementTable.getFragment(i));
			sss.setMol(fragment, testMolecule);
			if (sss.isFragmentInMolecule()) {
				incrementSum += sIncrementTable.getIncrement(i);
				fragmentCount++;
				}
			}

		double druglikeness = (fragmentCount == 0) ? -1 : incrementSum / Math.sqrt(fragmentCount);
		return druglikeness + "\t" + fragmentCount + "\t" + testMolecule.getAtoms();
		}

	/**
	 * If assessDruglikeness() was called multiple times in multiple threads, then
	 * getDetail() won't retrieve the expected detail.
	 * @return
	 */
	public ParameterizedStringList getDetail() {
		return mDetail;
		}
	}
